-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_AutoScale"

-- **************************************************
-- General information about this script
-- **************************************************

LM_AutoScale = {}

function LM_AutoScale:Name()
	return "AutoScale"
end

function LM_AutoScale:Version()
	return "5.0"
end

function LM_AutoScale:Description()
	return "Automatically scales a 3D layer to fit the Moho workspace."
end

function LM_AutoScale:Creator()
	return "Lost Marble"
end

function LM_AutoScale:UILabel()
	return("Auto-Scale")
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_AutoScale:IsEnabled(moho)
	if (moho.layer:LayerType() ~= MOHO.LT_3D) then
		return false
	end
	return true
end

function LM_AutoScale:Run(moho)
	local mesh = moho:Mesh3D()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(nil)
	moho.document:SetDirty()

	local huge = 1e6
	local min = LM.Vector3:new_local()
	local max = LM.Vector3:new_local()

	min:Set(huge, huge, huge)
	max:Set(-huge, -huge, -huge)

	for i = 0, mesh:CountPoints() - 1 do
		local v = mesh:Point(i)

		min.x = math.min(min.x, v.x)
		min.y = math.min(min.y, v.y)
		min.z = math.min(min.z, v.z)

		max.x = math.max(max.x, v.x)
		max.y = math.max(max.y, v.y)
		max.z = math.max(max.z, v.z)
	end

	local scale = max.x - min.x
	scale = math.max(scale, max.y - min.y)
	scale = math.max(scale, max.z - min.z)
	if (scale > 0.0) then
		local scaleVec = LM.Vector3:new_local()
		scale = 1 / scale
		scaleVec:Set(scale, scale, scale)
		moho.layer.fScale:SetValue(moho.frame, scaleVec)
	else
		scale = 1
	end

	local center = (min + max) / -2
	moho.layer.fTranslation:SetValue(moho.frame, center * scale)
end
