-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Cube"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Cube = {}

function LM_Cube:Name()
	return "Cube"
end

function LM_Cube:Version()
	return "5.0"
end

function LM_Cube:Description()
	return "Creates a 3D cube in a new layer."
end

function LM_Cube:Creator()
	return "Lost Marble"
end

function LM_Cube:UILabel()
	return("Cube")
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_Cube:IsEnabled(moho)
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	return true
end

function LM_Cube:Run(moho)
	local layer = moho:CreateNewLayer(MOHO.LT_3D)
	local mesh = moho:Mesh3D()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(nil)
	moho.document:SetDirty()

	local startPt = mesh:CountPoints()
	local vec = LM.Vector3:new_local()

	-- top points
	vec:Set(-0.5, 0.5, 0.5)
	mesh:AddPoint(vec)
	vec:Set(0.5, 0.5, 0.5)
	mesh:AddPoint(vec)
	vec:Set(0.5, 0.5, -0.5)
	mesh:AddPoint(vec)
	vec:Set(-0.5, 0.5, -0.5)
	mesh:AddPoint(vec)

	-- bottom points
	vec:Set(-0.5, -0.5, -0.5)
	mesh:AddPoint(vec)
	vec:Set(0.5, -0.5, -0.5)
	mesh:AddPoint(vec)
	vec:Set(0.5, -0.5, 0.5)
	mesh:AddPoint(vec)
	vec:Set(-0.5, -0.5, 0.5)
	mesh:AddPoint(vec)

	local mat = mesh:CreateNewMaterial()
	MOHO.MultiplyColor(mat.color, 1.25)
	mesh:AddFace(startPt + 0, startPt + 1, startPt + 2, startPt + 3) -- top
	mat = mesh:CreateNewMaterial()
	MOHO.MultiplyColor(mat.color, 0.5)
	mesh:AddFace(startPt + 4, startPt + 5, startPt + 6, startPt + 7) -- bottom
	mat = mesh:CreateNewMaterial()
	mesh:AddFace(startPt + 0, startPt + 7, startPt + 6, startPt + 1) -- front
	mat = mesh:CreateNewMaterial()
	MOHO.MultiplyColor(mat.color, 0.65)
	mesh:AddFace(startPt + 2, startPt + 5, startPt + 4, startPt + 3) -- back
	mat = mesh:CreateNewMaterial()
	MOHO.MultiplyColor(mat.color, 0.75)
	mesh:AddFace(startPt + 3, startPt + 4, startPt + 7, startPt + 0) -- left
	mat = mesh:CreateNewMaterial()
	MOHO.MultiplyColor(mat.color, 0.75)
	mesh:AddFace(startPt + 1, startPt + 6, startPt + 5, startPt + 2) -- right
end
