-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Torus"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Torus = {}

function LM_Torus:Name()
	return "Torus"
end

function LM_Torus:Version()
	return "5.0"
end

function LM_Torus:Description()
	return "Creates a 3D torus in a new layer."
end

function LM_Torus:Creator()
	return "Lost Marble"
end

function LM_Torus:UILabel()
	return("Torus")
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_Torus:IsEnabled(moho)
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	return true
end

function LM_Torus:Run(moho)
	local layer = moho:CreateNewLayer(MOHO.LT_3D)
	local mesh = moho:Mesh3D()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(nil)
	moho.document:SetDirty()

	local inc = 24

	local startPt = mesh:CountPoints()
	local vec = LM.Vector3:new_local()
	local incI = inc
	local angleI = math.pi * 2 / incI
	local incJ = inc / 2
	local angleJ = math.pi * 2 / incJ

	for i = 0, incI - 1 do
		for j = 0, incJ - 1 do
			vec:Set(0, 0.25, 0)
			vec:Rotate(LM.Z_AXIS, -j * angleJ)
			vec.x = vec.x + 1
			vec:Rotate(LM.Y_AXIS, i * angleI)
			vec:Rotate(LM.X_AXIS, math.rad(20))
			mesh:AddPoint(vec)
		end
	end

	for i = 0, incI - 1 do
		local nextI = i + 1
		if (nextI == incI) then
			nextI = 0
		end
		for j = 0, incJ - 1 do
			local nextJ = j + 1
			if (nextJ == incJ) then
				nextJ = 0
			end
			mesh:AddFace(startPt + j + i * incJ,
				startPt + nextJ + i * incJ,
				startPt + nextJ + nextI * incJ,
				startPt + j + nextI * incJ)
		end
	end

	mesh:ScaleToRadius(0.5)

	layer.fLayerShadow:SetValue(0, true)

	layer.fShadowOffset:SetValue(0, 0)
	layer.fShadowBlur:SetValue(0, 0)
	layer.fShadowExpansion:SetValue(0, moho:PixelToDoc(1))
	layer.fShadowColor:SetValue(0, mesh:EdgeColor())
end
