-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_OrbitCamera"

-- **************************************************
-- General information about this script
-- **************************************************

LM_OrbitCamera = {}

function LM_OrbitCamera:Name()
	return "Orbit Camera"
end

function LM_OrbitCamera:Version()
	return "5.0"
end

function LM_OrbitCamera:Description()
	return "Moves the camera in a path to orbit the workspace."
end

function LM_OrbitCamera:Creator()
	return "Lost Marble"
end

function LM_OrbitCamera:UILabel()
	return("Orbit Camera...")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_OrbitCamera.duration = 72
LM_OrbitCamera.radius = 1.7321
LM_OrbitCamera.clockwise = true

-- **************************************************
-- Orbit Camera dialog
-- **************************************************

local LM_OrbitCameraDialog = {}

function LM_OrbitCameraDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Orbit Camera", LM_OrbitCameraDialog)
	local l = d:GetLayout()

	d.moho = moho

	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Duration (frames)"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Radius"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.duration = LM.GUI.TextControl(0, "0.0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.duration)
			d.radius = LM.GUI.TextControl(0, "0.0000", 0, LM.GUI.FIELD_UFLOAT)
			l:AddChild(d.radius)
		l:Pop()
	l:Pop()

	d.clockwise = LM.GUI.CheckBox("Clockwise")
	l:AddChild(d.clockwise, LM.GUI.ALIGN_LEFT)

	return d
end

function LM_OrbitCameraDialog:UpdateWidgets()
	self.duration:SetValue(LM_OrbitCamera.duration)
	self.radius:SetValue(LM_OrbitCamera.radius)
	self.clockwise:SetValue(LM_OrbitCamera.clockwise)
end

function LM_OrbitCameraDialog:OnValidate()
	local b = true
	if (not self:Validate(self.duration, 4, 1000000)) then
		b = false
	end
	if (not self:Validate(self.radius, 0.01, 100)) then
		b = false
	end
	return b
end

function LM_OrbitCameraDialog:OnOK()
	LM_OrbitCamera.duration = self.duration:IntValue()
	LM_OrbitCamera.radius = self.radius:FloatValue()
	LM_OrbitCamera.clockwise = self.clockwise:Value()
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_OrbitCamera:Run(moho)
	local dlog = LM_OrbitCameraDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	moho.document:SetDirty()
	moho.document:PrepUndo(moho.layer)

	local inc = LM.Round(self.duration / 24)
	if (inc < 1) then
		inc = 1
	end

	local v = LM.Vector3:new_local()
	local panTilt = LM.Vector2:new_local()

	for frame = 0, self.duration, inc do
		local angle = frame / self.duration
		angle = LM.Lerp(angle, 0, math.pi * 2)
		if (not self.clockwise) then
			angle = -angle
		end

		v:Set(0, 0, self.radius)
		v:Rotate(LM.Y_AXIS, angle)
		moho.document.fCameraTrack:SetValue(moho.frame + frame, v)

		panTilt:Set(0, angle)
		moho.document.fCameraPanTilt:SetValue(moho.frame + frame, panTilt)
	end
end
