-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_SplitCurve"

-- **************************************************
-- General information about this script
-- **************************************************

LM_SplitCurve = {}

function LM_SplitCurve:Name()
	return "Split Curve"
end

function LM_SplitCurve:Version()
	return "5.0"
end

function LM_SplitCurve:Description()
	return "Splits up selected curve segments by adding extra points."
end

function LM_SplitCurve:Creator()
	return "Lost Marble"
end

function LM_SplitCurve:UILabel()
	return("Split Curve...")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_SplitCurve.splitCount = 2

-- **************************************************
-- Split Curve dialog
-- **************************************************

local LM_SplitCurveDialog = {}

function LM_SplitCurveDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Split Curve", LM_SplitCurveDialog)
	local l = d:GetLayout()

	d.moho = moho

	l:AddChild(LM.GUI.StaticText("Insert points into selected curve segments:"), LM.GUI.ALIGN_CENTER)
	l:PushH(LM.GUI.ALIGN_CENTER)
		l:AddChild(LM.GUI.StaticText("Point count"))
		d.splitCount = LM.GUI.TextControl(0, "0000", 0, LM.GUI.FIELD_UINT)
		l:AddChild(d.splitCount)
	l:Pop()

	return d
end

function LM_SplitCurveDialog:UpdateWidgets()
	self.splitCount:SetValue(LM_SplitCurve.splitCount)
end

function LM_SplitCurveDialog:OnValidate()
	local b = true
	if (not self:Validate(self.splitCount, 1, 10)) then
		b = false
	end
	return b
end

function LM_SplitCurveDialog:OnOK()
	LM_SplitCurve.splitCount = self.splitCount:IntValue()
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_SplitCurve:IsEnabled(moho)
	if (moho.layer:LayerType() ~= MOHO.LT_VECTOR) then
		return false
	end
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	return true
end

function LM_SplitCurve:Run(moho)
	local dlog = LM_SplitCurveDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	for curveID = 0, mesh:CountCurves() - 1 do
		local curve = mesh:Curve(curveID)
		local ptCount = curve:CountSegments() * self.splitCount
		local pts = {}

		for segID = curve:CountSegments() - 1, 0, -1 do
			if (curve:IsSegmentSelected(segID)) then
				for i = self.splitCount, 1, -1 do
					local v = curve:PointOnSegment(segID, i / (self.splitCount + 1))
					table.insert(pts, v)
				end
			end
		end

		local ptID = 1

		for segID = curve:CountSegments() - 1, 0, -1 do
			if (curve:IsSegmentSelected(segID)) then
				for i = 0, self.splitCount - 1 do
					mesh:AddPoint(pts[ptID], curveID, segID, moho.frame)
					ptID = ptID + 1
				end
			end
		end
	end
end
