-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Triangle"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Triangle = {}

function LM_Triangle:Name()
	return "Triangle"
end

function LM_Triangle:Version()
	return "5.0"
end

function LM_Triangle:Description()
	return "Creates a new triangle shape in the current layer."
end

function LM_Triangle:Creator()
	return "Lost Marble"
end

function LM_Triangle:UILabel()
	return("Triangle")
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_Triangle:IsEnabled(moho)
	if (moho.layer:LayerType() ~= MOHO.LT_VECTOR) then
		return false
	end
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	return true
end

function LM_Triangle:Run(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	local n = mesh:CountPoints()
	local v = LM.Vector2:new_local()
	local r = 0.75

	mesh:SelectNone()

	v.x = r * math.cos(math.pi / 2)
	v.y = r * math.sin(math.pi / 2)
	mesh:AddLonePoint(v, 0)

	v.x = r * math.cos(math.pi + math.pi / 6)
	v.y = r * math.sin(math.pi + math.pi / 6)
	mesh:AppendPoint(v, 0)

	v.x = r * math.cos(2 * math.pi - math.pi / 6)
	v.y = r * math.sin(2 * math.pi - math.pi / 6)
	mesh:AppendPoint(v, 0)

	v.x = r * math.cos(math.pi / 2)
	v.y = r * math.sin(math.pi / 2)
	mesh:AppendPoint(v, 0)
	mesh:WeldPoints(n, n + 3, 0)

	mesh:Point(n):SetCurvature(MOHO.PEAKED, 0)
	mesh:Point(n + 1):SetCurvature(MOHO.PEAKED, 0)
	mesh:Point(n + 2):SetCurvature(MOHO.PEAKED, 0)

	mesh:SelectConnected()
	moho:CreateShape(true)
end
