-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_LayerShadow"

-- **************************************************
-- General information about this script
-- **************************************************

LM_LayerShadow = {}

function LM_LayerShadow:Name()
	return "Layer Shadow"
end

function LM_LayerShadow:Version()
	return "5.0"
end

function LM_LayerShadow:Description()
	return "Creates a perspective shadow effect for the current layer."
end

function LM_LayerShadow:Creator()
	return "Lost Marble"
end

function LM_LayerShadow:UILabel()
	return("Perspective Shadow...")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_LayerShadow.opacity = 50
LM_LayerShadow.blur = 8

-- **************************************************
-- Layer Shadow dialog
-- **************************************************

local LM_LayerShadowDialog = {}

function LM_LayerShadowDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Perspective Shadow", LM_LayerShadowDialog)
	local l = d:GetLayout()

	d.moho = moho

	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Opacity (%)"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Blur radius"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.opacity = LM.GUI.TextControl(0, "0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.opacity)
			d.blur = LM.GUI.TextControl(0, "0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.blur)
		l:Pop()
	l:Pop()

	return d
end

function LM_LayerShadowDialog:UpdateWidgets()
	self.opacity:SetValue(LM_LayerShadow.opacity)
	self.blur:SetValue(LM_LayerShadow.blur)
end

function LM_LayerShadowDialog:OnValidate()
	local b = true
	if (not self:Validate(self.opacity, 0, 100)) then
		b = false
	end
	if (not self:Validate(self.blur, 0, 256)) then
		b = false
	end
	return b
end

function LM_LayerShadowDialog:OnOK()
	LM_LayerShadow.opacity = self.opacity:IntValue()
	LM_LayerShadow.blur = self.blur:IntValue()
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_LayerShadow:Run(moho)
	local dlog = LM_LayerShadowDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	moho.document:SetDirty()
	moho.document:PrepUndo(nil)

	local origLayer = moho.layer

	local vec = LM.Vector3:new_local()

	local shadowMask = moho:DuplicateLayer(moho.layer)
	shadowMask:SetName("Shadow Mask")
	vec:Set(1, 0, 0)
	shadowMask.fShear:SetValue(moho.frame, vec)
	vec = shadowMask.fScale:GetValue(moho.frame)
	vec.y = vec.y * 0.25
	shadowMask.fScale:SetValue(moho.frame, vec)

	local shadowGroup = moho:LayerAsGroup(moho:CreateNewLayer(MOHO.LT_GROUP))
	shadowGroup:SetName("Shadow")

	moho:PlaceLayerInGroup(shadowMask, shadowGroup);

	local shadowLayer = moho:CreateNewLayer(MOHO.LT_VECTOR)
	shadowLayer:SetName("Shadow")
	moho:PlaceLayerInGroup(shadowLayer, shadowGroup);

	local mesh = moho:Mesh()
	if (mesh ~= nil) then
		mesh:SelectNone()

		local v = LM.Vector2:new_local()

		v:Set(-2, 1.1)
		mesh:AddLonePoint(v, 0)
		v:Set(2, 1.1)
		mesh:AppendPoint(v, 0)
		v:Set(2, -1.1)
		mesh:AppendPoint(v, 0)
		v:Set(-2, -1.1)
		mesh:AppendPoint(v, 0)
		v:Set(-2, 1.1)
		mesh:AppendPoint(v, 0)
		mesh:WeldPoints(0, 4, 0)

		mesh:Point(0):SetCurvature(MOHO.PEAKED, 0)
		mesh:Point(1):SetCurvature(MOHO.PEAKED, 0)
		mesh:Point(2):SetCurvature(MOHO.PEAKED, 0)
		mesh:Point(3):SetCurvature(MOHO.PEAKED, 0)

		mesh:SelectConnected()
		local id = moho:CreateShape(true)
		if (id >= 0) then
			local shape = mesh:Shape(id)
			shape.fHasOutline = false
			shape:RemoveStyles()
			shape.fMyStyle.fFillCol.value.r = 0
			shape.fMyStyle.fFillCol.value.g = 0
			shape.fMyStyle.fFillCol.value.b = 0
			shape.fMyStyle.fFillCol.value.a = 255
			shape.fMyStyle.fFillCol:StoreValue()
		end
	end
	shadowGroup.fAlpha:SetValue(0, self.opacity / 100)
	shadowGroup.fBlur:SetValue(0, moho:PixelToDoc(self.blur))

	shadowGroup:SetGroupMask(MOHO.GROUP_MASK_HIDE_ALL)
	shadowMask:SetMaskingMode(MOHO.MM_ADD_MASK_INVIS)
	moho:SetSelLayer(shadowGroup)
	shadowGroup:Expand(false)

	moho:PlaceLayerBehindAnother(shadowGroup, origLayer)
end
