-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_LayerTrail"

-- **************************************************
-- General information about this script
-- **************************************************

LM_LayerTrail = {}

function LM_LayerTrail:Name()
	return "Layer Trail"
end

function LM_LayerTrail:Version()
	return "5.0"
end

function LM_LayerTrail:Description()
	return "Draws a trail behind a moving layer."
end

function LM_LayerTrail:Creator()
	return "Lost Marble"
end

function LM_LayerTrail:UILabel()
	return("Layer Trail...")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_LayerTrail.startFrame = 1
LM_LayerTrail.endFrame = 1
LM_LayerTrail.dashed = false

-- **************************************************
-- Orbit Camera dialog
-- **************************************************

local LM_LayerTrailDialog = {}

function LM_LayerTrailDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Layer Trail", LM_LayerTrailDialog)
	local l = d:GetLayout()

	d.moho = moho

	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Start frame"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("End frame"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.startFrame = LM.GUI.TextControl(0, "0.0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.startFrame)
			d.endFrame = LM.GUI.TextControl(0, "0.0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.endFrame)
		l:Pop()
	l:Pop()

	d.dashed = LM.GUI.CheckBox("Dashed line")
	l:AddChild(d.dashed, LM.GUI.ALIGN_LEFT)

	return d
end

function LM_LayerTrailDialog:UpdateWidgets()
	self.startFrame:SetValue(LM_LayerTrail.startFrame)
	self.endFrame:SetValue(LM_LayerTrail.endFrame)
	self.dashed:SetValue(LM_LayerTrail.dashed)
end

function LM_LayerTrailDialog:OnValidate()
	local b = true
	if (not self:Validate(self.startFrame, 0, 1000000)) then
		b = false
	end
	if (not self:Validate(self.endFrame, self.startFrame:IntValue() + 1, 1000000)) then
		b = false
	end
	return b
end

function LM_LayerTrailDialog:OnOK()
	LM_LayerTrail.startFrame = self.startFrame:IntValue()
	LM_LayerTrail.endFrame = self.endFrame:IntValue()
	LM_LayerTrail.dashed = self.dashed:Value()
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_LayerTrail:Run(moho)
	self.startFrame = 1
	self.endFrame = moho.document:EndFrame()

	local dlog = LM_LayerTrailDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	moho.document:PrepUndo(nil)
	moho.document:SetDirty()

	local parentLayer = moho.layer
	local layer = moho:CreateNewLayer(MOHO.LT_VECTOR)
	layer:SetName("Layer Trail")
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local startFrame = self.startFrame
	local endFrame = self.endFrame
	local curFrame = moho.frame
	local m = LM.Matrix:new_local()
	local origM = LM.Matrix:new_local()
	local vec = LM.Vector2:new_local()
	local origin = parentLayer:Origin()

	-- first, create a curve with a new point for each frame of movement
	local dashedOn = true
	for frame = startFrame, endFrame do
		moho:SetCurFrame(frame)
		parentLayer:GetFullTransform(frame, m, nil)
		vec:Set(origin)
		m:Transform(vec)
		if (frame == startFrame) then
			mesh:AddLonePoint(vec, 0)
		else
			mesh:AppendPoint(vec, 0)
			mesh:SelectNone()
			local id = mesh:CountPoints() - 1
			mesh:Point(id).fSelected = true
			mesh:Point(id - 1).fSelected = true
			if (self.dashed) then
				if (dashedOn) then
					moho:CreateShape(false, frame, false)
				end
			else
				moho:CreateShape(false, frame, false)
			end
			mesh:Point(0).fParent = -1
			mesh:Point(mesh:CountPoints() - 1).fParent = -1
			dashedOn = not dashedOn
		end
	end

	mesh:SelectNone()

	-- next, compensate these points for any movement higher up in the hierarchy
	-- examples: the target layer is part of a group with motion of its own, or is attached to a bone
	layer:GetFullTransform(0, origM, nil)
	for frame = startFrame, endFrame do
		layer:GetFullTransform(frame, m, nil)
		m:Invert()
		for i = 0, mesh:CountPoints() - 1 do
			vec:Set(mesh:Point(i).fAnimPos:GetValue(0))
			origM:Transform(vec)
			m:Transform(vec)
			mesh:Point(i).fAnimPos:SetValue(frame, vec)
		end
	end

	moho:SetCurFrame(curFrame)
end
