-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_AutoWeld"

-- **************************************************
-- General information about this script
-- **************************************************

LM_AutoWeld = {}

function LM_AutoWeld:Name()
	return "Auto Weld"
end

function LM_AutoWeld:Version()
	return "5.0"
end

function LM_AutoWeld:Description()
	return "Automatically welds the selected points."
end

function LM_AutoWeld:Creator()
	return "Lost Marble"
end

function LM_AutoWeld:UILabel()
	return("Auto Weld")
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_AutoWeld:IsEnabled(moho)
	if (moho:CountSelectedPoints() > 0) then
		return true
	else
		return false
	end
end

function LM_AutoWeld:Run(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	local m = LM.Matrix:new_local()
	local testVec1 = LM.Vector2:new_local()
	local testVec2 = LM.Vector2:new_local()
	local p1 = LM.Point:new_local()
	local p2 = LM.Point:new_local()
	local dist = 0

	moho.layer:GetFullTransform(moho.frame, m, moho.document)

	-- first, try welding any overlapping points
	local doAgain = true
	while doAgain do
		doAgain = false
		local id1 = 0
		while id1 < mesh:CountPoints() do
			local pt1 = mesh:Point(id1)
			if (pt1.fSelected) then
				local id2 = mesh:ClosestPoint(pt1.fPos, id1)
				local pt2 = mesh:Point(id2)
				if (id2 > id1 and (pt1:IsEndpoint() or pt2:IsEndpoint())) then
					testVec1:Set(pt1.fPos)
					testVec2:Set(pt2.fPos)
					m:Transform(testVec1)
					m:Transform(testVec2)
					moho.view:Graphics():WorldToScreen(testVec1, p1)
					moho.view:Graphics():WorldToScreen(testVec2, p2)
					p1.x = p1.x - p2.x
					p1.y = p1.y - p2.y
					dist = (p1.x * p1.x) + (p1.y * p1.y)
					if (dist < 100) then -- found one close enough to weld
						if (not mesh:ArePointsAdjacent(id1, id2)) then
							if (mesh:WeldPoints(id2, id1, moho.frame)) then
								-- It might be nice for some cases to automatically smooth,
								-- but as a general thing, this should be up to the user,
								-- and is easy for the user to do later.
								--pt1:SetCurvature(MOHO.SMOOTH, moho.frame)
								doAgain = true
							end
						end
					end
				end
			end
			id1 = id1 + 1
		end
	end

--[[
	-- next, try welding dead-end points to the middle of a nearby curve.
	local curveID = -1
	local segID = -1

	for i = 0, mesh:CountPoints() - 1 do
		local pt1 = mesh:Point(i)
		if (pt1.fSelected and pt1:IsEndpoint()) then
			local id2 = mesh:ClosestPoint(pt1.fPos, i)
			local pt2 = mesh:Point(id2)

			testVec1:Set(pt1.fPos)
			testVec2:Set(pt2.fPos)
			m:Transform(testVec1)
			m:Transform(testVec2)
			moho.view:Graphics():WorldToScreen(testVec1, p1)
			moho.view:Graphics():WorldToScreen(testVec2, p2)
			p2.x = p1.x - p2.x
			p2.y = p1.y - p2.y
			dist = (p1.x * p1.x) + (p1.y * p1.y)
			if (dist > 100) then -- the closest point will not interfere with an edge weld
				curveID, segID = pt1:GetEndpointEdge(curveID, segID)
				curveID, segID = moho.view:PickEdge(p1, curveID, segID)
				if (curveID >= 0) then -- add a point in the middle of some curve
					if (not mesh:Curve(curveID):IsPointOnSegment(i, segID)) then -- don't weld the point back on itself
						mesh:AddPoint(pt1.fPos, curveID, segID, moho.frame)
						mesh:WeldPoints(mesh:CountPoints() - 1, i, moho.frame)
					end
				end
			end
		end
	end
]]
end
