-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Rain"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Rain = {}

function LM_Rain:Name()
	return "Rain"
end

function LM_Rain:Version()
	return "5.0"
end

function LM_Rain:Description()
	return "Creates a simple rain shower."
end

function LM_Rain:Creator()
	return "Lost Marble"
end

function LM_Rain:UILabel()
	return("Rain")
end

-- **************************************************
-- The guts of this script
-- **************************************************

LM_Rain.color = LM.ColorVector:new_local()
LM_Rain.color:Set(0.55, 0.65, 0.77, 0.5)
LM_Rain.pCount = 500
LM_Rain.lifetime = 12

function LM_Rain:Run(moho)
	moho.document:PrepUndo(nil)
	moho.document:SetDirty()

	local v3 = LM.Vector3:new_local()
	local particles = moho:LayerAsParticle(moho:CreateNewLayer(MOHO.LT_PARTICLE))
	particles:SetName("Rain")

	particles:SetNumParticles(self.pCount, 20)
	particles:SetLifetime(self.lifetime)
	particles:SetOrientation(true)
	particles:SetFreeFloating(true)
	particles:SetEvenlySpaced(false)
	particles:SetRandomStartTime(false)
	particles:SetDirection(math.rad(260), math.rad(5))
	v3:Set(5, 0.1, 2)
	particles:SetSourceDimensions(v3)
	particles:SetVelocity(5, 2)
	particles:SetDamping(0)
	particles:SetAcceleration(0, 0)
	particles:SetFullSpeedStart(true)

	for i = 1, 6 do
		self:AddParticle(moho, particles)
	end

	particles:FinalizeSettings()
	particles:Expand(false)
	v3:Set(0.2, 1.1, 0)
	particles.fTranslation:SetValue(0, v3)
	moho:SetSelLayer(particles)
end

function LM_Rain:AddParticle(moho, particles)
	-- Create a new vector layer and add it to the particle group
	local layer = moho:CreateNewLayer(MOHO.LT_VECTOR)
	moho:PlaceLayerInGroup(layer, particles)
	layer:SetName("p" .. particles:CountLayers())

	-- Create a little droplet shape
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local v = LM.Vector2:new_local()
	v:Set(0, 0.06)
	mesh:AddLonePoint(v, 0)
	v:Set(0, -0.06)
	mesh:AppendPoint(v, 0)
	local pt = mesh:Point(0)
	pt.fWidth:SetValue(0, 3 / 240)

	-- Now add a shape and set its color and style properties
	mesh:SelectConnected()
	local id = moho:CreateShape(false, 0, false)
	if (id >= 0) then
		local shape = mesh:Shape(id)
		shape:RemoveStyles()

		local color = self.color * MOHO.RandomRange(0.9, 1.1)
		color.r = color.r * MOHO.RandomRange(0.95, 1.05)
		color.g = color.g * MOHO.RandomRange(0.95, 1.05)
		color.b = color.b * MOHO.RandomRange(0.95, 1.05)
		color.a = MOHO.RandomRange(0.4, 0.6)
		shape.fMyStyle.fLineCol:SetValue(0, color:AsColorStruct())
	end
end
