-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Smoke"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Smoke = {}

function LM_Smoke:Name()
	return "Smoke"
end

function LM_Smoke:Version()
	return "5.0"
end

function LM_Smoke:Description()
	return "Creates a smoke effect using particles."
end

function LM_Smoke:Creator()
	return "Lost Marble"
end

function LM_Smoke:UILabel()
	return("Smoke...")
end

-- **************************************************
-- Smoke dialog
-- **************************************************

local LM_SmokeDialog = {}

function LM_SmokeDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Smoke", LM_SmokeDialog)
	local l = d:GetLayout()

	d.moho = moho

	l:PushH()
		l:AddChild(LM.GUI.StaticText("Smoke color"))
		d.color = LM.GUI.ColorSwatch(false)
		l:AddChild(d.color)
	l:Pop()

	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Particle lifetime"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Flame lifetime"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.lifetime = LM.GUI.TextControl(0, "0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.lifetime, LM.GUI.ALIGN_LEFT)
			d.flameDuration = LM.GUI.TextControl(0, "0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.flameDuration, LM.GUI.ALIGN_LEFT)
		l:Pop()
	l:Pop()

	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Style:"), LM.GUI.ALIGN_LEFT)
			l:Indent()
				d.cloud = LM.GUI.RadioButton("Cloud")
				l:AddChild(d.cloud, LM.GUI.ALIGN_LEFT)
				d.plume = LM.GUI.RadioButton("Plume")
				l:AddChild(d.plume, LM.GUI.ALIGN_LEFT)
			l:Unindent()
		l:Pop()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Smoke distribution:"), LM.GUI.ALIGN_LEFT)
			l:Indent()
				d.narrow = LM.GUI.RadioButton("Narrow")
				l:AddChild(d.narrow, LM.GUI.ALIGN_LEFT)
				d.medium = LM.GUI.RadioButton("Medium")
				l:AddChild(d.medium, LM.GUI.ALIGN_LEFT)
				d.wide = LM.GUI.RadioButton("Wide")
				l:AddChild(d.wide, LM.GUI.ALIGN_LEFT)
			l:Unindent()
		l:Pop()
	l:Pop()

	d.extraShading = LM.GUI.CheckBox("Extra shading (slower to render)")
	l:AddChild(d.extraShading, LM.GUI.ALIGN_LEFT)

	return d
end

function LM_SmokeDialog:UpdateWidgets()
	self.color:SetValue(LM_Smoke.color:AsColorStruct())
	self.lifetime:SetValue(LM_Smoke.lifetime)
	self.flameDuration:SetValue(LM_Smoke.flameDuration)

	self.cloud:SetValue(LM_Smoke.style == 1)
	self.plume:SetValue(LM_Smoke.style == 2)

	self.narrow:SetValue(LM_Smoke.distribution == 1)
	self.medium:SetValue(LM_Smoke.distribution == 2)
	self.wide:SetValue(LM_Smoke.distribution == 3)

	self.extraShading:SetValue(LM_Smoke.extraShading)
end

function LM_SmokeDialog:OnValidate()
	local b = true
	if (not self:Validate(self.lifetime, 1, 10000)) then
		b = false
	end
	if (not self:Validate(self.flameDuration, 0, self.lifetime:Value() / 2)) then
		b = false
	end
	return b
end

function LM_SmokeDialog:OnOK()
	LM_Smoke.color:Set(self.color:Value())
	LM_Smoke.lifetime = self.lifetime:IntValue()
	LM_Smoke.flameDuration = self.flameDuration:IntValue()

	if (self.cloud:Value()) then
		LM_Smoke.style = 1
	elseif (self.plume:Value()) then
		LM_Smoke.style = 2
	end

	if (self.narrow:Value()) then
		LM_Smoke.distribution = 1
	elseif (self.medium:Value()) then
		LM_Smoke.distribution = 2
	elseif (self.wide:Value()) then
		LM_Smoke.distribution = 3
	end

	LM_Smoke.extraShading = self.extraShading:Value()
end

-- **************************************************
-- The guts of this script
-- **************************************************

LM_Smoke.color = LM.ColorVector:new_local()
LM_Smoke.color:Set(0.95, 0.95, 0.95)
LM_Smoke.pCount = 500
LM_Smoke.lifetime = 30
LM_Smoke.style = 2 -- 1:cloud 2:plume
LM_Smoke.distribution = 1 -- 1:narrow 2:medium 3:wide
LM_Smoke.flameDuration = 4
LM_Smoke.flameColor = LM.ColorVector:new_local()
LM_Smoke.flameColor:Set(0.3, 0.15, 0.08)
LM_Smoke.flameColor = LM_Smoke.flameColor * 3
LM_Smoke.extraShading = true
LM_Smoke.darkSmokeColor = LM.ColorVector:new_local()
LM_Smoke.darkSmokeColor:Set(0.1, 0.1, 0.1)

function LM_Smoke:Run(moho)
	local dlog = LM_SmokeDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	moho.document:PrepUndo(nil)
	moho.document:SetDirty()

	if (self.flameDuration > 0) then -- create two particle systems: smoke and flame
		local group = moho:LayerAsGroup(moho:CreateNewLayer(MOHO.LT_GROUP))
		group:SetName("Smoke")
		local smokeLayer = self:CreateSmoke(moho)
		moho:PlaceLayerInGroup(smokeLayer, group)
		moho:SetSelLayer(group)
		local flameLayer = self:CreateFlame(moho)
		moho:PlaceLayerInGroup(flameLayer, group)
		group:Expand(false)
		moho:SetSelLayer(group)
	else -- just the basic smoke
		local smokeLayer = self:CreateSmoke(moho)
		moho:SetSelLayer(smokeLayer)
	end
end

function LM_Smoke:CreateSmoke(moho)
	local v3 = LM.Vector3:new_local()
	local particles = moho:LayerAsParticle(moho:CreateNewLayer(MOHO.LT_PARTICLE))
	particles:SetName("Smoke")

	particles:SetNumParticles(self.pCount, 20)
	particles:SetLifetime(self.lifetime)
	particles:SetOrientation(false)
	particles:SetFreeFloating(true)
	particles:SetEvenlySpaced(false)
	particles:SetRandomStartTime(false)
	if (self.style == 1) then -- cloud
		particles:SetDirection(0, math.rad(180))
		if (self.distribution == 1) then -- narrow
			v3:Set(0.1, 0.1, 0.1)
			particles:SetSourceDimensions(v3)
			particles:SetVelocity(0.2, 0.2)
			particles:SetDamping(3)
		elseif (self.distribution == 2) then -- medium
			v3:Set(0.2, 0.2, 0.2)
			particles:SetSourceDimensions(v3)
			particles:SetVelocity(0.5, 0.5)
			particles:SetDamping(3)
		elseif (self.distribution == 3) then -- wide
			v3:Set(0.4, 0.4, 0.4)
			particles:SetSourceDimensions(v3)
			particles:SetVelocity(1, 0.5)
			particles:SetDamping(3)
		end
		particles:SetAcceleration(0, 0)
	elseif (self.style == 2) then -- plume
		if (self.distribution == 1) then -- narrow
			particles:SetDirection(math.rad(90), math.rad(10))
			v3:Set(0.1, 0.1, 0.1)
			particles:SetSourceDimensions(v3)
			particles:SetVelocity(2, 0.25)
			particles:SetDamping(2)
		elseif (self.distribution == 2) then -- medium
			particles:SetDirection(math.rad(90), math.rad(20))
			v3:Set(0.2, 0.2, 0.2)
			particles:SetSourceDimensions(v3)
			particles:SetVelocity(2, 0.5)
			particles:SetDamping(2)
		elseif (self.distribution == 3) then -- wide
			particles:SetDirection(math.rad(90), math.rad(30))
			v3:Set(0.4, 0.4, 0.4)
			particles:SetSourceDimensions(v3)
			particles:SetVelocity(3, 1)
			particles:SetDamping(2)
		end
		particles:SetAcceleration(0, 2)
	end
	particles:SetFullSpeedStart(true)

	for i = 1, 12 do
		self:AddSmokeParticle(moho, particles)
	end

	particles:FinalizeSettings()
	particles:Expand(false)
	return particles
end

function LM_Smoke:CreateFlame(moho)
	local v3 = LM.Vector3:new_local()
	local particles = moho:LayerAsParticle(moho:CreateNewLayer(MOHO.LT_PARTICLE))
	particles:SetName("Flame")

	particles:SetNumParticles(self.pCount / 5, 20)
	particles:SetLifetime(self.flameDuration)
	particles:SetOrientation(false)
	particles:SetFreeFloating(true)
	particles:SetEvenlySpaced(false)
	particles:SetRandomStartTime(false)
	if (self.style == 1) then -- cloud
		particles:SetDirection(0, math.rad(180))
		if (self.distribution == 1) then -- narrow
			v3:Set(0.1, 0.1, 0.1)
			particles:SetSourceDimensions(v3 * 0.75)
			particles:SetVelocity(0.2, 0.2)
			particles:SetDamping(3)
		elseif (self.distribution == 2) then -- medium
			v3:Set(0.2, 0.2, 0.2)
			particles:SetSourceDimensions(v3 * 0.75)
			particles:SetVelocity(0.5, 0.5)
			particles:SetDamping(3)
		elseif (self.distribution == 3) then -- wide
			v3:Set(0.4, 0.4, 0.4)
			particles:SetSourceDimensions(v3 * 0.75)
			particles:SetVelocity(1, 0.5)
			particles:SetDamping(3)
		end
		particles:SetAcceleration(0, 0.5)
	elseif (self.style == 2) then -- plume
		if (self.distribution == 1) then -- narrow
			particles:SetDirection(math.rad(90), math.rad(10))
			v3:Set(0.1, 0.1, 0.1)
			particles:SetSourceDimensions(v3 * 0.75)
			particles:SetVelocity(2, 0.25)
			particles:SetDamping(2)
		elseif (self.distribution == 2) then -- medium
			particles:SetDirection(math.rad(90), math.rad(20))
			v3:Set(0.2, 0.2, 0.2)
			particles:SetSourceDimensions(v3 * 0.75)
			particles:SetVelocity(2, 0.5)
			particles:SetDamping(2)
		elseif (self.distribution == 3) then -- wide
			particles:SetDirection(math.rad(90), math.rad(30))
			v3:Set(0.4, 0.4, 0.4)
			particles:SetSourceDimensions(v3 * 0.75)
			particles:SetVelocity(3, 1)
			particles:SetDamping(2)
		end
		particles:SetAcceleration(0, 2)
	end
	particles:SetFullSpeedStart(true)

	for i = 1, 6 do
		self:AddFlameParticle(moho, particles)
	end

	particles:FinalizeSettings()
	particles:Expand(false)
	return particles
end

function LM_Smoke:AddSmokeParticle(moho, particles)
	-- Create a new vector layer and add it to the particle group
	local layer = moho:CreateNewLayer(MOHO.LT_VECTOR)
	moho:PlaceLayerInGroup(layer, particles)
	layer:SetName("p" .. particles:CountLayers())

	-- Create a little blob shape
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local numPoints = 4
	local n = mesh:CountPoints()
	local v = LM.Vector2:new_local()
	local angle = math.pi / 2
	local dAngle = math.pi / numPoints
	local radius = MOHO.RandomRange(0.002, 0.1)
	local flameDuration = 0

	if (self.style == 2) then -- plume
		radius = radius * 1.5
	end

	if (self.flameDuration > 0) then
		flameDuration = self.flameDuration + 1
	end

	mesh:SelectNone()
	for i = 1, numPoints do
		local r = radius * MOHO.RandomRange(0.25, 1)
		v.x = r * math.cos(angle)
		v.y = r * math.sin(angle)
		if (i == 1) then
			mesh:AddLonePoint(v, 0)
		else
			mesh:AppendPoint(v, 0)
		end
		angle = angle + dAngle

		r = radius * MOHO.RandomRange(0.25, 1)
		v.x = r * math.cos(angle)
		v.y = r * math.sin(angle)
		mesh:AppendPoint(v, 0)
		angle = angle + dAngle
	end
	mesh:AppendPoint(v, 0)
	mesh:WeldPoints(n + numPoints * 2, n, 0)

	-- Now add a shape and set its color and style properties
	local smokeStartFrame = 0
	mesh:SelectConnected()
	local id = moho:CreateShape(true, 0, false)
	if (id >= 0) then
		local shape = mesh:Shape(id)
		shape.fHasOutline = false
		shape:RemoveStyles()
		shape.fMyStyle:SetSoftEdge(MOHO.RandomRange(6, 12) / 240)

		local shadingColor = self.color * MOHO.RandomRange(0.5, 1.5)
		shadingColor.r = shadingColor.r * MOHO.RandomRange(0.9, 1.1)
		shadingColor.g = shadingColor.g * MOHO.RandomRange(0.9, 1.1)
		shadingColor.b = shadingColor.b * MOHO.RandomRange(0.9, 1.1)
		shadingColor:Invert()
		shadingColor.a = 0.5
		if (self.extraShading) then
			shape.fMyStyle:SetShading(math.rad(135), 3 / 240, 3 / 240, shadingColor:AsColorStruct())
		end

		if (flameDuration > 0) then -- apply a flame effect to begin the particle's lifetime
			smokeStartFrame = flameDuration
			local color = self.darkSmokeColor * MOHO.RandomRange(0.9, 1.1)
			color.r = color.r * MOHO.RandomRange(0.95, 1.05)
			color.g = color.g * MOHO.RandomRange(0.95, 1.05)
			color.b = color.b * MOHO.RandomRange(0.95, 1.05)
			color.a = MOHO.RandomRange(0.7, 0.8)
			shape.fMyStyle.fFillCol:SetValue(0, color:AsColorStruct())
			shape.fMyStyle.fFillCol:SetValue(smokeStartFrame - 1, color:AsColorStruct())
		end

		local color = self.color * MOHO.RandomRange(0.5, 1.5)
		color.r = color.r * MOHO.RandomRange(0.95, 1.05)
		color.g = color.g * MOHO.RandomRange(0.95, 1.05)
		color.b = color.b * MOHO.RandomRange(0.95, 1.05)
		color.a = MOHO.RandomRange(0.1, 0.35)
		shape.fMyStyle.fFillCol:SetValue(smokeStartFrame, color:AsColorStruct())
		color = self.color * MOHO.RandomRange(0.5, 1.5)
		color.r = color.r * MOHO.RandomRange(0.95, 1.05)
		color.g = color.g * MOHO.RandomRange(0.95, 1.05)
		color.b = color.b * MOHO.RandomRange(0.95, 1.05)
		color.a = MOHO.RandomRange(0.1, 0.35)
		shape.fMyStyle.fFillCol:SetValue(self.lifetime / 2, color:AsColorStruct())
		color = self.color * MOHO.RandomRange(0.5, 1.5)
		color.r = color.r * MOHO.RandomRange(0.95, 1.05)
		color.g = color.g * MOHO.RandomRange(0.95, 1.05)
		color.b = color.b * MOHO.RandomRange(0.95, 1.05)
		color.a = 0
		shape.fMyStyle.fFillCol:SetValue(self.lifetime, color:AsColorStruct())
	end

	-- Scale this layer to make it appear to grow and then fade out
	local scaleVec = LM.Vector3:new_local()
	scaleVec:Set(0.25, 0.25, 0.25)
	layer.fScale:SetValue(0, scaleVec)
	scaleVec:Set(2, 2, 2)
	layer.fScale:SetValue(self.lifetime / 2, scaleVec)
	if (self.style == 1) then -- cloud
		scaleVec:Set(0.5, 0.5, 0.5)
	elseif (self.style == 2) then -- plume
		scaleVec:Set(3, 3, 3)
	end
	layer.fScale:SetValue(self.lifetime, scaleVec)
	if (flameDuration > 0) then
		layer.fScale:SetValue(smokeStartFrame, layer.fScale:GetValue(smokeStartFrame))
		scaleVec:Set(0.75, 0.75, 0.75)
		layer.fScale:SetValue(0, scaleVec)
	end
end

function LM_Smoke:AddFlameParticle(moho, particles)
	-- Create a new vector layer and add it to the particle group
	local layer = moho:CreateNewLayer(MOHO.LT_VECTOR)
	moho:PlaceLayerInGroup(layer, particles)
	layer:SetName("p" .. particles:CountLayers())

	-- Create a little blob shape
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local numPoints = 4
	local n = mesh:CountPoints()
	local v = LM.Vector2:new_local()
	local angle = math.pi / 2
	local dAngle = math.pi / numPoints
	local radius = MOHO.RandomRange(0.02, 0.1)

	if (self.style == 2) then -- plume
		radius = radius * 1.5
	end

	mesh:SelectNone()
	for i = 1, numPoints do
		local r = radius * MOHO.RandomRange(0.25, 1)
		v.x = r * math.cos(angle)
		v.y = r * math.sin(angle)
		if (i == 1) then
			mesh:AddLonePoint(v, 0)
		else
			mesh:AppendPoint(v, 0)
		end
		angle = angle + dAngle

		r = radius * MOHO.RandomRange(0.25, 1)
		v.x = r * math.cos(angle)
		v.y = r * math.sin(angle)
		mesh:AppendPoint(v, 0)
		angle = angle + dAngle
	end
	mesh:AppendPoint(v, 0)
	mesh:WeldPoints(n + numPoints * 2, n, 0)

	-- Now add a shape and set its color and style properties
	mesh:SelectConnected()
	local id = moho:CreateShape(true, 0, false)
	if (id >= 0) then
		local shape = mesh:Shape(id)
		shape.fHasOutline = false
		shape:RemoveStyles()
		shape.fMyStyle:SetSoftEdge(MOHO.RandomRange(6, 12) / 240)

		local color = self.flameColor * MOHO.RandomRange(0.5, 1.5)
		color.r = color.r * MOHO.RandomRange(0.95, 1.05)
		color.g = color.g * MOHO.RandomRange(0.95, 1.05)
		color.b = color.b * MOHO.RandomRange(0.95, 1.05)
		color.a = MOHO.RandomRange(0.1, 0.35)
		shape.fMyStyle.fFillCol:SetValue(0, color:AsColorStruct())
		color = self.flameColor * MOHO.RandomRange(0.5, 1.5)
		color.r = color.r * MOHO.RandomRange(0.95, 1.05)
		color.g = color.g * MOHO.RandomRange(0.95, 1.05)
		color.b = color.b * MOHO.RandomRange(0.95, 1.05)
		color.a = MOHO.RandomRange(0.1, 0.35)
		shape.fMyStyle.fFillCol:SetValue(self.flameDuration / 2, color:AsColorStruct())
		color = self.flameColor * MOHO.RandomRange(0.5, 1.5)
		color.r = color.r * MOHO.RandomRange(0.95, 1.05)
		color.g = color.g * MOHO.RandomRange(0.95, 1.05)
		color.b = color.b * MOHO.RandomRange(0.95, 1.05)
		color.a = 0
		shape.fMyStyle.fFillCol:SetValue(self.flameDuration + 1, color:AsColorStruct())
	end

	-- Scale this layer to make it appear to grow and then fade out
	local scaleVec = LM.Vector3:new_local()
	scaleVec:Set(0.5, 0.5, 0.5)
	layer.fScale:SetValue(0, scaleVec)
	scaleVec:Set(2, 2, 2)
	layer.fScale:SetValue(self.lifetime / 2, scaleVec)
	if (self.style == 1) then -- cloud
		scaleVec:Set(0.5, 0.5, 0.5)
	elseif (self.style == 2) then -- plume
		scaleVec:Set(3, 3, 3)
	end
	layer.fScale:SetValue(self.lifetime, scaleVec)
	layer:SetBlendingMode(MOHO.BM_ADD)
end
