-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Snow"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Snow = {}

function LM_Snow:Name()
	return "Snow"
end

function LM_Snow:Version()
	return "5.0"
end

function LM_Snow:Description()
	return "Creates a simple snowstorm."
end

function LM_Snow:Creator()
	return "Lost Marble"
end

function LM_Snow:UILabel()
	return("Snow")
end

-- **************************************************
-- The guts of this script
-- **************************************************

LM_Snow.color = LM.ColorVector:new_local()
LM_Snow.color:Set(0.9, 0.9, 0.9, 0.75)
LM_Snow.pCount = 500
LM_Snow.lifetime = 60

function LM_Snow:Run(moho)
	moho.document:PrepUndo(nil)
	moho.document:SetDirty()

	local v3 = LM.Vector3:new_local()
	local particles = moho:LayerAsParticle(moho:CreateNewLayer(MOHO.LT_PARTICLE))
	particles:SetName("Snow")

	particles:SetNumParticles(self.pCount, 20)
	particles:SetLifetime(self.lifetime)
	particles:SetOrientation(true)
	particles:SetFreeFloating(true)
	particles:SetEvenlySpaced(false)
	particles:SetRandomStartTime(false)
	particles:SetDirection(math.rad(260), math.rad(5))
	v3:Set(5, 0.1, 2)
	particles:SetSourceDimensions(v3)
	particles:SetVelocity(1, 0.2)
	particles:SetDamping(0)
	particles:SetAcceleration(0, 0)
	particles:SetFullSpeedStart(true)

	for i = 1, 6 do
		self:AddParticle(moho, particles)
	end

	particles:FinalizeSettings()
	particles:Expand(false)
	v3:Set(0.2, 1.1, 0)
	particles.fTranslation:SetValue(0, v3)
	moho:SetSelLayer(particles)
end

function LM_Snow:AddParticle(moho, particles)
	-- Create a new vector layer and add it to the particle group
	local layer = moho:CreateNewLayer(MOHO.LT_VECTOR)
	moho:PlaceLayerInGroup(layer, particles)
	layer:SetName("p" .. particles:CountLayers())

	-- Create a little droplet shape
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local radius = MOHO.RandomRange(0.01, 0.02)
	local v = LM.Vector2:new_local()
	v:Set(0, radius)
	mesh:AddLonePoint(v, 0)
	v:Set(-radius, 0)
	mesh:AppendPoint(v, 0)
	v:Set(0, -radius)
	mesh:AppendPoint(v, 0)
	v:Set(radius, 0)
	mesh:AppendPoint(v, 0)
	mesh:AppendPoint(v, 0)
	mesh:WeldPoints(4, 0, 0)

	-- Now add a shape and set its color and style properties
	mesh:SelectConnected()
	local id = moho:CreateShape(true, 0, false)
	if (id >= 0) then
		local shape = mesh:Shape(id)
		shape.fHasOutline = false
		shape:RemoveStyles()
		shape.fMyStyle:SetSoftEdge(MOHO.RandomRange(6, 12) / 240)

		local color = self.color * MOHO.RandomRange(0.9, 1.1)
		color.a = MOHO.RandomRange(0.4, 0.6)
		shape.fMyStyle.fFillCol:SetValue(0, color:AsColorStruct())
	end
end
