-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Sparkles"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Sparkles = {}

function LM_Sparkles:Name()
	return "Sparkles"
end

function LM_Sparkles:Version()
	return "5.0"
end

function LM_Sparkles:Description()
	return "Creates a sparkle effect."
end

function LM_Sparkles:Creator()
	return "Lost Marble"
end

function LM_Sparkles:UILabel()
	return("Sparkles...")
end

-- **************************************************
-- Sparkles dialog
-- **************************************************

local LM_SparklesDialog = {}

function LM_SparklesDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Sparkles", LM_SparklesDialog)
	local l = d:GetLayout()

	d.moho = moho

	l:PushH()
		l:AddChild(LM.GUI.StaticText("Choose color:"))
		d.color = LM.GUI.ColorSwatch(false)
		l:AddChild(d.color)
	l:Pop()

	return d
end

function LM_SparklesDialog:UpdateWidgets()
	self.color:SetValue(LM_Sparkles.color:AsColorStruct())
end

function LM_SparklesDialog:OnOK()
	LM_Sparkles.color:Set(self.color:Value())
end

-- **************************************************
-- The guts of this script
-- **************************************************

LM_Sparkles.color = LM.ColorVector:new_local()
LM_Sparkles.color:Set(1, 1, 1)
LM_Sparkles.pCount = 100
LM_Sparkles.lifetime = 20

function LM_Sparkles:Run(moho)
	local dlog = LM_SparklesDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	moho.document:PrepUndo(nil)
	moho.document:SetDirty()

	local v3 = LM.Vector3:new_local()
	local particles = moho:LayerAsParticle(moho:CreateNewLayer(MOHO.LT_PARTICLE))
	particles:SetName("Sparkles")

	particles:SetNumParticles(self.pCount, 20)
	particles:SetLifetime(self.lifetime)
	particles:SetOrientation(true)
	particles:SetFreeFloating(true)
	particles:SetEvenlySpaced(false)
	particles:SetRandomStartTime(false)
	particles:SetDirection(0, math.rad(180))
	v3:Set(0.2, 0.2, 0)
	particles:SetSourceDimensions(v3)
	particles:SetVelocity(0.01, 0)
	particles:SetDamping(0)
	particles:SetAcceleration(0, 0)
	particles:SetFullSpeedStart(true)

	for i = 1, 12 do
		self:AddParticle(moho, particles)
	end

	particles:FinalizeSettings()
	particles:Expand(false)
	moho:SetSelLayer(particles)
end

function LM_Sparkles:AddParticle(moho, particles)
	-- Create a new vector layer and add it to the particle group
	local layer = moho:CreateNewLayer(MOHO.LT_VECTOR)
	moho:PlaceLayerInGroup(layer, particles)
	layer:SetName("p" .. particles:CountLayers())

	-- Create a little droplet shape
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local radius = MOHO.RandomRange(0.05, 0.1)
	local blurRadius = MOHO.RandomRange(16, 20) / 240
	local v = LM.Vector2:new_local()

	mesh:SelectNone()
	v:Set(0, radius)
	mesh:AddLonePoint(v, 0)
	v:Set(-radius, 0)
	mesh:AppendPoint(v, 0)
	v:Set(0, -radius)
	mesh:AppendPoint(v, 0)
	v:Set(radius, 0)
	mesh:AppendPoint(v, 0)
	mesh:AppendPoint(v, 0)
	mesh:WeldPoints(4, 0, 0)

	-- Now add a shape and set its color and style properties
	mesh:SelectConnected()
	local id = moho:CreateShape(true, 0, false)
	if (id >= 0) then
		local shape = mesh:Shape(id)
		shape.fHasOutline = false
		shape:RemoveStyles()
		shape.fMyStyle:SetSoftEdge(blurRadius)
		shape.fMyStyle.fFillCol:SetValue(0, self.color:AsColorStruct())
	end

	mesh:SelectNone()
	radius = radius - radius / 4
	v:Set(0, radius)
	mesh:AddLonePoint(v, 0)
	v:Set(-radius, 0)
	mesh:AppendPoint(v, 0)
	v:Set(0, -radius)
	mesh:AppendPoint(v, 0)
	v:Set(radius, 0)
	mesh:AppendPoint(v, 0)
	mesh:AppendPoint(v, 0)
	mesh:WeldPoints(8, 4, 0)

	-- Add a second shape in the center of the first, without blur
	mesh:SelectConnected()
	local id = moho:CreateShape(true, 0, false)
	if (id >= 0) then
		local shape = mesh:Shape(id)
		shape.fHasOutline = false
		shape:RemoveStyles()
		shape.fMyStyle.fFillCol:SetValue(0, self.color:AsColorStruct())
	end

	for i = 0, mesh:CountPoints() - 1 do
		mesh:Point(i):SetCurvature(0.391379, 0)
	end

	local v3 = LM.Vector3:new_local()
	v3:Set(1, 1, 1)
	layer.fScale:SetValue(0, v3)
	v3:Set(0, 0, 0)
	layer.fScale:SetValue(self.lifetime, v3)

	radius = MOHO.RandomRange(0.5, 1)
	local angle = math.rad(MOHO.RandomRange(-180, 180))

	v3:Set(radius / 3, 0, 0)
	v3:Rotate(LM.Z_AXIS, angle / 3)
	layer.fTranslation:SetValue(self.lifetime / 3, v3)

	v3:Set(2 * radius / 3, 0, 0)
	v3:Rotate(LM.Z_AXIS, 2 * angle / 3)
	layer.fTranslation:SetValue(2 * self.lifetime / 3, v3)

	v3:Set(radius, 0, 0)
	v3:Rotate(LM.Z_AXIS, angle)
	layer.fTranslation:SetValue(self.lifetime, v3)
end
