-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_OpenFile"

-- **************************************************
-- General information about this script
-- **************************************************

LM_OpenFile = {}

function LM_OpenFile:Name()
	return "Open File"
end

function LM_OpenFile:Version()
	return "5.0"
end

function LM_OpenFile:Description()
	return "Data file test script"
end

function LM_OpenFile:Creator()
	return "Lost Marble"
end

function LM_OpenFile:UILabel()
	return("Load Data File...")
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_OpenFile:IsEnabled(moho)
	if (moho.layer:LayerType() ~= MOHO.LT_VECTOR) then
		return false
	end
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	return true
end

function LM_OpenFile:Run(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	LM.GUI.Alert(LM.GUI.ALERT_INFO, "This script demonstrates how to read data from an external file.",
		"When prompted for a file, look in the Moho program folder under:",
		"scripts/menu/Script Writing/sample_data_file.txt", "OK", nil, nil)

	LM.GUI.Alert(LM.GUI.ALERT_INFO, "This script isn't meant to be useful on its own.",
		"It is an example of how you might start out if you",
		"want to import some custom data from a file.", "OK", nil, nil)

	local path = LM.GUI.OpenFile("Select Data File")
	if (path == "") then
		return
	end

	local f = io.open(path, "r")
	if (f == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	mesh:SelectNone()

	local v = LM.Vector2:new_local()
	local cmd = f:read()
	while (cmd ~= nil) do
		if (cmd == "moveto") then
			v.x = f:read("*n")
			v.y = f:read("*n")
			mesh:AddLonePoint(v, moho.frame)
		elseif (cmd == "curveto") then
			v.x = f:read("*n")
			v.y = f:read("*n")
			mesh:AppendPoint(v, moho.frame)
		end

		cmd = f:read()
	end

	f:close()

	mesh:SelectConnected()
end
