-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_BoneSound"

-- **************************************************
-- General information about this script
-- **************************************************

LM_BoneSound = {}

function LM_BoneSound:Name()
	return "Bone Sound"
end

function LM_BoneSound:Version()
	return "5.0"
end

function LM_BoneSound:Description()
	return "Uses a sound file to control a bone's movement."
end

function LM_BoneSound:Creator()
	return "Lost Marble"
end

function LM_BoneSound:UILabel()
	return("Bone Audio Wiggle...")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_BoneSound.magnitude = 90
LM_BoneSound.stepSize = 2

-- **************************************************
-- Bone Sound dialog
-- **************************************************

local LM_BoneSoundDialog = {}

function LM_BoneSoundDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Bone Audio Wiggle", LM_BoneSoundDialog)
	local l = d:GetLayout()

	d.moho = moho

	l:AddChild(LM.GUI.StaticText("When this dialog closes, you will be prompted to select an audio file."), LM.GUI.ALIGN_LEFT)
	l:AddChild(LM.GUI.StaticText("The volume of the audio will be used to modulate the angle of the selected bone."), LM.GUI.ALIGN_LEFT)
	l:AddChild(LM.GUI.Divider(false), LM.GUI.ALIGN_FILL)

	l:PushH(LM.GUI.ALIGN_CENTER)
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Max angle"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Frame step"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.magnitude = LM.GUI.TextControl(0, "0.0000", 0, LM.GUI.FIELD_FLOAT)
			l:AddChild(d.magnitude)
			d.stepSize = LM.GUI.TextControl(0, "0.0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.stepSize)
		l:Pop()
	l:Pop()

	return d
end

function LM_BoneSoundDialog:UpdateWidgets()
	self.magnitude:SetValue(LM_BoneSound.magnitude)
	self.stepSize:SetValue(LM_BoneSound.stepSize)
end

function LM_BoneSoundDialog:OnValidate()
	local b = true
	if (not self:Validate(self.magnitude, -360, 360)) then
		b = false
	end
	if (not self:Validate(self.stepSize, 1, 1000)) then
		b = false
	end
	return b
end

function LM_BoneSoundDialog:OnOK()
	LM_BoneSound.magnitude = self.magnitude:FloatValue()
	LM_BoneSound.stepSize = self.stepSize:FloatValue()
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_BoneSound:IsEnabled(moho)
	if (moho.layer:LayerType() ~= MOHO.LT_BONE) then
		return false
	end
	if (moho.frame < 1) then
		return false
	end
	if (moho:CountSelectedBones() < 1) then
		return false
	end
	return true
end

function LM_BoneSound:Run(moho)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	-- ask user for the angle offset magnitude
	local dlog = LM_BoneSoundDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	-- let user select an audio file
	local path = LM.GUI.OpenFile("Select Audio File")
	if (path == "") then
		return
	end

	-- begin audio extraction
	local audioDuration = moho:BeginAudioExtraction(path)
	if (audioDuration <= 0) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	local frame = moho.frame
	local baseAngle = {}
	local magnitude = math.rad(self.magnitude)
	local audioTime = 0
	local frameDuration = self.stepSize / moho.document:Fps()

	for i = 0, skel:CountBones() - 1 do
		table.insert(baseAngle, skel:Bone(i).fAnimAngle:GetValue(frame))
	end

	while (audioTime < audioDuration) do
		local amp = moho:GetAudioAmplitude(audioTime, frameDuration)

		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			if (bone.fSelected) then
				local angle = baseAngle[i + 1] + amp * magnitude
				bone.fAnimAngle:SetValue(frame, angle)
			end
		end

		audioTime = audioTime + frameDuration
		frame = frame + self.stepSize
	end

	moho:EndAudioExtraction()
end
