-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Fade"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Fade = {}

function LM_Fade:Name()
	return "Fade"
end

function LM_Fade:Version()
	return "5.0"
end

function LM_Fade:Description()
	return "Fade in/out the current layer."
end

function LM_Fade:Creator()
	return "Lost Marble"
end

function LM_Fade:UILabel()
	return("Fade...")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_Fade.duration = 12
LM_Fade.blur = 16
LM_Fade.fadeOut = true

-- **************************************************
-- Fade dialog
-- **************************************************

local LM_FadeDialog = {}

function LM_FadeDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Fade", LM_FadeDialog)
	local l = d:GetLayout()

	d.moho = moho

	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Duration (frames)"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Blur radius"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.duration = LM.GUI.TextControl(0, "0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.duration)
			d.blur = LM.GUI.TextControl(0, "0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.blur)
		l:Pop()
	l:Pop()

	d.fadeIn = LM.GUI.RadioButton("Fade in")
	l:AddChild(d.fadeIn, LM.GUI.ALIGN_LEFT)
	d.fadeOut = LM.GUI.RadioButton("Fade out")
	l:AddChild(d.fadeOut, LM.GUI.ALIGN_LEFT)

	return d
end

function LM_FadeDialog:UpdateWidgets()
	self.duration:SetValue(LM_Fade.duration)
	self.blur:SetValue(LM_Fade.blur)
	self.fadeIn:SetValue(not LM_Fade.fadeOut)
	self.fadeOut:SetValue(LM_Fade.fadeOut)
end

function LM_FadeDialog:OnValidate()
	local b = true
	if (not self:Validate(self.duration, 2, 1000000)) then
		b = false
	end
	if (not self:Validate(self.blur, 0, 30)) then
		b = false
	end
	return b
end

function LM_FadeDialog:OnOK()
	LM_Fade.duration = self.duration:IntValue()
	LM_Fade.blur = self.blur:IntValue()
	LM_Fade.fadeOut = self.fadeOut:Value()
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_Fade:Run(moho)
	local dlog = LM_FadeDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	moho.document:SetDirty()
	moho.document:PrepUndo(moho.layer)

	if (self.fadeOut) then
		moho.layer.fVisibility:SetValue(moho.frame, true)
		moho.layer.fVisibility:SetValue(moho.frame + self.duration, false)

		moho.layer.fAlpha:AddKey(moho.frame)
		moho.layer.fAlpha:SetValue(moho.frame + self.duration, 0)

		moho.layer.fBlur:AddKey(moho.frame)
		moho.layer.fBlur:SetValue(moho.frame + self.duration, moho:PixelToDoc(self.blur))
	else
		moho.layer.fVisibility:SetValue(moho.frame, true)

		moho.layer.fAlpha:SetValue(moho.frame, 0)
		moho.layer.fAlpha:SetValue(moho.frame + self.duration, 1)

		moho.layer.fBlur:SetValue(moho.frame, moho:PixelToDoc(self.blur))
		moho.layer.fBlur:SetValue(moho.frame + self.duration, 0)
	end

	--fView->SendMessage(MSG_CHANNEL_ALPHA)
	--fView->SendMessage(MSG_CHANNEL_BLUR)
	--fView->SendMessage(MSG_CHANNEL_VISIBILITY)
end
