-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_WavyFade"

-- **************************************************
-- General information about this script
-- **************************************************

LM_WavyFade = {}

function LM_WavyFade:Name()
	return "Wavy Fade"
end

function LM_WavyFade:Version()
	return "5.0"
end

function LM_WavyFade:Description()
	return "Fade in/out the current layer with a wavy motion effect."
end

function LM_WavyFade:Creator()
	return "Lost Marble"
end

function LM_WavyFade:UILabel()
	return("Wavy Fade...")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_WavyFade.duration = 48
LM_WavyFade.frequency = 1
LM_WavyFade.amplitude = 1
LM_WavyFade.blur = 16
LM_WavyFade.fadeOut = true

-- **************************************************
-- Wavy dialog
-- **************************************************

local LM_WavyFadeDialog = {}

function LM_WavyFadeDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Wavy Fade", LM_WavyFadeDialog)
	local l = d:GetLayout()

	d.moho = moho

	l:AddChild(LM.GUI.StaticText("Note: This script adds a lot of keyframes,"), LM.GUI.ALIGN_LEFT)
	l:AddChild(LM.GUI.StaticText("so it's best to use it when you're finished"), LM.GUI.ALIGN_LEFT)
	l:AddChild(LM.GUI.StaticText("with all other animation in this layer."), LM.GUI.ALIGN_LEFT)
	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Duration (frames)"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Frequency"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Amplitude"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Blur radius"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.duration = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.duration)
			d.frequency = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_UFLOAT)
			l:AddChild(d.frequency)
			d.amplitude = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_UFLOAT)
			l:AddChild(d.amplitude)
			d.blur = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.blur)
		l:Pop()
	l:Pop()

	d.fadeIn = LM.GUI.RadioButton("Fade in")
	l:AddChild(d.fadeIn, LM.GUI.ALIGN_LEFT)
	d.fadeOut = LM.GUI.RadioButton("Fade out")
	l:AddChild(d.fadeOut, LM.GUI.ALIGN_LEFT)

	return d
end

function LM_WavyFadeDialog:UpdateWidgets()
	self.duration:SetValue(LM_WavyFade.duration)
	self.frequency:SetValue(LM_WavyFade.frequency)
	self.amplitude:SetValue(LM_WavyFade.amplitude)
	self.blur:SetValue(LM_WavyFade.blur)
	self.fadeIn:SetValue(not LM_WavyFade.fadeOut)
	self.fadeOut:SetValue(LM_WavyFade.fadeOut)
end

function LM_WavyFadeDialog:OnValidate()
	local b = true
	if (not self:Validate(self.duration, 2, 1000000)) then
		b = false
	end
	if (not self:Validate(self.frequency, 0.01, 100)) then
		b = false
	end
	if (not self:Validate(self.amplitude, 0.01, 4)) then
		b = false
	end
	if (not self:Validate(self.blur, 0, 30)) then
		b = false
	end
	return b
end

function LM_WavyFadeDialog:OnOK()
	LM_WavyFade.duration = self.duration:IntValue()
	LM_WavyFade.frequency = self.frequency:FloatValue()
	LM_WavyFade.amplitude = self.amplitude:FloatValue()
	LM_WavyFade.blur = self.blur:IntValue()
	LM_WavyFade.fadeOut = self.fadeOut:Value()
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_WavyFade:IsEnabled(moho)
	if (moho:CountSelectedPoints() > 0) then
		return true
	end
	return false
end

function LM_WavyFade:Run(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local dlog = LM_WavyFadeDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	moho.document:SetDirty()
	moho.document:PrepUndo(moho.layer)

	local selList = MOHO.SelectedPointList(mesh)

	local newPos = LM.Vector2:new_local()

	local amp1 = 0.16 * self.amplitude
	local freq1_t = 0.3 * self.frequency
	local freq1_s = 6
	local phase1 = 0

	local amp2 = 0.2 * self.amplitude
	local freq2_t = 0.25 * self.frequency
	local freq2_s = 5
	local phase2 = 0.5

	local amp3 = 0.05 * self.amplitude
	local freq3_t = 0.8 * self.frequency
	local freq3_s = 16
	local phase3 = 1.3

	local amp4 = 0.06 * self.amplitude
	local freq4_t = 0.6 * self.frequency
	local freq4_s = 13
	local phase4 = 2.6

	local inc = 1

	for i, pt in selList do
		-- first, add "neutral" keyframes to prevent unwanted motion
		for fr = 0, self.duration, inc do
			pt.fAnimPos:AddKey(moho.frame + fr)
		end
		-- next, replace the neutral keys with the actual wavy motion effect
		for fr = 0, self.duration, inc do
			local percent = fr / self.duration
			local v = pt.fAnimPos:GetValue(moho.frame + fr)

			if (not self.fadeOut) then
				percent = 1 - percent
			end

			newPos.x = v.x
			newPos.y = v.y
			newPos.y = newPos.y + math.sin(fr * freq1_t + v.x * freq1_s + v.y + phase1) * LM.Lerp(percent, 0, amp1)
			newPos.x = newPos.x + math.sin(fr * freq2_t + v.y * freq2_s + v.x + phase2) * LM.Lerp(percent, 0, amp2)
			v.x = newPos.x
			v.y = newPos.y
			newPos.y = newPos.y + math.sin(fr * freq3_t + v.y * freq3_s + phase3) * LM.Lerp(percent, 0, amp3)
			newPos.x = newPos.x + math.sin(fr * freq4_t + v.x * freq4_s + phase4) * LM.Lerp(percent, 0, amp4)

			pt.fAnimPos:SetValue(moho.frame + fr, newPos)
		end
	end

	if (self.fadeOut) then
		moho.layer.fVisibility:SetValue(moho.frame, true)
		moho.layer.fVisibility:SetValue(moho.frame + self.duration / 3, true)
		moho.layer.fVisibility:SetValue(moho.frame + self.duration, false)

		moho.layer.fAlpha:AddKey(moho.frame)
		moho.layer.fAlpha:AddKey(moho.frame + self.duration / 3)
		moho.layer.fAlpha:SetValue(moho.frame + self.duration, 0)

		moho.layer.fBlur:AddKey(moho.frame)
		moho.layer.fBlur:AddKey(moho.frame + self.duration / 3)
		moho.layer.fBlur:SetValue(moho.frame + self.duration, moho:PixelToDoc(self.blur))
	else
		moho.layer.fVisibility:SetValue(moho.frame, true)

		moho.layer.fAlpha:SetValue(moho.frame, 0)
		moho.layer.fAlpha:SetValue(moho.frame + self.duration - self.duration / 3, 1)
		moho.layer.fAlpha:SetValue(moho.frame + self.duration, 1)

		moho.layer.fBlur:SetValue(moho.frame, moho:PixelToDoc(self.blur))
		moho.layer.fBlur:SetValue(moho.frame + self.duration - self.duration / 3, 0)
		moho.layer.fBlur:SetValue(moho.frame + self.duration, 0)
	end

	selList = nil

	--fView->SendMessage(MSG_CHANNEL_POINT)
end
