-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_BlackHole"

-- **************************************************
-- General information about this script
-- **************************************************

LM_BlackHole = {}

function LM_BlackHole:Name()
	return "Black Hole"
end

function LM_BlackHole:Version()
	return "5.0"
end

function LM_BlackHole:Description()
	return "Sucks the selected points down into the layer's origin."
end

function LM_BlackHole:Creator()
	return "Lost Marble"
end

function LM_BlackHole:UILabel()
	return("Black Hole...")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_BlackHole.duration = 24
LM_BlackHole.angle = 180
LM_BlackHole.reverse = false

-- **************************************************
-- Black Hole dialog
-- **************************************************

local LM_BlackHoleDialog = {}

function LM_BlackHoleDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Black Hole", LM_BlackHoleDialog)
	local l = d:GetLayout()

	d.moho = moho

	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Duration (frames)"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Angle"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.duration = LM.GUI.TextControl(0, "0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.duration)
			d.angle = LM.GUI.TextControl(0, "0000", 0, LM.GUI.FIELD_INT)
			l:AddChild(d.angle)
		l:Pop()
	l:Pop()

	d.reverse = LM.GUI.CheckBox("Run backwards")
	l:AddChild(d.reverse, LM.GUI.ALIGN_LEFT)

	return d
end

function LM_BlackHoleDialog:UpdateWidgets()
	self.duration:SetValue(LM_BlackHole.duration)
	self.angle:SetValue(LM_BlackHole.angle)
	self.reverse:SetValue(LM_BlackHole.reverse)
end

function LM_BlackHoleDialog:OnValidate()
	local b = true
	if (not self:Validate(self.duration, 2, 1000000)) then
		b = false
	end
	return b
end

function LM_BlackHoleDialog:OnOK()
	LM_BlackHole.duration = self.duration:IntValue()
	LM_BlackHole.angle = self.angle:IntValue()
	LM_BlackHole.reverse = self.reverse:Value()
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_BlackHole:IsEnabled(moho)
	if (moho:CountSelectedPoints() > 0) then
		return true
	end
	return false
end

function LM_BlackHole:Run(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local dlog = LM_BlackHoleDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	moho.document:SetDirty()
	moho.document:PrepUndo(moho.layer)

	local maxRadius = 0
	local origin = moho.layer:Origin()
	local selList = MOHO.SelectedPointList(mesh)

	for i, pt in selList do
		local v = pt.fPos - origin
		maxRadius = math.max(maxRadius, v:Mag())
	end

	local inc = LM.Round(self.duration / (16 * math.abs(self.angle) / 360))
	if (inc < 1) then
		inc = 1
	end

	for fr = 0, self.duration, inc do
		local frame = fr
		if (self.reverse) then
			frame = self.duration - fr
		end
		for i, pt in selList do
			local v = pt.fPos - origin
			local f = frame / self.duration
			local g = v:Mag() / maxRadius

			-- contraction
			f = LM.Lerp(g, f * 3, f)
			if (f > 1) then
				f = 1
			end
			v = v * LM.Lerp(f, 1, 0)

			-- twist
			f = frame / self.duration
			g = 1 - v:Mag() / maxRadius
			g = g * g * g
			g = 1 - g
			v:Rotate(LM.Lerp(g, LM.Lerp(f, 0, math.rad(self.angle)), 0))
			pt.fAnimPos:SetValue(moho.frame + fr, origin + v)
		end
	end

	selList = nil

	--fView->SendMessage(MSG_CHANNEL_POINT)
end
