-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Wavy"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Wavy = {}

function LM_Wavy:Name()
	return "Wavy"
end

function LM_Wavy:Version()
	return "5.0"
end

function LM_Wavy:Description()
	return "Moves the selected points in a wavy, underwater-style motion."
end

function LM_Wavy:Creator()
	return "Lost Marble"
end

function LM_Wavy:UILabel()
	return("Wavy...")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_Wavy.duration = 72
LM_Wavy.frequency = 1
LM_Wavy.amplitude = 1
LM_Wavy.constant = true

-- **************************************************
-- Wavy dialog
-- **************************************************

local LM_WavyDialog = {}

function LM_WavyDialog:new(moho)
	local d = LM.GUI.SimpleDialog("Wavy", LM_WavyDialog)
	local l = d:GetLayout()

	d.moho = moho

	l:AddChild(LM.GUI.StaticText("Note: This script adds a lot of keyframes,"), LM.GUI.ALIGN_LEFT)
	l:AddChild(LM.GUI.StaticText("so it's best to use it when you're finished"), LM.GUI.ALIGN_LEFT)
	l:AddChild(LM.GUI.StaticText("with all other animation in this layer."), LM.GUI.ALIGN_LEFT)
	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Duration (frames)"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Frequency"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Amplitude"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.duration = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_UINT)
			l:AddChild(d.duration)
			d.frequency = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_UFLOAT)
			l:AddChild(d.frequency)
			d.amplitude = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_UFLOAT)
			l:AddChild(d.amplitude)
		l:Pop()
	l:Pop()

	d.constant = LM.GUI.RadioButton("Constant amplitude")
	l:AddChild(d.constant, LM.GUI.ALIGN_LEFT)
	d.nonConstant = LM.GUI.RadioButton("Temporary wave")
	l:AddChild(d.nonConstant, LM.GUI.ALIGN_LEFT)

	return d
end

function LM_WavyDialog:UpdateWidgets()
	self.duration:SetValue(LM_Wavy.duration)
	self.frequency:SetValue(LM_Wavy.frequency)
	self.amplitude:SetValue(LM_Wavy.amplitude)
	self.constant:SetValue(LM_Wavy.constant)
	self.nonConstant:SetValue(not LM_Wavy.constant)
end

function LM_WavyDialog:OnValidate()
	local b = true
	if (not self:Validate(self.duration, 2, 1000000)) then
		b = false
	end
	if (not self:Validate(self.frequency, 0.01, 100)) then
		b = false
	end
	if (not self:Validate(self.amplitude, 0.01, 4)) then
		b = false
	end
	return b
end

function LM_WavyDialog:OnOK()
	LM_Wavy.duration = self.duration:IntValue()
	LM_Wavy.frequency = self.frequency:FloatValue()
	LM_Wavy.amplitude = self.amplitude:FloatValue()
	LM_Wavy.constant = self.constant:Value()
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_Wavy:IsEnabled(moho)
	if (moho:CountSelectedPoints() > 0) then
		return true
	end
	return false
end

function LM_Wavy:Run(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local dlog = LM_WavyDialog:new(moho)
	if (dlog:DoModal() == LM.GUI.MSG_CANCEL) then
		return
	end

	moho.document:SetDirty()
	moho.document:PrepUndo(moho.layer)

	local startFrame = moho.frame

	local selList = MOHO.SelectedPointList(mesh)

	local v = LM.Vector2:new_local()
	local newPos = LM.Vector2:new_local()

	local amp1 = 0.04 * self.amplitude
	local freq1_t = 0.15 * self.frequency
	local freq1_s = 6
	local phase1 = 0

	local amp2 = 0.05 * self.amplitude
	local freq2_t = 0.125 * self.frequency
	local freq2_s = 5
	local phase2 = 0.5

	local amp3 = 0.0125 * self.amplitude
	local freq3_t = 0.4 * self.frequency
	local freq3_s = 16
	local phase3 = 1.3

	local amp4 = 0.015 * self.amplitude
	local freq4_t = 0.3 * self.frequency
	local freq4_s = 13
	local phase4 = 2.6

	local inc = 1

	-- first, add "neutral" keyframes to prevent unwanted motion
	for i, pt in selList do
		for fr = 0, self.duration, inc do
			pt.fAnimPos:AddKey(moho.frame + fr)
		end
	end

	-- next, replace the neutral keys with the actual wavy motion effect
	for fr = 0, self.duration, inc do
		moho:SetCurFrame(startFrame + fr)
		for i, pt in selList do
			if (not self.constant) then
				local percent = fr / self.duration
				if (percent <= 0.5) then
					percent = percent / 0.5
				else
					percent = percent - 0.5
					percent = 1 - percent / 0.5
				end

				amp1 = 0.04 * self.amplitude
				amp1 = LM.Lerp(percent, 0, amp1)

				amp2 = 0.05 * self.amplitude
				amp2 = LM.Lerp(percent, 0, amp2)

				amp3 = 0.0125 * self.amplitude
				amp3 = LM.Lerp(percent, 0, amp3)

				amp4 = 0.015 * self.amplitude
				amp4 = LM.Lerp(percent, 0, amp4)
			end

			--local v = pt.fAnimPos:GetValue(startFrame + fr)
			v:Set(pt.fPos)
			newPos:Set(v)

			newPos.y = newPos.y + math.sin(fr * freq1_t + v.x * freq1_s + v.y + phase1) * amp1
			newPos.x = newPos.x + math.sin(fr * freq2_t + v.y * freq2_s + v.x + phase2) * amp2
			v.x = newPos.x
			v.y = newPos.y
			newPos.y = newPos.y + math.sin(fr * freq3_t + v.y * freq3_s + phase3) * amp3
			newPos.x = newPos.x + math.sin(fr * freq4_t + v.x * freq4_s + phase4) * amp4

			v:Set(pt.fAnimPos:GetValue(startFrame + fr))
			newPos.x = newPos.x + v.x - pt.fPos.x
			newPos.y = newPos.y + v.y - pt.fPos.y
			pt.fAnimPos:SetValue(startFrame + fr, newPos)
		end
	end

	moho:SetCurFrame(startFrame)

	selList = nil
end
