-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Arrow"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Arrow = {}

function LM_Arrow:Name()
	return "Arrow"
end

function LM_Arrow:Version()
	return "5.0"
end

function LM_Arrow:Description()
	return "Draw an arrow (hold shift to constrain)"
end

function LM_Arrow:Creator()
	return "Lost Marble"
end

function LM_Arrow:UILabel()
	return("Arrow")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_Arrow.basePt = -1
LM_Arrow.startPt = -1
LM_Arrow.endPt = -1

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_Arrow:IsEnabled(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return false
	end
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	return true
end

function LM_Arrow:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	mesh:SelectNone()

	self.basePt = mesh:CountPoints()

	if (moho.gridOn) then
		moho:SnapToGrid(mouseEvent.vec)
	end
	-- the main line
	mesh:AddPoint(mouseEvent.vec, -1, moho.frame)
	mesh:Point(self.basePt).fSelected = true
	mesh:Point(self.basePt + 1).fSelected = true
	moho:CreateShape(false)

	-- the start arrow
	if (self.startArrow) then
		self.startPt = mesh:CountPoints()
		mesh:AddLonePoint(mouseEvent.vec, moho.frame)
		mesh:AppendPoint(mouseEvent.vec, moho.frame)
		mesh:AppendPoint(mouseEvent.vec, moho.frame)
		mesh:AppendPoint(mouseEvent.vec, moho.frame)
		mesh:AppendPoint(mouseEvent.vec, moho.frame)
		mesh:WeldPoints(self.startPt + 4, self.startPt, 0)
		-- peak the corners of the start arrow
		mesh:Point(self.startPt):SetCurvature(MOHO.PEAKED, moho.frame)
		mesh:Point(self.startPt + 2):SetCurvature(MOHO.PEAKED, moho.frame)
		mesh:Point(self.startPt + 3):SetCurvature(MOHO.PEAKED, moho.frame)
		-- attach the start arrow to the main line
		mesh:WeldPoints(self.startPt + 3, self.basePt, 0)
		-- fill in the start arrowhead
		mesh:SelectNone()
		mesh:Point(self.basePt).fSelected = true
		mesh:Point(self.startPt).fSelected = true
		mesh:Point(self.startPt + 1).fSelected = true
		mesh:Point(self.startPt + 2).fSelected = true
		local shapeID = moho:CreateShape(true)
		if (shapeID >= 0) then
			local shape = mesh:Shape(shapeID)
			shape.fMyStyle.fFillCol:SetValue(moho.frame, shape.fMyStyle.fLineCol:GetValue(moho.frame))
			shape.fMyStyle.fFillCol:SetValue(0, shape.fMyStyle.fLineCol:GetValue(0))
		end
	end

	-- the end arrow
	if (self.endArrow) then
		self.endPt = mesh:CountPoints()
		mesh:AddLonePoint(mouseEvent.vec, moho.frame)
		mesh:AppendPoint(mouseEvent.vec, moho.frame)
		mesh:AppendPoint(mouseEvent.vec, moho.frame)
		mesh:AppendPoint(mouseEvent.vec, moho.frame)
		mesh:AppendPoint(mouseEvent.vec, moho.frame)
		mesh:WeldPoints(self.endPt + 4, self.endPt, 0)
		-- peak the corners of the end arrow
		mesh:Point(self.endPt):SetCurvature(MOHO.PEAKED, moho.frame)
		mesh:Point(self.endPt + 2):SetCurvature(MOHO.PEAKED, moho.frame)
		mesh:Point(self.endPt + 3):SetCurvature(MOHO.PEAKED, moho.frame)
		-- attach the end arrow to the main line
		mesh:WeldPoints(self.endPt + 3, self.basePt + 1, 0)
		-- fill in the end arrowhead
		mesh:SelectNone()
		mesh:Point(self.basePt + 1).fSelected = true
		mesh:Point(self.endPt).fSelected = true
		mesh:Point(self.endPt + 1).fSelected = true
		mesh:Point(self.endPt + 2).fSelected = true
		local shapeID = moho:CreateShape(true)
		if (shapeID >= 0) then
			local shape = mesh:Shape(shapeID)
			shape.fMyStyle.fFillCol:SetValue(moho.frame, shape.fMyStyle.fLineCol:GetValue(moho.frame))
			shape.fMyStyle.fFillCol:SetValue(0, shape.fMyStyle.fLineCol:GetValue(0))
		end
	end

	mesh:Point(self.basePt).fSelected = true
	mesh:SelectConnected()

	mouseEvent.view:DrawMe()
end

function LM_Arrow:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (moho.gridOn) then
		moho:SnapToGrid(mouseEvent.startVec)
		moho:SnapToGrid(mouseEvent.vec)
	end

	-- line up the endpoints
	local startPt = mesh:Point(self.basePt)
	startPt.fPos:Set(mouseEvent.startVec)
	local endPt = mesh:Point(self.basePt + 1)
	endPt.fPos:Set(mouseEvent.vec)

	if (mouseEvent.shiftKey) then
		if (math.abs(endPt.fPos.x - startPt.fPos.x) > math.abs(endPt.fPos.y - startPt.fPos.y)) then
			endPt.fPos.y = startPt.fPos.y
		else
			endPt.fPos.x = startPt.fPos.x
		end
	end

	-- line up the start arrow
	if (self.startArrow) then
		local v = startPt.fPos - endPt.fPos
		v = v:Norm() * 0.075
		mesh:Point(self.startPt + 1).fPos:Set(startPt.fPos - v)
		v = v * 1.5
		v:Rotate(math.rad(20))
		mesh:Point(self.startPt).fPos:Set(startPt.fPos - v)
		v:Rotate(math.rad(-40))
		mesh:Point(self.startPt + 2).fPos:Set(startPt.fPos - v)
	end

	-- line up the end arrow
	if (self.endArrow) then
		local v = endPt.fPos - startPt.fPos
		v = v:Norm() * 0.075
		mesh:Point(self.endPt + 1).fPos:Set(endPt.fPos - v)
		v = v * 1.5
		v:Rotate(math.rad(20))
		mesh:Point(self.endPt).fPos:Set(endPt.fPos - v)
		v:Rotate(math.rad(-40))
		mesh:Point(self.endPt + 2).fPos:Set(endPt.fPos - v)
	end

	moho:AddPointKeyframe(moho.frame)
	mouseEvent.view:DrawMe()
end

function LM_Arrow:OnMouseUp(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho:AddPointKeyframe(moho.frame)
	mouseEvent.view:DrawMe()
	moho:NewKeyframe(CHANNEL_POINT)
	moho:UpdateSelectedChannels()
end

function LM_Arrow:OnKeyDown(moho, keyEvent)
	LM_SelectPoints:OnKeyDown(moho, keyEvent)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_Arrow.START = MOHO.MSG_BASE
LM_Arrow.END = MOHO.MSG_BASE + 1
LM_Arrow.startArrow = false
LM_Arrow.endArrow = true

function LM_Arrow:DoLayout(moho, layout)
	self.startCheck = LM.GUI.CheckBox("Arrow at start", self.START)
	layout:AddChild(self.startCheck)

	self.endCheck = LM.GUI.CheckBox("Arrow at end", self.END)
	layout:AddChild(self.endCheck)
end

function LM_Arrow:UpdateWidgets(moho)
	self.startCheck:SetValue(LM_Arrow.startArrow)
	self.endCheck:SetValue(LM_Arrow.endArrow)
end

function LM_Arrow:HandleMessage(moho, view, msg)
	if (msg == self.START) then
		LM_Arrow.startArrow = self.startCheck:Value()
		if (LM_Arrow.startArrow == false and self.endCheck:Value() == false) then
			LM_Arrow.endArrow = true
			self.endCheck:SetValue(LM_Arrow.endArrow)
		end
	elseif (msg == self.END) then
		LM_Arrow.endArrow = self.endCheck:Value()
		if (LM_Arrow.endArrow == false and self.startCheck:Value() == false) then
			LM_Arrow.startArrow = true
			self.startCheck:SetValue(LM_Arrow.startArrow)
		end
	end
end
