-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_BendPointsH"

-- **************************************************
-- General information about this script
-- **************************************************

LM_BendPointsH = {}

function LM_BendPointsH:Name()
	return "Bend Points H"
end

function LM_BendPointsH:Version()
	return "5.0"
end

function LM_BendPointsH:Description()
	return "Bend selected points in a horizontal arc"
end

function LM_BendPointsH:Creator()
	return "Lost Marble"
end

function LM_BendPointsH:UILabel()
	return("Bend Points H")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_BendPointsH.minVec = LM.Vector2:new_local()
LM_BendPointsH.maxVec = LM.Vector2:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_BendPointsH:IsEnabled(moho)
	if (moho:CountSelectedPoints() > 1) then
		return true
	end
	return false
end

function LM_BendPointsH:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	mesh:SelectedBounds(self.minVec, self.maxVec)

	self.selList = MOHO.SelectedPointList(mesh)
	mesh:PrepMovePoints()

	self:OnMouseMoved(moho, mouseEvent)
end

function LM_BendPointsH:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local reversed = false
	local maxRadius = 100
	local minCircumference = self.maxVec.x - self.minVec.x
	local minRadius = minCircumference / (2 * math.pi)
	local radius = maxRadius
	local bendCenter = LM.Vector2:new_local()
	local circleCenter = LM.Vector2:new_local()
	local percent = 2 * (mouseEvent.pt.y - mouseEvent.startPt.y) / mouseEvent.view:Graphics():Height()

	if (percent < 0) then
		percent = -percent
		reversed = true
	end

	percent = LM.Clamp(percent, 0, 1)
	percent = 1 - percent
	percent = percent * percent
	percent = percent * percent
	radius = LM.Lerp(percent, minRadius, maxRadius)

	if (reversed) then
		bendCenter.x = (self.minVec.x + self.maxVec.x) / 2
		bendCenter.y = self.maxVec.y
		circleCenter.x = bendCenter.x
		circleCenter.y = bendCenter.y + radius
	else
		bendCenter.x = (self.minVec.x + self.maxVec.x) / 2
		bendCenter.y = self.minVec.y
		circleCenter.x = bendCenter.x
		circleCenter.y = bendCenter.y - radius
	end

	for i, pt in self.selList do
		local arcLen = pt.fTempPos.x - bendCenter.x
		local angle = arcLen / radius -- radians = arc length of a circle / radius of that circle
		if (reversed) then
			local ptRadius = radius - pt.fTempPos.y + bendCenter.y
			pt.fPos:Set(0, -ptRadius)
			pt.fPos:Rotate(angle)
		else
			local ptRadius = radius + pt.fTempPos.y - bendCenter.y
			pt.fPos:Set(0, ptRadius)
			pt.fPos:Rotate(-angle)
		end
		pt.fPos.x = pt.fPos.x + circleCenter.x
		pt.fPos.y = pt.fPos.y + circleCenter.y
	end

	moho:AddPointKeyframe(moho.frame)
	mouseEvent.view:DrawMe()
end

function LM_BendPointsH:OnMouseUp(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho:AddPointKeyframe(moho.frame)
	moho:NewKeyframe(CHANNEL_POINT)

	self.selList = nil
	mouseEvent.view:DrawMe()
end

function LM_BendPointsH:OnKeyDown(moho, keyEvent)
	LM_SelectPoints:OnKeyDown(moho, keyEvent)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_BendPointsH.DUMMY = MOHO.MSG_BASE
LM_BendPointsH.RESET = MOHO.MSG_BASE + 1
LM_BendPointsH.SELECTITEM = MOHO.MSG_BASE + 2

function LM_BendPointsH:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Group")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	layout:AddChild(LM.GUI.Button("Reset", self.RESET))
end

function LM_BendPointsH:UpdateWidgets(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	MOHO.BuildGroupMenu(self.menu, mesh, self.SELECTITEM, self.DUMMY)
end

function LM_BendPointsH:HandleMessage(moho, view, msg)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg == self.RESET) then
		if (moho.frame > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()

			for i = 0, mesh:CountPoints() - 1 do
				local pt = mesh:Point(i)
				if (pt.fSelected) then
					pt:SetPos(pt.fAnimPos:GetValue(0), moho.frame)
				end
			end
		end
		moho:NewKeyframe(CHANNEL_POINT)
	elseif (msg >= self.SELECTITEM) then
		mesh:SelectNone()
		local i = msg - self.SELECTITEM
		local name = mesh:Group(i):Name()
		mesh:SelectGroup(name)
		moho:UpdateUI()
	end
end
