-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_BindLayer"

-- **************************************************
-- General information about this script
-- **************************************************

LM_BindLayer = {}

function LM_BindLayer:Name()
	return "Bind Layer"
end

function LM_BindLayer:Version()
	return "5.0"
end

function LM_BindLayer:Description()
	return "Bind this layer to a bone in its parent layer"
end

function LM_BindLayer:Creator()
	return "Lost Marble"
end

function LM_BindLayer:UILabel()
	return("Bind Layer")
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_BindLayer:IsEnabled(moho)
	if (moho.frame > 0) then
		return false
	end
	local skel = moho:ParentSkeleton()
	if (skel == nil) then
		return false
	end
	return true
end

function LM_BindLayer:OnMouseDown(moho, mouseEvent)
	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()
	self:OnBind(moho, mouseEvent)
end

function LM_BindLayer:OnMouseMoved(moho, mouseEvent)
	self:OnBind(moho, mouseEvent)
end

function LM_BindLayer:OnBind(moho, mouseEvent)
	local skel = moho:ParentSkeleton()
	if (skel == nil) then
		return
	end

	local id = -1
	id = mouseEvent.view:PickBone(mouseEvent.pt, mouseEvent.vec, moho.layer:Parent(), true)

	for i = 0, skel:CountBones() - 1 do
		skel:Bone(i).fSelected = (i == id)
	end

	moho.layer:SetLayerParentBone(id)

	moho:UpdateBonePointSelection()
	mouseEvent.view:DrawMe()
	moho:UpdateSelectedChannels()
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************
--[[
LM_BindLayer.AUTOFILL = MOHO.MSG_BASE
LM_BindLayer.AUTOOUTLINE = MOHO.MSG_BASE + 1
LM_BindLayer.autoFill = true
LM_BindLayer.autoOutline = true

function LM_BindLayer:DoLayout(moho, layout)
	self.autoFillCheck = LM.GUI.CheckBox("Auto fill", self.AUTOFILL)
	layout:AddChild(self.autoFillCheck)

	self.autoOutlineCheck = LM.GUI.CheckBox("Auto outline", self.AUTOOUTLINE)
	layout:AddChild(self.autoOutlineCheck)
end

function LM_BindLayer:UpdateWidgets(moho)
	self.autoFillCheck:SetValue(self.autoFill)
	self.autoOutlineCheck:SetValue(self.autoOutline)
end

function LM_BindLayer:HandleMessage(moho, view, msg)
	if (msg == self.AUTOFILL) then
		self.autoFill = self.autoFillCheck:Value()
	elseif (msg == self.AUTOOUTLINE) then
		self.autoOutline = self.autoOutlineCheck:Value()
	end
end
]]
