-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_BindPoints"

-- **************************************************
-- General information about this script
-- **************************************************

LM_BindPoints = {}

function LM_BindPoints:Name()
	return "Bind Points"
end

function LM_BindPoints:Version()
	return "5.0"
end

function LM_BindPoints:Description()
	return "Select/deselect points (press <space> to bind points, hold <shift> to modify current selection, <ctrl> to toggle lasso mode, <alt> to select a new bone)"
end

function LM_BindPoints:Creator()
	return "Lost Marble"
end

function LM_BindPoints:UILabel()
	return("Bind Points")
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_BindPoints:IsEnabled(moho)
	if (moho.frame > 0) then
		return false
	end
	if (moho:CountPoints() < 1) then
		return false
	end
	local skel = moho:ParentSkeleton()
	if (skel == nil) then
		return false
	end
	return true
end

function LM_BindPoints:OnMouseDown(moho, mouseEvent)
	if (mouseEvent.altKey) then
		LM_SelectBone:Select(moho, mouseEvent.pt, mouseEvent.vec, mouseEvent.view, false, false)
		return
	end
	LM_SelectPoints:OnMouseDown(moho, mouseEvent)
end

function LM_BindPoints:OnMouseMoved(moho, mouseEvent)
	if (mouseEvent.altKey) then
		LM_SelectBone:Select(moho, mouseEvent.pt, mouseEvent.vec, mouseEvent.view, false, false)
		return
	end
	LM_SelectPoints:OnMouseMoved(moho, mouseEvent)
end

function LM_BindPoints:OnMouseUp(moho, mouseEvent)
	if (mouseEvent.altKey) then
		return
	end
	LM_SelectPoints:OnMouseUp(moho, mouseEvent)
end

function LM_BindPoints:OnKeyDown(moho, keyEvent)
	if (keyEvent.key == ' ') then
		self:HandleMessage(moho, keyEvent.view, self.BIND)
	elseif (keyEvent.keyCode == LM.GUI.KEY_UP) then
		LM_SelectBone:OnKeyDown(moho, keyEvent)
	elseif (keyEvent.keyCode == LM.GUI.KEY_DOWN) then
		LM_SelectBone:OnKeyDown(moho, keyEvent)
	end
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_BindPoints.BIND = MOHO.MSG_BASE
LM_BindPoints.LASSO = MOHO.MSG_BASE + 1
LM_BindPoints.DUMMY = MOHO.MSG_BASE + 2
LM_BindPoints.SELECTBONE = MOHO.MSG_BASE + 3
LM_BindPoints.SELECTITEM = MOHO.MSG_BASE + 300

function LM_BindPoints:DoLayout(moho, layout)
	self.boneMenu = LM.GUI.Menu("Select Bone")

	self.bonePopup = LM.GUI.PopupMenu(128, false)
	self.bonePopup:SetMenu(self.boneMenu)
	layout:AddChild(self.bonePopup)

	self.pointMenu = LM.GUI.Menu("Select Group")

	self.pointPopup = LM.GUI.PopupMenu(128, false)
	self.pointPopup:SetMenu(self.pointMenu)
	layout:AddChild(self.pointPopup)

	self.bindBut = LM.GUI.Button("Bind Points", self.BIND)
	layout:AddChild(self.bindBut)

	self.lassoCheck = LM.GUI.CheckBox("Lasso mode", self.LASSO)
	layout:AddChild(self.lassoCheck)
end

function LM_BindPoints:UpdateWidgets(moho)
	local skel = moho:ParentSkeleton()
	if (skel == nil) then
		return
	end
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	MOHO.BuildBoneMenu(self.boneMenu, skel, self.SELECTBONE, self.DUMMY)
	MOHO.BuildGroupMenu(self.pointMenu, mesh, self.SELECTITEM, self.DUMMY)
	self.lassoCheck:SetValue(LM_SelectPoints.lassoMode)
end

function LM_BindPoints:HandleMessage(moho, view, msg)
	local skel = moho:ParentSkeleton()
	if (skel == nil) then
		return
	end

	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg == self.BIND) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		local boneID = -1
		for i = 0, skel:CountBones() - 1 do
			if (skel:Bone(i).fSelected) then
				boneID = i
				break
			end
		end
		for i = 0, mesh:CountPoints() - 1 do
			local pt = mesh:Point(i)
			if (pt.fSelected) then -- attach selected points
				pt.fParent = boneID
			elseif (pt.fParent == boneID) then -- detach unselected points that used to be attached
				pt.fParent = -1
			end
		end
	elseif (msg == self.LASSO) then
		LM_SelectPoints.lassoMode = self.lassoCheck:Value()
	elseif (msg >= self.SELECTBONE and msg < self.SELECTITEM) then
		for i = 0, skel:CountBones() - 1 do
			skel:Bone(i).fSelected = (i == msg - self.SELECTBONE)
		end
		moho:UpdateBonePointSelection()
		moho:UpdateUI()
	elseif (msg >= self.SELECTITEM) then
		mesh:SelectNone()
		local i = msg - self.SELECTITEM
		local name = mesh:Group(i):Name()
		mesh:SelectGroup(name)
		moho:UpdateUI()
	end
end
