-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_BoneStrength"

-- **************************************************
-- General information about this script
-- **************************************************

LM_BoneStrength = {}

function LM_BoneStrength:Name()
	return "Bone Strength"
end

function LM_BoneStrength:Version()
	return "5.0"
end

function LM_BoneStrength:Description()
	return "Drag side to side to adjust the strength of the selected bone"
end

function LM_BoneStrength:Creator()
	return "Lost Marble"
end

function LM_BoneStrength:UILabel()
	return("Bone Strength")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_BoneStrength.savedVal = 0

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_BoneStrength:IsEnabled(moho)
	if (moho.frame > 0) then
		return false
	end
	if (moho:CountBones() < 1) then
		return false
	end
	return true
end

function LM_BoneStrength:OnMouseDown(moho, mouseEvent)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()
	if (moho:CountSelectedBones() < 2) then
		for i = 0, skel:CountBones() - 1 do
			skel:Bone(i).fSelected = false
		end
		local id = mouseEvent.view:PickBone(mouseEvent.pt, mouseEvent.vec, moho.layer, false)
		skel:Bone(id).fSelected = true
	end

	self.savedVal = 0
	mouseEvent.view:DrawMe()
	moho:UpdateSelectedChannels()
end

function LM_BoneStrength:OnMouseMoved(moho, mouseEvent)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	local newVal = 10 * (mouseEvent.pt.x - mouseEvent.startPt.x) / mouseEvent.view:Graphics():Width()

	for i = 0, skel:CountBones() - 1 do
		local bone = skel:Bone(i)
		if (bone.fSelected) then
			bone.fStrength = bone.fStrength - self.savedVal + newVal
			if (bone.fStrength < 0) then
				bone.fStrength = 0
			end
		end
	end

	self.savedVal = newVal
	mouseEvent.view:DrawMe()
end

function LM_BoneStrength:OnKeyDown(moho, keyEvent)
	LM_SelectBone:OnKeyDown(moho, keyEvent)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_BoneStrength.CHANGE = MOHO.MSG_BASE
LM_BoneStrength.DUMMY = MOHO.MSG_BASE + 1
LM_BoneStrength.SELECTITEM = MOHO.MSG_BASE + 2

function LM_BoneStrength:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Bone")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	layout:AddChild(LM.GUI.StaticText("Bone strength:"))
	self.strength = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_UFLOAT)
	self.strength:SetWheelInc(0.1)
	layout:AddChild(self.strength)
end

function LM_BoneStrength:UpdateWidgets(moho)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	local selID = skel:SelectedBoneID()

	MOHO.BuildBoneMenu(self.menu, skel, self.SELECTITEM, self.DUMMY)

	if (moho:CountSelectedBones() > 0) then
		local selCount = 0
		local strength = 0
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			if (bone.fSelected) then
				selCount = selCount + 1
				strength = strength + bone.fStrength
			end
		end
		self.strength:SetValue(strength / selCount)
	else
		self.strength:SetValue("")
	end
end

function LM_BoneStrength:HandleMessage(moho, view, msg)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	if (msg == self.CHANGE) then
		if (moho:CountSelectedBones() > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, skel:CountBones() - 1 do
				local bone = skel:Bone(i)
				if (bone.fSelected) then
					bone.fStrength = self.strength:FloatValue()
				end
			end
			moho.layer:UpdateCurFrame()
		end
	elseif (msg >= self.SELECTITEM) then
		for i = 0, skel:CountBones() - 1 do
			skel:Bone(i).fSelected = (i == msg - self.SELECTITEM)
		end
		moho:UpdateUI()
	end
end
