-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Curvature"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Curvature = {}

function LM_Curvature:Name()
	return "Curvature"
end

function LM_Curvature:Version()
	return "5.0"
end

function LM_Curvature:Description()
	return "Drag side to side to adjust the curvature through the selected points"
end

function LM_Curvature:Creator()
	return "Lost Marble"
end

function LM_Curvature:UILabel()
	return("Curvature")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_Curvature.numSel = 0
LM_Curvature.selID = -1
LM_Curvature.savedVal = 0

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_Curvature:IsEnabled(moho)
	if (moho:CountPoints() > 0) then
		return true
	end
	return false
end

function LM_Curvature:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	self.selID = -1
	self.selList = MOHO.SelectedPointList(mesh)
	self.numSel = table.getn(self.selList)

	if (self.numSel < 2) then -- work with only the closest selection
		-- find the closest point here
		self.selID = mesh:ClosestPoint(mouseEvent.startVec)
		if (self.selID >= 0) then
			self.numSel = 1
			mesh:SelectNone()
			local pt = mesh:Point(self.selID)
			pt.fSelected = true
			moho:UpdateSelectedChannels()
		end
	end

	self.savedVal = 0
	mouseEvent.view:DrawMe()
end

function LM_Curvature:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local newVal = (mouseEvent.pt.x - mouseEvent.startPt.x) / mouseEvent.view:Graphics():Width()
	local curve = nil
	local ptPos = -1

	if (self.numSel == 1) then
		local pt = mesh:Point(self.selID)
		for j = 0, pt:CountCurves() - 1 do
			curve, ptPos = pt:Curve(j, ptPos)
			local c = curve:GetCurvature(ptPos, moho.frame)
			c = c - self.savedVal + newVal
			c = LM.Clamp(c, MOHO.PEAKED, 2 / 3)
			curve:SetCurvature(ptPos, c, moho.frame)
		end
	else
		for i, pt in self.selList do
			for j = 0, pt:CountCurves() - 1 do
				curve, ptPos = pt:Curve(j, ptPos)
				local c = curve:GetCurvature(ptPos, moho.frame)
				c = c - self.savedVal + newVal
				c = LM.Clamp(c, MOHO.PEAKED, 2 / 3)
				curve:SetCurvature(ptPos, c, moho.frame)
			end
		end
	end

	self.savedVal = newVal
	mouseEvent.view:DrawMe()
end

function LM_Curvature:OnMouseUp(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho:NewKeyframe(CHANNEL_CURVE)
	self.selList = nil
	mouseEvent.view:DrawMe()
end

function LM_Curvature:OnKeyDown(moho, keyEvent)
	LM_SelectPoints:OnKeyDown(moho, keyEvent)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_Curvature.DUMMY = MOHO.MSG_BASE
LM_Curvature.SELECTITEM = MOHO.MSG_BASE + 1

function LM_Curvature:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Group")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)
end

function LM_Curvature:UpdateWidgets(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	MOHO.BuildGroupMenu(self.menu, mesh, self.SELECTITEM, self.DUMMY)
end

function LM_Curvature:HandleMessage(moho, view, msg)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg >= self.SELECTITEM) then
		mesh:SelectNone()
		local i = msg - self.SELECTITEM
		local name = mesh:Group(i):Name()
		mesh:SelectGroup(name)
		moho:UpdateUI()
	end
end
