-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_HideEdge"

-- **************************************************
-- General information about this script
-- **************************************************

LM_HideEdge = {}

function LM_HideEdge:Name()
	return "Hide Edge"
end

function LM_HideEdge:Version()
	return "5.0"
end

function LM_HideEdge:Description()
	return "Click on an edge to hide/show it"
end

function LM_HideEdge:Creator()
	return "Lost Marble"
end

function LM_HideEdge:UILabel()
	return("Hide Edge")
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_HideEdge:IsEnabled(moho)
	if (moho:CountShapes() > 0) then
		return true
	end
	return false
end

function LM_HideEdge:OnMouseDown(moho, mouseEvent)
	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	self.edgeList = { { -1, -1 } }
	self:EdgePicker(moho, mouseEvent)
end

function LM_HideEdge:OnMouseMoved(moho, mouseEvent)
	self:EdgePicker(moho, mouseEvent)
end

function LM_HideEdge:OnMouseUp(moho, mouseEvent)
	self.edgeList = nil
end

function LM_HideEdge:EdgePicker(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local curveID = -1
	local segID = -1
	local workWithEdge = true

	curveID, segID = mouseEvent.view:PickEdge(mouseEvent.pt, curveID, segID)
	-- the edge list is maintained to make sure that each edge gets processed at most once during any given mouse drag
	for key, value in self.edgeList do
		if (value[1] == curveID and value[2] == segID) then
			workWithEdge = false
			break
		end
	end

	if (workWithEdge) then -- an edge was clicked on, that hasn't been clicked on before
		table.insert(self.edgeList, { curveID, segID })
		local curve = mesh:Curve(curveID)
		local segOn = curve:IsSegmentOn(segID)
		curve:SetSegmentOn(segID, not segOn)
	end
	mouseEvent.view:DrawMe()
end
