-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Noise"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Noise = {}

function LM_Noise:Name()
	return "Noise"
end

function LM_Noise:Version()
	return "5.0"
end

function LM_Noise:Description()
	return "Randomly distort selected points"
end

function LM_Noise:Creator()
	return "Lost Marble"
end

function LM_Noise:UILabel()
	return("Noise")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_Noise.minVec = LM.Vector2:new_local()
LM_Noise.maxVec = LM.Vector2:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_Noise:IsEnabled(moho)
	if (moho:CountSelectedPoints() > 1) then
		return true
	end
	return false
end

function LM_Noise:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	mesh:SelectedBounds(self.minVec, self.maxVec)
	self.ampScale = math.min(self.maxVec.x - self.minVec.x, self.maxVec.y - self.minVec.y) * 0.5
	local v = self.maxVec - self.minVec
	self.freqScale = 5 / v:Mag()

	self.selList = MOHO.SelectedPointList(mesh)
	mesh:PrepMovePoints()

	mouseEvent.view:DrawMe()
end

function LM_Noise:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local amp = self.ampScale * (mouseEvent.pt.x - mouseEvent.startPt.x) / mouseEvent.view:Graphics():Width()
	local posVec = LM.Vector3:new_local()

	for i, pt in self.selList do
		posVec:Set(pt.fTempPos.x, pt.fTempPos.y, 0)
		local noiseVec = moho:NoiseVector(posVec, amp, self.freqScale)
		pt.fPos.x = pt.fTempPos.x + noiseVec.x
		pt.fPos.y = pt.fTempPos.y + noiseVec.y
	end

	moho:AddPointKeyframe(moho.frame)
	mouseEvent.view:DrawMe()
end

function LM_Noise:OnMouseUp(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho:AddPointKeyframe(moho.frame)
	moho:NewKeyframe(CHANNEL_POINT)

	self.selList = nil
	mouseEvent.view:DrawMe()
end

function LM_Noise:OnKeyDown(moho, keyEvent)
	LM_SelectPoints:OnKeyDown(moho, keyEvent)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_Noise.DUMMY = MOHO.MSG_BASE
LM_Noise.RESET = MOHO.MSG_BASE + 1
LM_Noise.SELECTITEM = MOHO.MSG_BASE + 2

function LM_Noise:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Group")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	layout:AddChild(LM.GUI.Button("Reset", self.RESET))
end

function LM_Noise:UpdateWidgets(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	MOHO.BuildGroupMenu(self.menu, mesh, self.SELECTITEM, self.DUMMY)
end

function LM_Noise:HandleMessage(moho, view, msg)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg == self.RESET) then
		if (moho.frame > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()

			for i = 0, mesh:CountPoints() - 1 do
				local pt = mesh:Point(i)
				if (pt.fSelected) then
					pt:SetPos(pt.fAnimPos:GetValue(0), moho.frame)
				end
			end
		end
		moho:NewKeyframe(CHANNEL_POINT)
	elseif (msg >= self.SELECTITEM) then
		mesh:SelectNone()
		local i = msg - self.SELECTITEM
		local name = mesh:Group(i):Name()
		mesh:SelectGroup(name)
		moho:UpdateUI()
	end
end
