-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_OffsetBone"

-- **************************************************
-- General information about this script
-- **************************************************

LM_OffsetBone = {}

function LM_OffsetBone:Name()
	return "Offset Bone"
end

function LM_OffsetBone:Version()
	return "5.0"
end

function LM_OffsetBone:Description()
	return "Offset bone for animation (hold <shift> to constrain)"
end

function LM_OffsetBone:Creator()
	return "Lost Marble"
end

function LM_OffsetBone:UILabel()
	return("Offset Bone")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_OffsetBone.selID = -1
LM_OffsetBone.lastVec = LM.Vector2:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_OffsetBone:IsEnabled(moho)
	if (moho:CountBones() < 1) then
		return false
	end
	if (moho.frame > 0) then
		return false
	end
	return true
end

function LM_OffsetBone:OnMouseDown(moho, mouseEvent)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	self.lastVec:Set(mouseEvent.startVec)

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()
	self.selID = mouseEvent.view:PickBone(mouseEvent.pt, mouseEvent.vec, moho.layer, false)

	for i = 0, skel:CountBones() - 1 do
		local bone = skel:Bone(i)
		bone.fTempPos:Set(bone.fOffset)
		bone.fSelected = (i == self.selID)
	end

	moho.layer:UpdateCurFrame()
	moho:UpdateBonePointSelection()
	mouseEvent.view:DrawMe()
	moho:UpdateSelectedChannels()
end

function LM_OffsetBone:OnMouseMoved(moho, mouseEvent)
	local skel = moho:Skeleton()
	if ((skel == nil) or (self.selID < 0)) then
		return
	end

	local bone = skel:Bone(self.selID)
	bone.fOffset:Set(bone.fTempPos)
	skel:UpdateBoneMatrix(self.selID)

	local offset = mouseEvent.vec
	offset = offset - mouseEvent.startVec

	local boneVec = LM.Vector2:new_local()
	local inverseM = LM.Matrix:new_local()
	local parent = nil

	boneVec:Set(0, 0)
	if (bone.fParent >= 0) then
		parent = skel:Bone(bone.fParent)
		parent.fRestMatrix:Transform(boneVec)
	end
	boneVec = boneVec + offset
	if (parent) then
		inverseM:Set(parent.fRestMatrix)
		inverseM:Invert()
		inverseM:Transform(boneVec)
	end

	if (mouseEvent.shiftKey) then
		if (math.abs(boneVec.x) > math.abs(boneVec.y)) then
			boneVec.y = 0
		else
			boneVec.x = 0
		end
	end

	local v = bone.fOffset + boneVec
	if (moho.gridOn) then
		if (parent) then
			parent.fMovedMatrix:Transform(v)
		end
		moho:SnapToGrid(v)
		if (parent) then
			inverseM:Set(parent.fMovedMatrix)
			inverseM:Invert()
			inverseM:Transform(v)
		end
	end

	bone.fOffset:Set(v)
	moho.layer:UpdateCurFrame()

	mouseEvent.view:DrawMe()
end

function LM_OffsetBone:OnMouseUp(moho, mouseEvent)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	self.selID = -1

	mouseEvent.view:DrawMe()
end

function LM_OffsetBone:OnKeyDown(moho, keyEvent)
	LM_SelectBone:OnKeyDown(moho, keyEvent)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_OffsetBone.CHANGE = MOHO.MSG_BASE
LM_OffsetBone.RESET = MOHO.MSG_BASE + 1
LM_OffsetBone.DUMMY = MOHO.MSG_BASE + 2
LM_OffsetBone.SELECTITEM = MOHO.MSG_BASE + 3

function LM_OffsetBone:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Bone")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	layout:AddChild(LM.GUI.StaticText("Animation offset"))

	layout:AddChild(LM.GUI.StaticText("X:"))
	self.textX = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	self.textX:SetWheelInc(0.1)
	layout:AddChild(self.textX)

	layout:AddChild(LM.GUI.StaticText("Y:"))
	self.textY = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	self.textY:SetWheelInc(0.1)
	layout:AddChild(self.textY)

	self.reset = LM.GUI.Button("Reset", self.RESET)
	layout:AddChild(self.reset)
end

function LM_OffsetBone:UpdateWidgets(moho)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	local selID = skel:SelectedBoneID()

	MOHO.BuildBoneMenu(self.menu, skel, self.SELECTITEM, self.DUMMY)

	if (selID >= 0) then
		local bone = skel:Bone(selID)
		self.textX:SetValue(bone.fOffset.x)
		self.textY:SetValue(bone.fOffset.y)
	else
		self.textX:SetValue("")
		self.textY:SetValue("")
	end
end

function LM_OffsetBone:HandleMessage(moho, view, msg)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	if (msg == self.RESET) then
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			if (bone.fSelected) then
				moho.document:PrepUndo(moho.layer)
				moho.document:SetDirty()
				bone.fOffset:Set(0, 0)
				moho.layer:UpdateCurFrame()
				break
			end
		end
		self:UpdateWidgets(moho)
	elseif (msg == self.CHANGE) then
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			if (bone.fSelected) then
				moho.document:PrepUndo(moho.layer)
				moho.document:SetDirty()
				bone.fOffset.x = self.textX:FloatValue()
				bone.fOffset.y = self.textY:FloatValue()
				moho.layer:UpdateCurFrame()
				break
			end
		end
	elseif (msg >= self.SELECTITEM) then
		for i = 0, skel:CountBones() - 1 do
			skel:Bone(i).fSelected = (i == msg - self.SELECTITEM)
		end
		moho:UpdateUI()
	end
end
