-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_Oval"

-- **************************************************
-- General information about this script
-- **************************************************

LM_Oval = {}

function LM_Oval:Name()
	return "Oval"
end

function LM_Oval:Version()
	return "5.0"
end

function LM_Oval:Description()
	return "Draw an oval (hold <shift> to constrain, <alt> to center)"
end

function LM_Oval:Creator()
	return "Lost Marble"
end

function LM_Oval:UILabel()
	return("Oval")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_Oval.startVec = LM.Vector2:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_Oval:IsEnabled(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return false
	end
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	return true
end

function LM_Oval:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	local n = mesh:CountPoints()

	mesh:SelectNone()

	self.startVec:Set(mouseEvent.vec)
	if (moho.gridOn) then
		moho:SnapToGrid(self.startVec)
	end

	mesh:AddLonePoint(self.startVec, moho.frame)
	mesh:AppendPoint(self.startVec, moho.frame)
	mesh:AppendPoint(self.startVec, moho.frame)
	mesh:AppendPoint(self.startVec, moho.frame)
	mesh:AppendPoint(self.startVec, moho.frame)

	mesh:WeldPoints(n + 4, n, 0)

	-- set all their curvatures appropriately (0.391379 is the magic number for a circle)
	mesh:Point(n):SetCurvature(0.391379, moho.frame)
	mesh:Point(n + 1):SetCurvature(0.391379, moho.frame)
	mesh:Point(n + 2):SetCurvature(0.391379, moho.frame)
	mesh:Point(n + 3):SetCurvature(0.391379, moho.frame)

	mesh:SelectConnected()
	if (LM_Rectangle.autoFill) then
		local shapeID = moho:CreateShape(true)
		if (shapeID >= 0) then
			if (not LM_Rectangle.autoOutline) then
				mesh:Shape(shapeID).fHasOutline = false
			end
		end
	elseif (LM_Rectangle.autoOutline) then
		moho:CreateShape(false)
	end

	mouseEvent.view:DrawMe()
end

function LM_Oval:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local vec = LM.Vector2:new_local()
	vec:Set(mouseEvent.vec)
	if (moho.gridOn) then
		moho:SnapToGrid(vec)
	end

	local x1 = self.startVec.x
	local x2 = vec.x
	local y1 = self.startVec.y
	local y2 = vec.y

	if (mouseEvent.shiftKey) then
		-- constrain width = height
		if (x2 > x1) then
			if (y2 > y1) then
				y2 = y1 + (x2 - x1)
			else
				y2 = y1 - (x2 - x1)
			end
		else
			if (y2 > y1) then
				y2 = y1 - (x2 - x1)
			else
				y2 = y1 + (x2 - x1)
			end
		end
	end
	if (mouseEvent.altKey) then
		-- draw from center point
		x1 = x1 - (x2 - x1)
		y1 = y1 - (y2 - y1)
	end

	local n = mesh:CountPoints()

	local pt = mesh:Point(n - 4)
	pt.fPos.x = (x1 + x2) / 2
	pt.fPos.y = y1
	local pt = mesh:Point(n - 3)
	pt.fPos.x = x2
	pt.fPos.y = (y1 + y2) / 2
	local pt = mesh:Point(n - 2)
	pt.fPos.x = (x1 + x2) / 2
	pt.fPos.y = y2
	local pt = mesh:Point(n - 1)
	pt.fPos.x = x1
	pt.fPos.y = (y1 + y2) / 2

	-- now set the proper curvature
	local f = math.abs(x2 - x1) - math.abs(y2 - y1)
	if (math.abs(f) > 1e-6) then
		local n = mesh:CountPoints()

		if (math.abs(y2 - y1) > 1e-6) then
			f = (x2 - x1) / (y2 - y1)
		else
			f = 1000
		end
		f = math.abs(f)
		f = LM.Clamp(f, 0, 0.66667 / 0.391379) * 0.391379
		mesh:Point(n - 4):SetCurvature(f, moho.frame)
		mesh:Point(n - 2):SetCurvature(f, moho.frame)

		if (math.abs(x2 - x1) > 1e-6) then
			f = (y2 - y1) / (x2 - x1)
		else
			f = 1000
		end
		f = math.abs(f)
		f = LM.Clamp(f, 0, 0.66667 / 0.391379) * 0.391379
		mesh:Point(n - 3):SetCurvature(f, moho.frame)
		mesh:Point(n - 1):SetCurvature(f, moho.frame)
	else
		mesh:Point(n - 4):SetCurvature(0.391379, moho.frame)
		mesh:Point(n - 3):SetCurvature(0.391379, moho.frame)
		mesh:Point(n - 2):SetCurvature(0.391379, moho.frame)
		mesh:Point(n - 1):SetCurvature(0.391379, moho.frame)
	end


	moho:AddPointKeyframe(moho.frame)
	mouseEvent.view:DrawMe()
end

function LM_Oval:OnMouseUp(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho:AddPointKeyframe(moho.frame)
	mouseEvent.view:DrawMe()
	moho:NewKeyframe(CHANNEL_POINT)
	moho:NewKeyframe(CHANNEL_CURVE)
	moho:UpdateSelectedChannels()
end

function LM_Oval:OnKeyDown(moho, keyEvent)
	LM_SelectPoints:OnKeyDown(moho, keyEvent)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_Oval.AUTOFILL = MOHO.MSG_BASE
LM_Oval.AUTOOUTLINE = MOHO.MSG_BASE + 1

function LM_Oval:DoLayout(moho, layout)
	self.autoFillCheck = LM.GUI.CheckBox("Auto fill", self.AUTOFILL)
	layout:AddChild(self.autoFillCheck)

	self.autoOutlineCheck = LM.GUI.CheckBox("Auto outline", self.AUTOOUTLINE)
	layout:AddChild(self.autoOutlineCheck)
end

function LM_Oval:UpdateWidgets(moho)
	self.autoFillCheck:SetValue(LM_Rectangle.autoFill)
	self.autoOutlineCheck:SetValue(LM_Rectangle.autoOutline)
end

function LM_Oval:HandleMessage(moho, view, msg)
	if (msg == self.AUTOFILL) then
		LM_Rectangle.autoFill = self.autoFillCheck:Value()
	elseif (msg == self.AUTOOUTLINE) then
		LM_Rectangle.autoOutline = self.autoOutlineCheck:Value()
	end
end
