-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_PaintBucket"

-- **************************************************
-- General information about this script
-- **************************************************

LM_PaintBucket = {}

function LM_PaintBucket:Name()
	return "Paint Bucket"
end

function LM_PaintBucket:Version()
	return "5.0"
end

function LM_PaintBucket:Description()
	return "Click to fill a closed region with color (click-drag to preview the fill)"
end

function LM_PaintBucket:Creator()
	return "Lost Marble"
end

function LM_PaintBucket:UILabel()
	return("Paint Bucket")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_PaintBucket.dragging = false

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_PaintBucket:IsEnabled(moho)
	if (moho.layer:CurrentAction() ~= "") then
		return false -- creating new objects in the middle of an action can lead to unexpected results
	end
	if (moho:CountEdges() > 0) then
		return true
	end
	return false
end

function LM_PaintBucket:OnMouseDown(moho, mouseEvent)
	self.dragging = true
	mouseEvent.view:FloodSelect(mouseEvent.pt, 1)
	mouseEvent.view:DrawMe()
end

function LM_PaintBucket:OnMouseMoved(moho, mouseEvent)
	mouseEvent.view:FloodSelect(mouseEvent.pt, 1)
	mouseEvent.view:DrawMe()
end

function LM_PaintBucket:OnMouseUp(moho, mouseEvent)
	mouseEvent.view:FloodSelect(mouseEvent.pt, 1)

	if (moho:CountSelectedEdges() > 0) then
		local mesh = moho:Mesh()
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
		local shapeID = moho:CreateShape(true) -- only filled shapes
		if (shapeID >= 0) then
			moho:NewKeyframe(CHANNEL_FILL)
			moho:NewKeyframe(CHANNEL_LINE)
			moho:UpdateSelectedChannels()
		end
	end

	self.dragging = false
	mouseEvent.view:DrawMe()
end

function LM_PaintBucket:OnKeyDown(moho, keyEvent)
	LM_SelectShape:OnKeyDown(moho, keyEvent)
end

function LM_PaintBucket:DrawMe(moho, view)
	if (self.dragging) then
		local g = view:Graphics()
		local matrix = LM.Matrix:new_local()

		moho.layer:GetFullTransform(moho.frame, matrix, moho.document)
		g:Push()
		g:ApplyMatrix(matrix)

		view:DrawPreviewShape()

		g:Pop()
	end
end
