-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_PanTiltCamera"

-- **************************************************
-- General information about this script
-- **************************************************

LM_PanTiltCamera = {}

function LM_PanTiltCamera:Name()
	return "Pan/Tilt Camera"
end

function LM_PanTiltCamera:Version()
	return "5.0"
end

function LM_PanTiltCamera:Description()
	return "Pan (rotate) the camera side to side, and up and down (hold <shift> to constrain)"
end

function LM_PanTiltCamera:Creator()
	return "Lost Marble"
end

function LM_PanTiltCamera:UILabel()
	return("Pan/Tilt Camera")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_PanTiltCamera.startVal = LM.Vector2:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_PanTiltCamera:OnMouseDown(moho, mouseEvent)
	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	self.startVal:Set(moho.document.fCameraPanTilt.value)
	moho.document.fCameraPanTilt:AddKey(moho.frame)

	mouseEvent.view:DrawMe()
end

function LM_PanTiltCamera:OnMouseMoved(moho, mouseEvent)
	local v = LM.Vector2:new_local()

	v:Set(self.startVal)

	v.x = v.x - (math.pi / 2) * (mouseEvent.pt.y - mouseEvent.startPt.y) / mouseEvent.view:Graphics():Height()
	v.y = v.y - math.pi * (mouseEvent.pt.x - mouseEvent.startPt.x) / mouseEvent.view:Graphics():Width()

	v.x = LM.Clamp(v.x, -math.pi / 2, math.pi / 2)

	if (mouseEvent.shiftKey) then
		local dx = math.abs(mouseEvent.pt.x - mouseEvent.startPt.x)
		local dy = math.abs(mouseEvent.pt.y - mouseEvent.startPt.y)
		if (dx > dy * 3) then
			v.x = self.startVal.x
		else
			v.y = self.startVal.y
		end
	end

	moho.document.fCameraPanTilt:SetValue(moho.frame, v)
	moho.document:DepthSort()

	mouseEvent.view:DrawMe()
end

function LM_PanTiltCamera:OnMouseUp(moho, mouseEvent)
	moho:NewKeyframe(CHANNEL_CAMERA_PANTILT)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_PanTiltCamera.CHANGE = MOHO.MSG_BASE
LM_PanTiltCamera.RESET = MOHO.MSG_BASE + 1

function LM_PanTiltCamera:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.StaticText("Pan:"))
	self.pan = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	self.pan:SetWheelInc(5)
	layout:AddChild(self.pan)

	layout:AddChild(LM.GUI.StaticText("Tilt:"))
	self.tilt = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	self.tilt:SetWheelInc(5)
	layout:AddChild(self.tilt)

	layout:AddChild(LM.GUI.Button("Reset", self.RESET))
end

function LM_PanTiltCamera:UpdateWidgets(moho)
	self.pan:SetValue(math.deg(moho.document.fCameraPanTilt.value.y))
	self.tilt:SetValue(math.deg(moho.document.fCameraPanTilt.value.x))
end

function LM_PanTiltCamera:HandleMessage(moho, view, msg)
	local newVal = LM.Vector2:new_local()

	if (msg == self.RESET) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		if (moho.frame == 0) then
			newVal:Set(0, 0)
			moho.document.fCameraPanTilt:SetValue(0, newVal)
		else
			newVal:Set(moho.document.fCameraPanTilt:GetValue(0))
			moho.document.fCameraPanTilt:SetValue(moho.frame, newVal)
		end
		self:UpdateWidgets(moho)
		moho:NewKeyframe(CHANNEL_CAMERA_PANTILT)
	elseif (msg == self.CHANGE) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		newVal.y = math.rad(self.pan:FloatValue())
		newVal.x = math.rad(self.tilt:FloatValue())
		local vec = moho.document.fCameraPanTilt:GetValue(moho.frame) - newVal
		if (vec:Mag() > 0.0001) then
			moho.document.fCameraPanTilt:SetValue(moho.frame, newVal)
			moho:NewKeyframe(CHANNEL_CAMERA_PANTILT)
		end
	end
end
