-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_ParticleLayer"

-- **************************************************
-- General information about this script
-- **************************************************

LM_ParticleLayer = {}

function LM_ParticleLayer:Name()
	return "Particle Layer"
end

function LM_ParticleLayer:Version()
	return "5.0"
end

function LM_ParticleLayer:Description()
	return "Turn a particle layer on or off"
end

function LM_ParticleLayer:Creator()
	return "Lost Marble"
end

function LM_ParticleLayer:UILabel()
	return("Particle Layer")
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_ParticleLayer:IsEnabled(moho)
	local particleLayer = moho:LayerAsParticle(moho.layer)
	if (particleLayer) then
		return true
	end
	return false
end

-- **************************************************
-- Particle dialog
-- **************************************************

local LM_ParticleDialog = {}

function LM_ParticleDialog:new()
	local d = LM.GUI.SimpleDialog("Particle Options", LM_ParticleDialog)
	local l = d:GetLayout()

	l:PushH()

	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Particle count"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Preview particles"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Lifetime (frames)"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Source width"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Source height"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Source depth"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Velocity"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Velocity spread"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Damping"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			d.count = LM.GUI.TextControl(0, "0000", LM_ParticleLayer.DLOG_CHANGE, LM.GUI.FIELD_UINT)
			l:AddChild(d.count, LM.GUI.ALIGN_LEFT)
			d.pCount = LM.GUI.TextControl(0, "0000", LM_ParticleLayer.DLOG_CHANGE, LM.GUI.FIELD_UINT)
			l:AddChild(d.pCount, LM.GUI.ALIGN_LEFT)
			d.lifetime = LM.GUI.TextControl(0, "0000", LM_ParticleLayer.DLOG_CHANGE, LM.GUI.FIELD_UINT)
			l:AddChild(d.lifetime, LM.GUI.ALIGN_LEFT)
			d.width = LM.GUI.TextControl(0, "0000", LM_ParticleLayer.DLOG_CHANGE, LM.GUI.FIELD_UFLOAT)
			l:AddChild(d.width, LM.GUI.ALIGN_LEFT)
			d.height = LM.GUI.TextControl(0, "0000", LM_ParticleLayer.DLOG_CHANGE, LM.GUI.FIELD_UFLOAT)
			l:AddChild(d.height, LM.GUI.ALIGN_LEFT)
			d.depth = LM.GUI.TextControl(0, "0000", LM_ParticleLayer.DLOG_CHANGE, LM.GUI.FIELD_UFLOAT)
			l:AddChild(d.depth, LM.GUI.ALIGN_LEFT)
			d.velocity = LM.GUI.TextControl(0, "0000", LM_ParticleLayer.DLOG_CHANGE, LM.GUI.FIELD_UFLOAT)
			l:AddChild(d.velocity, LM.GUI.ALIGN_LEFT)
			d.vSpread = LM.GUI.TextControl(0, "0000", LM_ParticleLayer.DLOG_CHANGE, LM.GUI.FIELD_UFLOAT)
			l:AddChild(d.vSpread, LM.GUI.ALIGN_LEFT)
			d.damping = LM.GUI.TextControl(0, "0000", LM_ParticleLayer.DLOG_CHANGE, LM.GUI.FIELD_UFLOAT)
			l:AddChild(d.damping, LM.GUI.ALIGN_LEFT)
		l:Pop()
	l:Pop()

	l:AddChild(LM.GUI.Divider(true), LM.GUI.ALIGN_FILL)

	l:PushV()
		d.onStart = LM.GUI.CheckBox("On at start", LM_ParticleLayer.DLOG_CHANGE)
		l:AddChild(d.onStart, LM.GUI.ALIGN_LEFT)
		d.fullStart = LM.GUI.CheckBox("Full speed start", LM_ParticleLayer.DLOG_CHANGE)
		l:AddChild(d.fullStart, LM.GUI.ALIGN_LEFT)
		d.orient = LM.GUI.CheckBox("Orient particles", LM_ParticleLayer.DLOG_CHANGE)
		l:AddChild(d.orient, LM.GUI.ALIGN_LEFT)
		d.freeFloat = LM.GUI.CheckBox("Free floating", LM_ParticleLayer.DLOG_CHANGE)
		l:AddChild(d.freeFloat, LM.GUI.ALIGN_LEFT)
		d.evenlySpaced = LM.GUI.CheckBox("Evenly spaced", LM_ParticleLayer.DLOG_CHANGE)
		l:AddChild(d.evenlySpaced, LM.GUI.ALIGN_LEFT)
		d.randomPlay = LM.GUI.CheckBox("Randomize playback", LM_ParticleLayer.DLOG_CHANGE)
		l:AddChild(d.randomPlay, LM.GUI.ALIGN_LEFT)
	l:Pop()

	l:Pop()

	l:AddChild(LM.GUI.Divider(false), LM.GUI.ALIGN_FILL)

	l:PushH(LM.GUI.ALIGN_CENTER)
		d.dirAngle = LM.GUI.AngleWidget(LM_ParticleLayer.DLOG_CHANGE)
		l:AddChild(d.dirAngle, LM.GUI.ALIGN_CENTER)
		l:AddChild(LM.GUI.StaticText("Direction"), LM.GUI.ALIGN_CENTER)
		l:AddPadding()
		l:AddChild(LM.GUI.StaticText("Spread"), LM.GUI.ALIGN_CENTER)
		d.dirSpread = LM.GUI.TextControl(0, "0000", LM_ParticleLayer.DLOG_CHANGE, LM.GUI.FIELD_UFLOAT)
		l:AddChild(d.dirSpread, LM.GUI.ALIGN_CENTER)
	l:Pop()

	l:PushH(LM.GUI.ALIGN_CENTER)
		d.accelAngle = LM.GUI.AngleWidget(LM_ParticleLayer.DLOG_CHANGE)
		l:AddChild(d.accelAngle, LM.GUI.ALIGN_CENTER)
		l:AddChild(LM.GUI.StaticText("Acceleration"), LM.GUI.ALIGN_CENTER)
		l:AddPadding()
		l:AddChild(LM.GUI.StaticText("Rate"), LM.GUI.ALIGN_CENTER)
		d.accelRate = LM.GUI.TextControl(0, "0000", LM_ParticleLayer.DLOG_CHANGE, LM.GUI.FIELD_UFLOAT)
		l:AddChild(d.accelRate, LM.GUI.ALIGN_CENTER)
	l:Pop()

	l:AddChild(LM.GUI.Button("Reset", LM_ParticleLayer.RESET))

	return d
end

function LM_ParticleDialog:UpdateWidgets()
	if (self.layer == nil) then
		return
	end

	-- ************* Text Fields **************
	local a = 0
	local b = 0

	a, b = self.layer:GetNumParticles(a, b)
	self.count:SetValue(a)
	self.pCount:SetValue(b)

	self.lifetime:SetValue(self.layer:Lifetime())

	local dim = self.layer:SourceDimensions()
	self.width:SetValue(dim.x)
	self.height:SetValue(dim.y)
	self.depth:SetValue(dim.z)

	a, b = self.layer:GetVelocity(a, b)
	self.velocity:SetValue(a)
	self.vSpread:SetValue(b)
	self.damping:SetValue(self.layer:Damping())

	-- ************* Check Boxes **************
	self.onStart:SetValue(self.layer:RunningTrack():GetValue(0))
	self.fullStart:SetValue(self.layer:FullSpeedStart())
	self.orient:SetValue(self.layer:Orientation())
	self.freeFloat:SetValue(self.layer:FreeFloating())
	self.evenlySpaced:SetValue(self.layer:EvenlySpaced())
	self.randomPlay:SetValue(self.layer:RandomStartTime())

	-- ************* Angle Controls **************
	a, b = self.layer:GetDirection(a, b)
	self.dirAngle:SetValue(a)
	self.dirSpread:SetValue(math.deg(b))
	a, b = self.layer:GetAcceleration(a, b)
	self.accelAngle:SetValue(a)
	self.accelRate:SetValue(b)
end

function LM_ParticleDialog:OnOK()
	if (self.layer == nil) then
		return
	end

	self.document:PrepUndo(self.layer)
	self.document:SetDirty()

	-- ************* Text Fields **************
	self.layer:SetNumParticles(self.count:IntValue(), self.pCount:IntValue())
	self.layer:SetLifetime(self.lifetime:IntValue())

	local v = LM.Vector3:new_local()
	v:Set(self.width:FloatValue(), self.height:FloatValue(), self.depth:FloatValue())
	self.layer:SetSourceDimensions(v)

	self.layer:SetVelocity(self.velocity:FloatValue(), self.vSpread:FloatValue())
	self.layer:SetDamping(self.damping:IntValue())

	-- ************* Check Boxes **************
	self.layer:RunningTrack():SetValue(0, self.onStart:Value())
	self.layer:SetFullSpeedStart(self.fullStart:Value())
	self.layer:SetOrientation(self.orient:Value())
	self.layer:SetFreeFloating(self.freeFloat:Value())
	self.layer:SetEvenlySpaced(self.evenlySpaced:Value())
	self.layer:SetRandomStartTime(self.randomPlay:Value())

	-- ************* Angle Controls **************
	self.layer:SetDirection(self.dirAngle:Value(), math.rad(self.dirSpread:FloatValue()))
	self.layer:SetAcceleration(self.accelAngle:Value(), self.accelRate:FloatValue())

	-- ************* Finalize **************
	self.layer:FinalizeSettings()
	MOHO.Redraw()
end

function LM_ParticleDialog:HandleMessage(msg)
	if ((msg == LM_ParticleLayer.DLOG_CHANGE) or (msg == LM.GUI.MSG_CANCEL)) then
		self:OnOK()
	elseif (msg == LM_ParticleLayer.RESET) then
		if (self.layer ~= nil) then
			self.document:PrepUndo(self.layer)
			self.document:SetDirty()

			self.layer:SetNumParticles(100, 20)
			self.layer:SetLifetime(24)
			local v = LM.Vector3:new_local()
			v:Set(0.1, 0.1, 0)
			self.layer:SetSourceDimensions(v)
			self.layer:SetVelocity(2, 0.5)
			self.layer:SetDamping(0)
			self.layer:RunningTrack():SetValue(0, true)
			self.layer:SetFullSpeedStart(true)
			self.layer:SetOrientation(true)
			self.layer:SetFreeFloating(true)
			self.layer:SetEvenlySpaced(false)
			self.layer:SetRandomStartTime(false)
			self.layer:SetDirection(math.rad(90), math.rad(20))
			self.layer:SetAcceleration(math.rad(270), 4)

			self:UpdateWidgets()
			self.layer:FinalizeSettings()
			MOHO.Redraw()
		end
	end
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_ParticleLayer.CHANGE = MOHO.MSG_BASE
LM_ParticleLayer.DLOG_BEGIN = MOHO.MSG_BASE + 1
LM_ParticleLayer.DLOG_CHANGE = MOHO.MSG_BASE + 2
LM_ParticleLayer.RESEED = MOHO.MSG_BASE + 3
LM_ParticleLayer.RESET = MOHO.MSG_BASE + 4

function LM_ParticleLayer:DoLayout(moho, layout)
	self.onOffCheck = LM.GUI.CheckBox("Particles on", self.CHANGE)
	layout:AddChild(self.onOffCheck)

	self.dlog = LM_ParticleDialog:new()
	self.popup = LM.GUI.PopupDialog("Particle Options", false, self.DLOG_BEGIN)
	self.popup:SetDialog(self.dlog)
	layout:AddChild(self.popup)

	layout:AddChild(LM.GUI.Button("Randomize", self.RESEED))
end

function LM_ParticleLayer:UpdateWidgets(moho)
	particleLayer = moho:LayerAsParticle(moho.layer)
	if (particleLayer == nil) then
		return
	end
	self.onOffCheck:SetValue(particleLayer:RunningTrack():GetValue(moho.frame))
end

function LM_ParticleLayer:HandleMessage(moho, view, msg)
	local particleLayer = moho:LayerAsParticle(moho.layer)
	if (particleLayer == nil) then
		return
	end

	if (msg == self.CHANGE) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
		particleLayer:RunningTrack():SetValue(moho.frame, self.onOffCheck:Value())
		moho:SetCurFrame(moho.frame) -- force the particles to be recalculated
		moho:NewKeyframe(CHANNEL_PARTICLE)
	elseif (msg == self.DLOG_BEGIN) then
		self.dlog.document = moho.document
		self.dlog.layer = particleLayer
	elseif (msg == self.RESEED) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
		particleLayer:SetRandomSeed(math.random(1, 16777216))
		moho:SetCurFrame(moho.frame) -- force the particles to be recalculated
	end
end
