-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_PerspectivePointsV"

-- **************************************************
-- General information about this script
-- **************************************************

LM_PerspectivePointsV = {}

function LM_PerspectivePointsV:Name()
	return "Perspective Points V"
end

function LM_PerspectivePointsV:Version()
	return "5.0"
end

function LM_PerspectivePointsV:Description()
	return "Add vertical perspective to selected points"
end

function LM_PerspectivePointsV:Creator()
	return "Lost Marble"
end

function LM_PerspectivePointsV:UILabel()
	return("Perspective Points V")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_PerspectivePointsV.minVec = LM.Vector2:new_local()
LM_PerspectivePointsV.maxVec = LM.Vector2:new_local()
LM_PerspectivePointsV.center = LM.Vector2:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_PerspectivePointsV:IsEnabled(moho)
	if (moho:CountSelectedPoints() > 1) then
		return true
	end
	return false
end

function LM_PerspectivePointsV:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	mesh:SelectedBounds(self.minVec, self.maxVec)
	self.center.x = (self.minVec.x + self.maxVec.x) / 2
	self.center.y = (self.minVec.y + self.maxVec.y) / 2

	self.selList = MOHO.SelectedPointList(mesh)
	mesh:PrepMovePoints()

	mouseEvent.view:DrawMe()
end

function LM_PerspectivePointsV:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local curVec = mouseEvent.vec - mouseEvent.startVec
	local f = -curVec.y / (self.maxVec.y - self.minVec.y)
	local prepAngle = 0
	local angle = 0
	if (f >= 0) then
		prepAngle = math.pi / 2
		angle = LM.Lerp(f, 0.001, 89.9)
	else
		prepAngle = -math.pi / 2
		angle = LM.Lerp(-f, 0.001, 89.9)
	end
	angle = LM.Clamp(angle, 0.001, 89.9)
	angle = math.rad(angle)

	local vec = LM.Vector2:new_local()
	local m = LM.Matrix:new_local()
	local m2 = LM.Matrix:new_local()

	local prepScale = self.maxVec.x - self.center.x
	f = self.maxVec.y - self.center.y
	prepScale = math.max(prepScale, f)
	local dist = 1 / math.tan(angle / 2)
	local nearr = dist - 1
	local farr = dist + 1
	m:Translate(self.center.x, self.center.y, 0)
	m:Scale(prepScale, prepScale, 1)
	m:Rotate(LM.X_AXIS, -prepAngle)
	m:Perspective(angle, nearr, farr, 1)
	m:Translate(0, 0, -dist)
	m:Rotate(LM.X_AXIS, prepAngle)
	m:Scale(1 / prepScale, 1 / prepScale, 1)
	m:Translate(-self.center.x, -self.center.y, 0)
	vec:Set(self.center)
	m:Transform(vec)
	m2:Translate(self.center.x - vec.x, self.center.y - vec.y, 0)
	m2:Multiply(m)

	for i, pt in self.selList do
		pt.fPos:Set(pt.fTempPos)
		m2:Transform(pt.fPos)
	end

	moho:AddPointKeyframe(moho.frame)
	mouseEvent.view:DrawMe()
end

function LM_PerspectivePointsV:OnMouseUp(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho:AddPointKeyframe(moho.frame)
	moho:NewKeyframe(CHANNEL_POINT)

	self.selList = nil
	mouseEvent.view:DrawMe()
end

function LM_PerspectivePointsV:OnKeyDown(moho, keyEvent)
	LM_SelectPoints:OnKeyDown(moho, keyEvent)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_PerspectivePointsV.DUMMY = MOHO.MSG_BASE
LM_PerspectivePointsV.RESET = MOHO.MSG_BASE + 1
LM_PerspectivePointsV.SELECTITEM = MOHO.MSG_BASE + 2

function LM_PerspectivePointsV:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Group")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	layout:AddChild(LM.GUI.Button("Reset", self.RESET))
end

function LM_PerspectivePointsV:UpdateWidgets(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	MOHO.BuildGroupMenu(self.menu, mesh, self.SELECTITEM, self.DUMMY)
end

function LM_PerspectivePointsV:HandleMessage(moho, view, msg)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg == self.RESET) then
		if (moho.frame > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()

			for i = 0, mesh:CountPoints() - 1 do
				local pt = mesh:Point(i)
				if (pt.fSelected) then
					pt:SetPos(pt.fAnimPos:GetValue(0), moho.frame)
				end
			end
		end
		moho:NewKeyframe(CHANNEL_POINT)
	elseif (msg >= self.SELECTITEM) then
		mesh:SelectNone()
		local i = msg - self.SELECTITEM
		local name = mesh:Group(i):Name()
		mesh:SelectGroup(name)
		moho:UpdateUI()
	end
end
