-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_RollCamera"

-- **************************************************
-- General information about this script
-- **************************************************

LM_RollCamera = {}

function LM_RollCamera:Name()
	return "Roll Camera"
end

function LM_RollCamera:Version()
	return "5.0"
end

function LM_RollCamera:Description()
	return "Roll the camera (hold <shift> to constrain)"
end

function LM_RollCamera:Creator()
	return "Lost Marble"
end

function LM_RollCamera:UILabel()
	return("Roll Camera")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_RollCamera.startVal = 0
LM_RollCamera.lastVec = LM.Vector2:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_RollCamera:OnMouseDown(moho, mouseEvent)
	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	self.startVal = moho.document.fCameraRoll.value
	moho.document.fCameraRoll:AddKey(moho.frame)

	mouseEvent.view:Graphics():ScreenToWorld(mouseEvent.pt, self.lastVec)

	mouseEvent.view:DrawMe()
end

function LM_RollCamera:OnMouseMoved(moho, mouseEvent)
	local angle = self.startVal
	local nextVec = LM.Vector2:new_local()
	local v1 = LM.Vector2:new_local()
	local v2 = LM.Vector2:new_local()

	mouseEvent.view:Graphics():ScreenToWorld(mouseEvent.pt, nextVec)
	v1:Set(self.lastVec)
	v2:Set(nextVec)
	v2:Rotate(-math.atan2(v1.y, v1.x))
	angle = angle + math.atan2(v2.y, v2.x)
	self.startVal = angle

	if (mouseEvent.shiftKey) then
		angle = angle / (math.pi / 4)
		angle = LM.Round(angle)
		angle = (math.pi / 4) * angle
	end

	self.lastVec:Set(nextVec)
	moho.document.fCameraRoll:SetValue(moho.frame, angle)
	moho.document:DepthSort()

	mouseEvent.view:DrawMe()
end

function LM_RollCamera:OnMouseUp(moho, mouseEvent)
	moho:NewKeyframe(CHANNEL_CAMERA_ROLL)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_RollCamera.CHANGE = MOHO.MSG_BASE
LM_RollCamera.RESET = MOHO.MSG_BASE + 1

function LM_RollCamera:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.StaticText("Roll:"))
	self.rotZ = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	self.rotZ:SetWheelInc(5)
	layout:AddChild(self.rotZ)

	layout:AddChild(LM.GUI.Button("Reset", self.RESET))
end

function LM_RollCamera:UpdateWidgets(moho)
	self.rotZ:SetValue(math.deg(moho.document.fCameraRoll.value))
end

function LM_RollCamera:HandleMessage(moho, view, msg)
	local newVal = 0

	if (msg == self.RESET) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		if (moho.frame == 0) then
			newVal = 0
			moho.document.fCameraRoll:SetValue(0, newVal)
		else
			newVal = moho.document.fCameraRoll:GetValue(0)
			moho.document.fCameraRoll:SetValue(moho.frame, newVal)
		end
		self:UpdateWidgets(moho)
		moho:NewKeyframe(CHANNEL_CAMERA_ROLL)
	elseif (msg == self.CHANGE) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		newVal = math.rad(self.rotZ:FloatValue())
		local d = moho.document.fCameraRoll:GetValue(moho.frame) - newVal
		if (math.abs(d) > 0.0001) then
			moho.document.fCameraRoll:SetValue(moho.frame, newVal)
			moho:NewKeyframe(CHANNEL_CAMERA_ROLL)
		end
	end
end
