-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_RotateLayer"

-- **************************************************
-- General information about this script
-- **************************************************

LM_RotateLayer = {}

function LM_RotateLayer:Name()
	return "Rotate Layer Z"
end

function LM_RotateLayer:Version()
	return "5.0"
end

function LM_RotateLayer:Description()
	return "Rotate entire layer (hold <shift> to constrain)"
end

function LM_RotateLayer:Creator()
	return "Lost Marble"
end

function LM_RotateLayer:UILabel()
	return("Rotate Layer Z")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_RotateLayer.startAngle = 0
LM_RotateLayer.matrix = LM.Matrix:new_local()
LM_RotateLayer.lastVec = LM.Vector2:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_RotateLayer:OnMouseDown(moho, mouseEvent)
	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	self.startAngle = moho.layer.fRotationZ.value
	moho.layer.fRotationZ:AddKey(moho.frame)

	moho.layer:GetFullTransform(moho.frame, self.matrix, moho.document)
	self.lastVec:Set(mouseEvent.view:Point2Vec(mouseEvent.pt, self.matrix))

	mouseEvent.view:DrawMe()
end

function LM_RotateLayer:OnMouseMoved(moho, mouseEvent)
	local nextVec = mouseEvent.view:Point2Vec(mouseEvent.pt, self.matrix)
	local angle = self.startAngle
	local origin = moho.layer:Origin()
	local v1 = self.lastVec - origin
	local v2 = nextVec - origin
	v2:Rotate(-math.atan2(v1.y, v1.x))

	local dAngle = math.atan2(v2.y, v2.x)
	local flipH = moho.layer.fFlipH.value
	local flipV = moho.layer.fFlipV.value
	if ((flipH and not flipV) or (flipV and not flipH)) then
		dAngle = -dAngle
	end

	angle = angle + dAngle
	self.startAngle = angle
	if (mouseEvent.shiftKey) then
		angle = angle / (math.pi / 4)
		angle = (math.pi / 4) * LM.Round(angle)
	end

	self.lastVec:Set(nextVec)
	moho.layer.fRotationZ:SetValue(moho.frame, angle)
	moho.document:DepthSort()

	mouseEvent.view:DrawMe()
end

function LM_RotateLayer:OnMouseUp(moho, mouseEvent)
	moho:NewKeyframe(CHANNEL_LAYER_ROT_Z)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_RotateLayer.CHANGE = MOHO.MSG_BASE
LM_RotateLayer.RESET = MOHO.MSG_BASE + 1

function LM_RotateLayer:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.StaticText("Z rotation:"))
	self.rotZ = LM.GUI.TextControl(0, "000.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	self.rotZ:SetWheelInc(5)
	layout:AddChild(self.rotZ)

	layout:AddChild(LM.GUI.Button("Reset", self.RESET))
end

function LM_RotateLayer:UpdateWidgets(moho)
	self.rotZ:SetValue(math.deg(moho.layer.fRotationZ.value))
end

function LM_RotateLayer:HandleMessage(moho, view, msg)
	local newVal = 0

	if (msg == self.RESET) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		if (moho.frame == 0) then
			newVal = 0
			moho.layer.fRotationZ:SetValue(0, newVal)
		else
			newVal = moho.layer.fRotationZ:GetValue(0)
			moho.layer.fRotationZ:SetValue(moho.frame, newVal)
		end
		self:UpdateWidgets(moho)
		moho:NewKeyframe(CHANNEL_LAYER_ROT_Z)
	elseif (msg == self.CHANGE) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		newVal = math.rad(self.rotZ:FloatValue())
		local d = moho.layer.fRotationZ:GetValue(moho.frame) - newVal
		if (math.abs(d) > 0.0001) then
			moho.layer.fRotationZ:SetValue(moho.frame, newVal)
			moho:NewKeyframe(CHANNEL_LAYER_ROT_Z)
		end
	end
end
