-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_RotateLayerX"

-- **************************************************
-- General information about this script
-- **************************************************

LM_RotateLayerX = {}

function LM_RotateLayerX:Name()
	return "Rotate Layer X"
end

function LM_RotateLayerX:Version()
	return "5.0"
end

function LM_RotateLayerX:Description()
	return "Rotate layer around the X axis (3D rotation) (hold <shift> to constrain)"
end

function LM_RotateLayerX:Creator()
	return "Lost Marble"
end

function LM_RotateLayerX:UILabel()
	return("Rotate Layer X")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_RotateLayerX.startAngle = 0

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_RotateLayerX:OnMouseDown(moho, mouseEvent)
	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	self.startAngle = moho.layer.fRotationX.value
	moho.layer.fRotationX:AddKey(moho.frame)

	mouseEvent.view:DrawMe()
end

function LM_RotateLayerX:OnMouseMoved(moho, mouseEvent)
	local angle = self.startAngle
	angle = angle + math.pi * (mouseEvent.pt.y - mouseEvent.startPt.y) / mouseEvent.view:Graphics():Height()

	if (mouseEvent.shiftKey) then
		angle = angle / (math.pi / 4)
		angle = (math.pi / 4) * LM.Round(angle)
	end

	moho.layer.fRotationX:SetValue(moho.frame, angle)
	moho.document:DepthSort()

	mouseEvent.view:DrawMe()
end

function LM_RotateLayerX:OnMouseUp(moho, mouseEvent)
	moho:NewKeyframe(CHANNEL_LAYER_ROT_X)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_RotateLayerX.CHANGE = MOHO.MSG_BASE
LM_RotateLayerX.RESET = MOHO.MSG_BASE + 1

function LM_RotateLayerX:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.StaticText("X rotation:"))
	self.rotX = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	self.rotX:SetWheelInc(5)
	layout:AddChild(self.rotX)

	layout:AddChild(LM.GUI.Button("Reset", self.RESET))
end

function LM_RotateLayerX:UpdateWidgets(moho)
	self.rotX:SetValue(math.deg(moho.layer.fRotationX.value))
end

function LM_RotateLayerX:HandleMessage(moho, view, msg)
	local newVal = 0

	if (msg == self.RESET) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		if (moho.frame == 0) then
			newVal = 0
			moho.layer.fRotationX:SetValue(0, newVal)
		else
			newVal = moho.layer.fRotationX:GetValue(0)
			moho.layer.fRotationX:SetValue(moho.frame, newVal)
		end
		self:UpdateWidgets(moho)
		moho:NewKeyframe(CHANNEL_LAYER_ROT_X)
	elseif (msg == self.CHANGE) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		newVal = math.rad(self.rotX:FloatValue())
		local d = moho.layer.fRotationX:GetValue(moho.frame) - newVal
		if (math.abs(d) > 0.0001) then
			moho.layer.fRotationX:SetValue(moho.frame, newVal)
			moho:NewKeyframe(CHANNEL_LAYER_ROT_X)
		end
	end
end
