-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_RotateLayerY"

-- **************************************************
-- General information about this script
-- **************************************************

LM_RotateLayerY = {}

function LM_RotateLayerY:Name()
	return "Rotate Layer Y"
end

function LM_RotateLayerY:Version()
	return "5.0"
end

function LM_RotateLayerY:Description()
	return "Rotate layer around the Y axis (3D rotation) (hold <shift> to constrain)"
end

function LM_RotateLayerY:Creator()
	return "Lost Marble"
end

function LM_RotateLayerY:UILabel()
	return("Rotate Layer Y")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_RotateLayerY.startAngle = 0

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_RotateLayerY:OnMouseDown(moho, mouseEvent)
	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	self.startAngle = moho.layer.fRotationY.value
	moho.layer.fRotationY:AddKey(moho.frame)

	mouseEvent.view:DrawMe()
end

function LM_RotateLayerY:OnMouseMoved(moho, mouseEvent)
	local angle = self.startAngle
	angle = angle + math.pi * (mouseEvent.pt.x - mouseEvent.startPt.x) / mouseEvent.view:Graphics():Width()

	if (mouseEvent.shiftKey) then
		angle = angle / (math.pi / 4)
		angle = (math.pi / 4) * LM.Round(angle)
	end

	moho.layer.fRotationY:SetValue(moho.frame, angle)
	moho.document:DepthSort()

	mouseEvent.view:DrawMe()
end

function LM_RotateLayerY:OnMouseUp(moho, mouseEvent)
	moho:NewKeyframe(CHANNEL_LAYER_ROT_Y)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_RotateLayerY.CHANGE = MOHO.MSG_BASE
LM_RotateLayerY.RESET = MOHO.MSG_BASE + 1

function LM_RotateLayerY:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.StaticText("Y rotation:"))
	self.rotY = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	self.rotY:SetWheelInc(5)
	layout:AddChild(self.rotY)

	layout:AddChild(LM.GUI.Button("Reset", self.RESET))
end

function LM_RotateLayerY:UpdateWidgets(moho)
	self.rotY:SetValue(math.deg(moho.layer.fRotationY.value))
end

function LM_RotateLayerY:HandleMessage(moho, view, msg)
	local newVal = 0

	if (msg == self.RESET) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		if (moho.frame == 0) then
			newVal = 0
			moho.layer.fRotationY:SetValue(0, newVal)
		else
			newVal = moho.layer.fRotationY:GetValue(0)
			moho.layer.fRotationY:SetValue(moho.frame, newVal)
		end
		self:UpdateWidgets(moho)
		moho:NewKeyframe(CHANNEL_LAYER_ROT_Y)
	elseif (msg == self.CHANGE) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		newVal = math.rad(self.rotY:FloatValue())
		local d = moho.layer.fRotationY:GetValue(moho.frame) - newVal
		if (math.abs(d) > 0.0001) then
			moho.layer.fRotationY:SetValue(moho.frame, newVal)
			moho:NewKeyframe(CHANNEL_LAYER_ROT_Y)
		end
	end
end
