-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_ScaleBone"

-- **************************************************
-- General information about this script
-- **************************************************

LM_ScaleBone = {}

function LM_ScaleBone:Name()
	return "Scale Bone"
end

function LM_ScaleBone:Version()
	return "5.0"
end

function LM_ScaleBone:Description()
	return "Scale bone"
end

function LM_ScaleBone:Creator()
	return "Lost Marble"
end

function LM_ScaleBone:UILabel()
	return("Scale Bone")
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_ScaleBone:IsEnabled(moho)
	if (moho.frame < 1) then
		return false
	end
	if (moho:CountBones() < 1) then
		return false
	end
	return true
end

function LM_ScaleBone:OnMouseDown(moho, mouseEvent)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()
	if (moho:CountSelectedBones() < 2) then
		for i = 0, skel:CountBones() - 1 do
			skel:Bone(i).fSelected = false
		end
		local id = mouseEvent.view:PickBone(mouseEvent.pt, mouseEvent.vec, moho.layer, false)
		skel:Bone(id).fSelected = true
	end

	for i = 0, skel:CountBones() - 1 do
		local bone = skel:Bone(i)

		if (bone.fSelected) then
			if (moho.frame == 0) then
				bone.fTempScale = bone.fAnimScale:GetValue(0)
			else
				bone.fTempScale = bone.fScale
			end
		end
	end

	moho:UpdateBonePointSelection()
	mouseEvent.view:DrawMe()
	moho:UpdateSelectedChannels()
end

function LM_ScaleBone:OnMouseMoved(moho, mouseEvent)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	for i = 0, skel:CountBones() - 1 do
		local bone = skel:Bone(i)
		if (bone.fSelected) then
			local scaleFactor = (mouseEvent.pt.x - mouseEvent.startPt.x) / 100
			if (scaleFactor < 0) then
				scaleFactor = 1 / (-scaleFactor + 1)
			else
				scaleFactor = scaleFactor + 1
			end
			bone.fScale = bone.fTempScale * scaleFactor

			bone.fAnimScale:SetValue(moho.frame, bone.fScale)
		end
	end

	moho.layer:UpdateCurFrame()
	mouseEvent.view:DrawMe()
end

function LM_ScaleBone:OnMouseUp(moho, mouseEvent)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	if ((moho.frame > 0) and (moho:CountSelectedBones() > 0)) then
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			if (bone.fSelected) then
				bone.fAnimScale:SetValue(moho.frame, bone.fScale)
			end
		end
		moho.layer:UpdateCurFrame()
		moho:NewKeyframe(CHANNEL_BONE_S)
	end

	mouseEvent.view:DrawMe()
end

function LM_ScaleBone:OnKeyDown(moho, keyEvent)
	LM_SelectBone:OnKeyDown(moho, keyEvent)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_ScaleBone.CHANGE = MOHO.MSG_BASE
LM_ScaleBone.RESET = MOHO.MSG_BASE + 1
LM_ScaleBone.DUMMY = MOHO.MSG_BASE + 2
LM_ScaleBone.SELECTITEM = MOHO.MSG_BASE + 3

function LM_ScaleBone:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Bone")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	layout:AddChild(LM.GUI.StaticText("Scale:"))
	self.scale = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	self.scale:SetWheelInc(0.1)
	layout:AddChild(self.scale)

	self.reset = LM.GUI.Button("Reset", self.RESET)
	layout:AddChild(self.reset)
end

function LM_ScaleBone:UpdateWidgets(moho)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	MOHO.BuildBoneMenu(self.menu, skel, self.SELECTITEM, self.DUMMY)

	if (moho:CountSelectedBones() > 0) then
		local selCount = 0
		local scale = 0
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			if (bone.fSelected) then
				selCount = selCount + 1
				scale = scale + bone.fScale
			end
		end
		self.scale:SetValue(scale / selCount)
	else
		self.scale:SetValue("")
	end

	if (moho.frame == 0) then
		self.reset:Enable(false)
	else
		self.reset:Enable(true)
	end
end

function LM_ScaleBone:HandleMessage(moho, view, msg)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	if (msg == self.RESET) then
		if (moho:CountSelectedBones() > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, skel:CountBones() - 1 do
				local bone = skel:Bone(i)
				if (bone.fSelected) then
					bone.fAnimScale:SetValue(moho.frame, bone.fAnimScale:GetValue(0))
				end
			end
			moho.layer:UpdateCurFrame()
			moho:NewKeyframe(CHANNEL_BONE_S)
			self:UpdateWidgets(moho)
		end
	elseif (msg == self.CHANGE) then
		if (moho:CountSelectedBones() > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, skel:CountBones() - 1 do
				local bone = skel:Bone(i)
				if (bone.fSelected) then
					bone.fScale = self.scale:FloatValue()
					bone.fAnimScale:SetValue(moho.frame, bone.fScale)
				end
			end
			moho.layer:UpdateCurFrame()
			moho:NewKeyframe(CHANNEL_BONE_S)
		end
	elseif (msg >= self.SELECTITEM) then
		for i = 0, skel:CountBones() - 1 do
			skel:Bone(i).fSelected = (i == msg - self.SELECTITEM)
		end
		moho:UpdateUI()
	end
end
