-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_SelectShape"

-- **************************************************
-- General information about this script
-- **************************************************

LM_SelectShape = {}

function LM_SelectShape:Name()
	return "Select Shape"
end

function LM_SelectShape:Version()
	return "5.0"
end

function LM_SelectShape:Description()
	return "Click on a shape to select it (hold <alt> for color picker, <alt>+<ctrl> for color pusher)"
end

function LM_SelectShape:Creator()
	return "Lost Marble"
end

function LM_SelectShape:UILabel()
	return("Select Shape")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_SelectShape.prevMousePt = LM.Point:new_local()
LM_SelectShape.prevSelID = -1

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_SelectShape:IsEnabled(moho)
	if (moho:CountShapes() > 0) then
		return true
	end
	return false
end

function LM_SelectShape:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	self.prevMousePt:Set(mouseEvent.pt)
	self.prevSelID = mouseEvent.view:PickShape(mouseEvent.pt)

	if (mouseEvent.altKey) then -- color/style picker
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
		if (self.prevSelID >= 0) then
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected and i ~= self.prevSelID) then
					local sourceShape = mesh:Shape(self.prevSelID)
					local destShape = mesh:Shape(i)
					if (mouseEvent.ctrlKey) then
						sourceShape:CopyStyleProperties(destShape)
					else
						destShape:CopyStyleProperties(sourceShape)
					end
					break
				end
			end
		end
	else -- shape picker
		for i = 0, mesh:CountShapes() - 1 do
			mesh:Shape(i).fSelected = false
		end
		if (self.prevSelID >= 0) then
			mesh:Shape(self.prevSelID).fSelected = true
		end
		moho:UpdateSelectedChannels()
	end

	mouseEvent.view:DrawMe()
end

function LM_SelectShape:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	self.prevMousePt:Set(mouseEvent.pt)
	self.prevSelID = mouseEvent.view:PickShape(mouseEvent.pt)

	if (mouseEvent.altKey) then -- color/style picker
		if (self.prevSelID >= 0) then
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected and i ~= self.prevSelID) then
					local sourceShape = mesh:Shape(self.prevSelID)
					local destShape = mesh:Shape(i)
					if (mouseEvent.ctrlKey) then
						sourceShape:CopyStyleProperties(destShape)
					else
						destShape:CopyStyleProperties(sourceShape)
					end
					break
				end
			end
		end
	else -- shape picker
		for i = 0, mesh:CountShapes() - 1 do
			mesh:Shape(i).fSelected = false
		end
		if (self.prevSelID >= 0) then
			mesh:Shape(self.prevSelID).fSelected = true
		end
		moho:UpdateSelectedChannels()
	end

	mouseEvent.view:DrawMe()
end

function LM_SelectShape:OnKeyDown(moho, keyEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (keyEvent.keyCode == LM.GUI.KEY_UP) then
		if (keyEvent.ctrlKey) then -- select higher in the stacking order
			self.prevSelID = self.prevSelID + 1
			local id = keyEvent.view:PickShape(self.prevMousePt, self.prevSelID)
			for i = 0, mesh:CountShapes() - 1 do
				mesh:Shape(i).fSelected = false
			end
			if (id >= 0) then
				mesh:Shape(id).fSelected = true
			end
			moho:UpdateSelectedChannels()
		elseif (keyEvent.shiftKey) then -- raise shape to top
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected) then
					mesh:RaiseShape(i, true)
					break
				end
			end
		else -- raise shape
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected) then
					mesh:RaiseShape(i, false)
					break
				end
			end
		end
		keyEvent.view:DrawMe()
	elseif (keyEvent.keyCode == LM.GUI.KEY_DOWN) then
		if (keyEvent.ctrlKey) then -- select lower in the stacking order
			self.prevSelID = self.prevSelID - 1
			id = keyEvent.view:PickShape(self.prevMousePt, self.prevSelID)
			for i = 0, mesh:CountShapes() - 1 do
				mesh:Shape(i).fSelected = false
			end
			if (id >= 0) then
				mesh:Shape(id).fSelected = true
			end
			moho:UpdateSelectedChannels()
		elseif (keyEvent.shiftKey) then -- lower shape to bottom
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected) then
					mesh:LowerShape(i, true)
					break
				end
			end
		else -- lower shape
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()
			for i = 0, mesh:CountShapes() - 1 do
				if (mesh:Shape(i).fSelected) then
					mesh:LowerShape(i, false)
					break
				end
			end
		end
		keyEvent.view:DrawMe()
	elseif ((keyEvent.keyCode == LM.GUI.KEY_DELETE) or (keyEvent.keyCode == LM.GUI.KEY_BACKSPACE)) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
		local i = 0
		while i < mesh:CountShapes() do
			if (mesh:Shape(i).fSelected) then
				mesh:DeleteShape(i)
			else
				i = i + 1
			end
		end
		keyEvent.view:DrawMe()
	end
end
