-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_SetOrigin"

-- **************************************************
-- General information about this script
-- **************************************************

LM_SetOrigin = {}

function LM_SetOrigin:Name()
	return "Set Origin"
end

function LM_SetOrigin:Version()
	return "5.0"
end

function LM_SetOrigin:Description()
	return "Click to set the origin of this layer (the point the layer rotates around)"
end

function LM_SetOrigin:Creator()
	return "Lost Marble"
end

function LM_SetOrigin:UILabel()
	return("Set Origin")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_SetOrigin.matrix = LM.Matrix:new_local()
LM_SetOrigin.layerSettingsWnd = nil

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_SetOrigin:OnMouseDown(moho, mouseEvent)
	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	self:SetOrigin(moho, mouseEvent)
	mouseEvent.view:DrawMe()
end

function LM_SetOrigin:OnMouseMoved(moho, mouseEvent)
	self:SetOrigin(moho, mouseEvent)
	mouseEvent.view:DrawMe()
end

function LM_SetOrigin:SetOrigin(moho, mouseEvent)
	local matrix = LM.Matrix:new_local()
	local beforeVec = LM.Vector3:new_local()
	local afterVec = LM.Vector3:new_local()

	moho.layer:GetFullTransform(moho.frame, matrix, nil)
	matrix:Transform(beforeVec)

	moho.layer:GetFullTransform(moho.frame, matrix, moho.document)
	moho.layer:SetOrigin(mouseEvent.view:Point2Vec(mouseEvent.pt, matrix))

	moho.layer:GetFullTransform(moho.frame, matrix, nil)
	matrix:Transform(afterVec)

	local newLayerPos = moho.layer.fTranslation.value + beforeVec - afterVec
	local v = newLayerPos - moho.layer.fTranslation.value
	if (v:Mag() > 0.000001) then
		moho.layer.fTranslation:SetValue(moho.frame, newLayerPos)
		moho:NewKeyframe(CHANNEL_LAYER_T)
	end

	moho.document:DepthSort()
end

-- **************************************************
-- Layer Settings dialog
-- **************************************************

local LM_LayerSettingsDialog = {}

LM_LayerSettingsDialog.CHANGE = MOHO.MSG_BASE
LM_LayerSettingsDialog.RESET = MOHO.MSG_BASE + 1

function LM_LayerSettingsDialog:new()
	local d = LM.GUI.SimpleDialog("Layer Settings", LM_LayerSettingsDialog)
	local l = d:GetLayout()

	l:PushH()
		l:PushV()
			l:AddChild(LM.GUI.StaticText("Position (X, Y, Z)"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Scale (X, Y, Z)"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Rotation (X, Y, Z)"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Shear (X, Y, Z)"), LM.GUI.ALIGN_LEFT)
			l:AddChild(LM.GUI.StaticText("Origin (X, Y)"), LM.GUI.ALIGN_LEFT)
		l:Pop()
		l:PushV()
			l:PushH()
				d.pX = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_FLOAT)
				l:AddChild(d.pX, LM.GUI.ALIGN_LEFT)
				d.pY = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_FLOAT)
				l:AddChild(d.pY, LM.GUI.ALIGN_LEFT)
				d.pZ = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_FLOAT)
				l:AddChild(d.pZ, LM.GUI.ALIGN_LEFT)
			l:Pop()
			l:PushH()
				d.sX = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_FLOAT)
				l:AddChild(d.sX, LM.GUI.ALIGN_LEFT)
				d.sY = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_FLOAT)
				l:AddChild(d.sY, LM.GUI.ALIGN_LEFT)
				d.sZ = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_FLOAT)
				l:AddChild(d.sZ, LM.GUI.ALIGN_LEFT)
			l:Pop()
			l:PushH()
				d.rX = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_FLOAT)
				l:AddChild(d.rX, LM.GUI.ALIGN_LEFT)
				d.rY = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_FLOAT)
				l:AddChild(d.rY, LM.GUI.ALIGN_LEFT)
				d.rZ = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_FLOAT)
				l:AddChild(d.rZ, LM.GUI.ALIGN_LEFT)
			l:Pop()
			l:PushH()
				d.shX = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_FLOAT)
				l:AddChild(d.shX, LM.GUI.ALIGN_LEFT)
				d.shY = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_FLOAT)
				l:AddChild(d.shY, LM.GUI.ALIGN_LEFT)
				d.shZ = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_FLOAT)
				l:AddChild(d.shZ, LM.GUI.ALIGN_LEFT)
			l:Pop()
			l:PushH()
				d.oX = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_FLOAT)
				l:AddChild(d.oX, LM.GUI.ALIGN_LEFT)
				d.oY = LM.GUI.TextControl(0, "00.0000", 0, LM.GUI.FIELD_FLOAT)
				l:AddChild(d.oY, LM.GUI.ALIGN_LEFT)
			l:Pop()
		l:Pop()
	l:Pop()

	l:AddChild(LM.GUI.Button("Reset", LM_LayerSettingsDialog.RESET))

	return d
end

function LM_LayerSettingsDialog:Update(moho)
	if (moho.layer == nil) then
		return
	end

	local layer = moho.layer

	self.pX:SetValue(layer.fTranslation.value.x)
	self.pY:SetValue(layer.fTranslation.value.y)
	self.pZ:SetValue(layer.fTranslation.value.z)

	self.sX:SetValue(layer.fScale.value.x)
	self.sY:SetValue(layer.fScale.value.y)
	self.sZ:SetValue(layer.fScale.value.z)

	self.rX:SetValue(layer.fRotationX.value)
	self.rY:SetValue(layer.fRotationY.value)
	self.rZ:SetValue(layer.fRotationZ.value)

	self.shX:SetValue(layer.fShear.value.x)
	self.shY:SetValue(layer.fShear.value.y)
	self.shZ:SetValue(layer.fShear.value.z)

	local origin = layer:Origin()
	self.oX:SetValue(origin.x)
	self.oY:SetValue(origin.y)
end

function LM_LayerSettingsDialog:OnOK()
	LM_SetOrigin.layerSettingsWnd = nil -- mark the window closed
end

function LM_LayerSettingsDialog_Update(moho)
	if (LM_SetOrigin.layerSettingsWnd) then
		LM_SetOrigin.layerSettingsWnd:Update(moho)
	end
end

-- register the layer window to be updated when changes are made
table.insert(MOHO.UpdateTable, LM_LayerSettingsDialog_Update)

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_SetOrigin.CHANGE = MOHO.MSG_BASE
LM_SetOrigin.RESET = MOHO.MSG_BASE + 1

function LM_SetOrigin:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.StaticText("Origin"))

	layout:AddChild(LM.GUI.StaticText("X:"))
	self.textX = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textX)

	layout:AddChild(LM.GUI.StaticText("Y:"))
	self.textY = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textY)

	layout:AddChild(LM.GUI.Button("Reset", self.RESET))

	--if (self.layerSettingsWnd == nil) then
	--	self.layerSettingsWnd = LM_LayerSettingsDialog:new()
	--	self.layerSettingsWnd:DoModeless()
	--end
end

function LM_SetOrigin:UpdateWidgets(moho)
	local origin = moho.layer:Origin()
	self.textX:SetValue(origin.x)
	self.textY:SetValue(origin.y)
	if (self.layerSettingsWnd) then
		self.layerSettingsWnd:Update(moho)
	end
end

function LM_SetOrigin:HandleMessage(moho, view, msg)
	local newVal = LM.Vector2:new_local()

	if (msg == self.RESET) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		newVal:Set(0, 0)
		moho.layer:SetOrigin(newVal)
		self:UpdateWidgets(moho)
	elseif (msg == self.CHANGE) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		newVal.x = self.textX:FloatValue()
		newVal.y = self.textY:FloatValue()
		moho.layer:SetOrigin(newVal)
	end
end
