-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_ShearLayerY"

-- **************************************************
-- General information about this script
-- **************************************************

LM_ShearLayerY = {}

function LM_ShearLayerY:Name()
	return "Shear Layer Y"
end

function LM_ShearLayerY:Version()
	return "5.0"
end

function LM_ShearLayerY:Description()
	return "Shear layer vertically"
end

function LM_ShearLayerY:Creator()
	return "Lost Marble"
end

function LM_ShearLayerY:UILabel()
	return("Shear Layer Y")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_ShearLayerY.startVal = LM.Vector3:new_local()
LM_ShearLayerY.matrix = LM.Matrix:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_ShearLayerY:OnMouseDown(moho, mouseEvent)
	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	self.startVal:Set(moho.layer.fShear.value)
	moho.layer.fShear:AddKey(moho.frame)
	moho.layer:GetFullTransform(moho.frame, self.matrix, moho.document)

	mouseEvent.view:DrawMe()
end

function LM_ShearLayerY:OnMouseMoved(moho, mouseEvent)
	local vec = mouseEvent.view:Point2Vec(mouseEvent.pt, self.matrix)
	local shearing = LM.Vector3:new_local()
	shearing:Set(self.startVal)
	local origin = moho.layer:Origin()

	-- shearing connected to actual drag amount
	local v1 = mouseEvent.startVec - origin
	local v2 = vec - origin
	if (math.abs(v1.x) > 1e-6) then
		shearing.y = shearing.y + (v2.y - v1.y) / v1.x
	end

	moho.layer.fShear:SetValue(moho.frame, shearing)
	moho.document:DepthSort()

	mouseEvent.view:DrawMe()
end

function LM_ShearLayerY:OnMouseUp(moho, mouseEvent)
	moho:NewKeyframe(CHANNEL_LAYER_SH)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_ShearLayerY.CHANGE = MOHO.MSG_BASE
LM_ShearLayerY.RESET = MOHO.MSG_BASE + 1

function LM_ShearLayerY:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.StaticText("Shear:"))

	layout:AddChild(LM.GUI.StaticText("X:"))
	self.shearX = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.shearX)

	layout:AddChild(LM.GUI.StaticText("Y:"))
	self.shearY = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.shearY)

	layout:AddChild(LM.GUI.StaticText("Z:"))
	self.shearZ = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.shearZ)

	layout:AddChild(LM.GUI.Button("Reset", self.RESET))
end

function LM_ShearLayerY:UpdateWidgets(moho)
	self.shearX:SetValue(moho.layer.fShear.value.x)
	self.shearY:SetValue(moho.layer.fShear.value.y)
	self.shearZ:SetValue(moho.layer.fShear.value.z)
end

function LM_ShearLayerY:HandleMessage(moho, view, msg)
	local newVal = LM.Vector3:new_local()

	if (msg == self.RESET) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		if (moho.frame == 0) then
			newVal:Set(0, 0, 0)
			moho.layer.fShear:SetValue(0, newVal)
		else
			newVal = moho.layer.fShear:GetValue(0)
			moho.layer.fShear:SetValue(moho.frame, newVal)
		end
		self:UpdateWidgets(moho)
		moho:NewKeyframe(CHANNEL_LAYER_SH)
	elseif (msg == self.CHANGE) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		newVal.x = self.shearX:FloatValue()
		newVal.y = self.shearY:FloatValue()
		newVal.z = self.shearZ:FloatValue()
		local vec = moho.layer.fShear:GetValue(moho.frame) - newVal
		if (vec:Mag() > 0.0001) then
			moho.layer.fShear:SetValue(moho.frame, newVal)
			moho:NewKeyframe(CHANNEL_LAYER_SH)
		end
	end
end
