-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_ShearPointsX"

-- **************************************************
-- General information about this script
-- **************************************************

LM_ShearPointsX = {}

function LM_ShearPointsX:Name()
	return "Shear Points X"
end

function LM_ShearPointsX:Version()
	return "5.0"
end

function LM_ShearPointsX:Description()
	return "Skew selected points horizontally"
end

function LM_ShearPointsX:Creator()
	return "Lost Marble"
end

function LM_ShearPointsX:UILabel()
	return("Shear Points X")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_ShearPointsX.minVec = LM.Vector2:new_local()
LM_ShearPointsX.maxVec = LM.Vector2:new_local()
LM_ShearPointsX.center = LM.Vector2:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_ShearPointsX:IsEnabled(moho)
	if (moho:CountSelectedPoints() > 1) then
		return true
	end
	return false
end

function LM_ShearPointsX:OnMouseDown(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	mesh:SelectedBounds(self.minVec, self.maxVec)
	self.center.x = (self.minVec.x + self.maxVec.x) / 2
	self.center.y = (self.minVec.y + self.maxVec.y) / 2

	self.selList = MOHO.SelectedPointList(mesh)
	mesh:PrepMovePoints()

	mouseEvent.view:DrawMe()
end

function LM_ShearPointsX:OnMouseMoved(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	local curVec = mouseEvent.vec - mouseEvent.startVec
	local divisor = self.maxVec.y - self.center.y

	for i, pt in self.selList do
		pt.fPos.x = pt.fTempPos.x + curVec.x * (pt.fTempPos.y - self.center.y) / divisor
	end

	moho:AddPointKeyframe(moho.frame)
	mouseEvent.view:DrawMe()
end

function LM_ShearPointsX:OnMouseUp(moho, mouseEvent)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	moho:AddPointKeyframe(moho.frame)
	moho:NewKeyframe(CHANNEL_POINT)

	self.selList = nil
	mouseEvent.view:DrawMe()
end

function LM_ShearPointsX:OnKeyDown(moho, keyEvent)
	LM_SelectPoints:OnKeyDown(moho, keyEvent)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_ShearPointsX.DUMMY = MOHO.MSG_BASE
LM_ShearPointsX.RESET = MOHO.MSG_BASE + 1
LM_ShearPointsX.SELECTITEM = MOHO.MSG_BASE + 2

function LM_ShearPointsX:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Group")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	layout:AddChild(LM.GUI.Button("Reset", self.RESET))
end

function LM_ShearPointsX:UpdateWidgets(moho)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	MOHO.BuildGroupMenu(self.menu, mesh, self.SELECTITEM, self.DUMMY)
end

function LM_ShearPointsX:HandleMessage(moho, view, msg)
	local mesh = moho:Mesh()
	if (mesh == nil) then
		return
	end

	if (msg == self.RESET) then
		if (moho.frame > 0) then
			moho.document:PrepUndo(moho.layer)
			moho.document:SetDirty()

			for i = 0, mesh:CountPoints() - 1 do
				local pt = mesh:Point(i)
				if (pt.fSelected) then
					pt:SetPos(pt.fAnimPos:GetValue(0), moho.frame)
				end
			end
		end
		moho:NewKeyframe(CHANNEL_POINT)
	elseif (msg >= self.SELECTITEM) then
		mesh:SelectNone()
		local i = msg - self.SELECTITEM
		local name = mesh:Group(i):Name()
		mesh:SelectGroup(name)
		moho:UpdateUI()
	end
end
