-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_SwitchLayer"

-- **************************************************
-- General information about this script
-- **************************************************

LM_SwitchLayer = {}

function LM_SwitchLayer:Name()
	return "Switch Layer"
end

function LM_SwitchLayer:Version()
	return "5.0"
end

function LM_SwitchLayer:Description()
	return "Change the currently active layer in a switch group"
end

function LM_SwitchLayer:Creator()
	return "Lost Marble"
end

function LM_SwitchLayer:UILabel()
	return("Switch Layer")
end

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_SwitchLayer:IsEnabled(moho)
	local switchLayer = moho:LayerAsSwitch(moho.layer)
	if (switchLayer) then
		return true
	end
	return false
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_SwitchLayer.SELECTITEM = MOHO.MSG_BASE

function LM_SwitchLayer:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.StaticText("Active child:"))

	self.menu = LM.GUI.Menu("Select Switch Layer")

	self.popup = LM.GUI.PopupMenu(128, true)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	self.childName = LM.GUI.DynamicText("", 512)
	layout:AddChild(self.childName)
end

function LM_SwitchLayer:UpdateWidgets(moho)
	local switchLayer = moho:LayerAsSwitch(moho.layer)
	if (switchLayer == nil) then
		return
	end

	self.menu:RemoveAllItems()
	for i = 0, switchLayer:CountLayers() - 1 do
		self.menu:AddItem(switchLayer:Layer(i):Name(), 0, self.SELECTITEM + i)
	end
	self.menu:SetCheckedLabel(switchLayer:SwitchValues():GetValue(moho.frame), true)
	self.popup:Redraw()
end

function LM_SwitchLayer:HandleMessage(moho, view, msg)
	local switchLayer = moho:LayerAsSwitch(moho.layer)
	if (switchLayer == nil) then
		return
	end

	if (msg >= self.SELECTITEM) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
		switchLayer:SwitchValues():SetValue(moho.frame, self.menu:FirstCheckedLabel())
		moho:NewKeyframe(CHANNEL_SWITCH)
	end
end
