-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_TrackCamera"

-- **************************************************
-- General information about this script
-- **************************************************

LM_TrackCamera = {}

function LM_TrackCamera:Name()
	return "Track Camera"
end

function LM_TrackCamera:Version()
	return "5.0"
end

function LM_TrackCamera:Description()
	return "Track camera side to side, and up and down (hold <shift> to constrain, <alt> to move forward and back)"
end

function LM_TrackCamera:Creator()
	return "Lost Marble"
end

function LM_TrackCamera:UILabel()
	return("Track Camera")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_TrackCamera.startVal = LM.Vector3:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_TrackCamera:OnMouseDown(moho, mouseEvent)
	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	self.startVal:Set(moho.document.fCameraTrack.value)
	moho.document.fCameraTrack:AddKey(moho.frame)

	mouseEvent.view:DrawMe()
end

function LM_TrackCamera:OnMouseMoved(moho, mouseEvent)
	local v1 = LM.Vector2:new_local()
	local v2 = LM.Vector2:new_local()
	local vec = LM.Vector3:new_local()
	local u1 = LM.Vector3:new_local()
	local u2 = LM.Vector3:new_local()
	local m = LM.Matrix:new_local()

	mouseEvent.view:Graphics():ScreenToWorld(mouseEvent.startPt, v1)
	mouseEvent.view:Graphics():ScreenToWorld(mouseEvent.pt, v2)

	vec:Set(self.startVal)

	u1:Set(v1.x, v1.y, 0)
	u2:Set(v2.x, v2.y, 0)
	if (mouseEvent.altKey) then
		u1:Set(0, 0, 0)
		u2:Set(0, 0, 2 * (mouseEvent.pt.y - mouseEvent.startPt.y) / mouseEvent.view:Graphics():Height())
	elseif (mouseEvent.shiftKey) then
		local pt = LM.Point:new_local()

		pt:Set(mouseEvent.pt)
		local dx = math.abs(mouseEvent.pt.x - mouseEvent.startPt.x)
		local dy = math.abs(mouseEvent.pt.y - mouseEvent.startPt.y)
		if (dx > dy * 3) then
			pt.y = mouseEvent.startPt.y
		else
			pt.x = mouseEvent.startPt.x
		end
		mouseEvent.view:Graphics():ScreenToWorld(pt, v2)
		u2:Set(v2.x, v2.y, 0)
	end

	m:Identity()
	m:Rotate(LM.Z_AXIS, moho.document.fCameraRoll.value)
	m:Rotate(LM.X_AXIS, -moho.document.fCameraPanTilt.value.x)
	m:Rotate(LM.Y_AXIS, -moho.document.fCameraPanTilt.value.y)
	m:Invert()
	m:Transform(u1)
	m:Transform(u2)

	u2 = u2 - u1

	vec = vec - u2

	moho.document.fCameraTrack:SetValue(moho.frame, vec)
	moho.document:DepthSort()

	mouseEvent.view:DrawMe()
end

function LM_TrackCamera:OnMouseUp(moho, mouseEvent)
	moho:NewKeyframe(CHANNEL_CAMERA_TRACK)
end

function LM_TrackCamera:OnKeyDown(moho, keyEvent)
	if (keyEvent.ctrlKey) then
		local inc = 1
		if (keyEvent.shiftKey) then
			inc = 10
		end

		local m = LM.Matrix:new_local()
		moho.layer:GetFullTransform(moho.frame, m, moho.document)

		local fakeME = {}
		fakeME.view = keyEvent.view
		fakeME.pt = LM.Point:new_local()
		fakeME.pt:Set(keyEvent.view:Graphics():Width() / 2, keyEvent.view:Graphics():Height() / 2)
		fakeME.startPt = LM.Point:new_local()
		fakeME.startPt:Set(fakeME.pt)
		fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
		fakeME.startVec = keyEvent.view:Point2Vec(fakeME.pt, m)
		fakeME.shiftKey = false
		fakeME.ctrlKey = false
		fakeME.altKey = keyEvent.altKey
		fakeME.penPressure = 0

		if (keyEvent.keyCode == LM.GUI.KEY_UP) then
			self:OnMouseDown(moho, fakeME)
			fakeME.pt.y = fakeME.pt.y - inc
			fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
			self:OnMouseMoved(moho, fakeME)
			self:OnMouseUp(moho, fakeME)
		elseif (keyEvent.keyCode == LM.GUI.KEY_DOWN) then
			self:OnMouseDown(moho, fakeME)
			fakeME.pt.y = fakeME.pt.y + inc
			fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
			self:OnMouseMoved(moho, fakeME)
			self:OnMouseUp(moho, fakeME)
		elseif (keyEvent.keyCode == LM.GUI.KEY_LEFT) then
			self:OnMouseDown(moho, fakeME)
			fakeME.pt.x = fakeME.pt.x - inc
			fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
			self:OnMouseMoved(moho, fakeME)
			self:OnMouseUp(moho, fakeME)
		elseif (keyEvent.keyCode == LM.GUI.KEY_RIGHT) then
			self:OnMouseDown(moho, fakeME)
			fakeME.pt.x = fakeME.pt.x + inc
			fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
			self:OnMouseMoved(moho, fakeME)
			self:OnMouseUp(moho, fakeME)
		end
	end
end

function LM_TrackCamera:DrawMe(moho, view)
	if (not self.displayOn) then
		return
	end

	local endFrame = moho.document.fCameraTrack:Duration()

	if (endFrame > 0) then
		local g = view:Graphics()
		local m = LM.Matrix:new_local()
		local vec = LM.Vector3:new_local()
		local oldVec = LM.Vector3:new_local()
		local tempLayer = MOHO.MohoLayer:new_local()

		g:SetColor(102, 152, 203)
		g:SetSmoothing(true)
		for frame = 0, endFrame do
			tempLayer:GetFullTransform(frame, m, moho.document)
			vec:Set(moho.document.fCameraTrack:GetValue(frame))
			m:Transform(vec)
			if (frame > 0) then
				g:DrawLine(oldVec.x, oldVec.y, vec.x, vec.y)
			end
			if (moho.document.fCameraTrack:HasKey(frame)) then
				g:DrawFatMarker(vec.x, vec.y, 3)
			end
			oldVec:Set(vec)
		end
		g:SetSmoothing(false)
	end
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_TrackCamera.CHANGE = MOHO.MSG_BASE
LM_TrackCamera.RESET = MOHO.MSG_BASE + 1
LM_TrackCamera.TOGGLE_DISPLAY = MOHO.MSG_BASE + 2
LM_TrackCamera.displayOn = false

function LM_TrackCamera:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.StaticText("Position"))

	layout:AddChild(LM.GUI.StaticText("X:"))
	self.textX = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textX)

	layout:AddChild(LM.GUI.StaticText("Y:"))
	self.textY = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textY)

	layout:AddChild(LM.GUI.StaticText("Z:"))
	self.textZ = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textZ)

	layout:AddChild(LM.GUI.Button("Reset", self.RESET))

	self.displayCheck = LM.GUI.CheckBox("Show path", self.TOGGLE_DISPLAY)
	layout:AddChild(self.displayCheck)
end

function LM_TrackCamera:UpdateWidgets(moho)
	self.textX:SetValue(moho.document.fCameraTrack.value.x)
	self.textY:SetValue(moho.document.fCameraTrack.value.y)
	self.textZ:SetValue(moho.document.fCameraTrack.value.z)
	self.displayCheck:SetValue(self.displayOn)
end

function LM_TrackCamera:HandleMessage(moho, view, msg)
	local newVal = LM.Vector3:new_local()

	if (msg == self.RESET) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		if (moho.frame == 0) then
			newVal:Set(0, 0, 5 - 3.2679)
			moho.document.fCameraTrack:SetValue(0, newVal)
		else
			newVal:Set(moho.document.fCameraTrack:GetValue(0))
			moho.document.fCameraTrack:SetValue(moho.frame, newVal)
		end
		self:UpdateWidgets(moho)
		moho:NewKeyframe(CHANNEL_CAMERA_TRACK)
	elseif (msg == self.CHANGE) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		newVal.x = self.textX:FloatValue()
		newVal.y = self.textY:FloatValue()
		newVal.z = self.textZ:FloatValue()
		local vec = moho.document.fCameraTrack:GetValue(moho.frame) - newVal
		if (vec:Mag() > 0.0001) then
			moho.document.fCameraTrack:SetValue(moho.frame, newVal)
			moho:NewKeyframe(CHANNEL_CAMERA_TRACK)
		end
	elseif (msg == self.TOGGLE_DISPLAY) then
		self.displayOn = self.displayCheck:Value()
	end
end
