-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_TranslateBone"

-- **************************************************
-- General information about this script
-- **************************************************

LM_TranslateBone = {}

function LM_TranslateBone:Name()
	return "Translate Bone"
end

function LM_TranslateBone:Version()
	return "5.0"
end

function LM_TranslateBone:Description()
	return "Move bone (hold <shift> to constrain)"
end

function LM_TranslateBone:Creator()
	return "Lost Marble"
end

function LM_TranslateBone:UILabel()
	return("Translate Bone")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_TranslateBone.selID = -1
LM_TranslateBone.boneEnd = -1
LM_TranslateBone.lastVec = LM.Vector2:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_TranslateBone:IsEnabled(moho)
	if (moho:CountBones() < 1) then
		return false
	end
	if (moho:CountSelectedBones() > 1) then
		return false
	end
	return true
end

function LM_TranslateBone:OnMouseDown(moho, mouseEvent)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	self.lastVec:Set(mouseEvent.startVec)

	if (self.selID < 0) then
		-- If selID already has a value, then this function is being called by LM_AddBone.
		-- In that case, PrepUndo has already been called, and the bone has been picked.
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()
		self.selID = mouseEvent.view:PickBone(mouseEvent.pt, mouseEvent.vec, moho.layer, false)
	end

	for i = 0, skel:CountBones() - 1 do
		local bone = skel:Bone(i)

		if (moho.frame == 0) then
			bone.fTempPos:Set(bone.fAnimPos:GetValue(0))
			bone.fTempLength = bone.fLength
			bone.fTempAngle = bone.fAnimAngle:GetValue(0)
		else
			bone.fTempPos:Set(bone.fPos)
		end

		if (i == self.selID) then
			bone.fSelected = true
			if (self.boneEnd < 0) then
				if (moho.frame == 0) then
					local boneVec = LM.Vector2:new_local()
					boneVec:Set(0, 0)
					bone.fRestMatrix:Transform(boneVec)
					boneVec = boneVec - mouseEvent.startVec
					local d = boneVec:Mag()
					self.boneEnd = 0
					boneVec:Set(bone.fLength, 0)
					bone.fRestMatrix:Transform(boneVec)
					boneVec = boneVec - mouseEvent.startVec
					if (boneVec:Mag() < d) then
						self.boneEnd = 1
					end
				else
					self.boneEnd = 0
				end
			end
		else
			bone.fSelected = false
		end
	end

	moho.layer:UpdateCurFrame()
	moho:UpdateBonePointSelection()
	mouseEvent.view:DrawMe()
	moho:UpdateSelectedChannels()
end

function LM_TranslateBone:OnMouseMoved(moho, mouseEvent)
	local skel = moho:Skeleton()
	if ((skel == nil) or (self.selID < 0)) then
		return
	end

	local bone = skel:Bone(self.selID)
	if (moho.frame == 0) then
		bone.fAnimPos:SetValue(0, bone.fTempPos)
		bone.fLength = bone.fTempLength
		bone.fAnimAngle:SetValue(0, bone.fTempAngle)
	else
		bone.fPos:Set(bone.fTempPos)
	end
	skel:UpdateBoneMatrix(self.selID)

	local offset = mouseEvent.vec
	if (self.boneEnd == 0) then
		offset = offset - mouseEvent.startVec
	elseif (self.boneEnd == 1) then
		offset = offset - self.lastVec
	end

	local boneVec = LM.Vector2:new_local()
	local inverseM = LM.Matrix:new_local()

	if (self.boneEnd == 0) then -- move the base of the bone
		local parent = nil
		boneVec:Set(0, 0)
		if (bone.fParent >= 0) then
			parent = skel:Bone(bone.fParent)
			if (moho.frame == 0) then
				parent.fRestMatrix:Transform(boneVec)
			else
				parent.fMovedMatrix:Transform(boneVec)
			end
		end
		boneVec = boneVec + offset
		if (parent) then
			if (moho.frame == 0) then
				inverseM:Set(parent.fRestMatrix)
			else
				inverseM:Set(parent.fMovedMatrix)
			end
			inverseM:Invert()
			inverseM:Transform(boneVec)
		end

		if (mouseEvent.shiftKey) then
			if (math.abs(boneVec.x) > math.abs(boneVec.y)) then
				boneVec.y = 0
			else
				boneVec.x = 0
			end
		end

		local v = nil
		if (moho.frame == 0) then
			v = bone.fAnimPos:GetValue(0) + boneVec
		else
			v = bone.fPos + boneVec
		end
		if (moho.gridOn) then
			if (parent) then
				parent.fMovedMatrix:Transform(v)
			end
			moho:SnapToGrid(v)
			if (parent) then
				inverseM:Set(parent.fMovedMatrix)
				inverseM:Invert()
				inverseM:Transform(v)
			end
		end

		bone.fAnimPos:SetValue(moho.frame, v)
		moho.layer:UpdateCurFrame()
	elseif (self.boneEnd == 1) then -- move the tip of the bone
		boneVec:Set(bone.fLength, 0)
		bone.fRestMatrix:Transform(boneVec)
		boneVec = boneVec + offset
		if (moho.gridOn) then
			moho:SnapToGrid(boneVec)
			self.lastVec:Set(boneVec)
		else
			self.lastVec:Set(mouseEvent.vec)
		end

		inverseM:Set(bone.fRestMatrix)
		inverseM:Invert()
		inverseM:Transform(boneVec)
		local dL = boneVec:Mag() - bone.fLength
		bone.fLength = bone.fLength + dL
		local angle = bone.fAnimAngle:GetValue(moho.frame)
		angle = angle + math.atan2(boneVec.y, boneVec.x)
		while angle > 2 * math.pi do
			angle = angle - 2 * math.pi
		end
		while angle < 0 do
			angle = angle + 2 * math.pi
		end
		bone.fTempAngle = angle
		bone.fTempLength = bone.fLength
		if (mouseEvent.shiftKey) then
			angle = angle / (math.pi / 4)
			angle = (math.pi / 4) * LM.Round(angle)
		end
		bone.fAnimAngle:SetValue(0, angle)
		for i = 0, skel:CountBones() - 1 do
			bone = skel:Bone(i)
			if (bone.fParent == self.selID) then
				boneVec:Set(bone.fTempPos)
				boneVec.x = boneVec.x + dL
				bone.fAnimPos:SetValue(moho.frame, boneVec)
			end
		end
		moho.layer:UpdateCurFrame()
	end

	mouseEvent.view:DrawMe()
end

function LM_TranslateBone:OnMouseUp(moho, mouseEvent)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	if ((self.selID >= 0) and (moho.frame > 0)) then
		local bone = skel:Bone(self.selID)
		bone.fAnimPos:SetValue(moho.frame, bone.fPos)
		moho.layer:UpdateCurFrame()
		moho:NewKeyframe(CHANNEL_BONE_T)
	end

	self.selID = -1
	self.boneEnd = -1

	mouseEvent.view:DrawMe()
end

function LM_TranslateBone:OnKeyDown(moho, keyEvent)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	LM_SelectBone:OnKeyDown(moho, keyEvent)
	if (keyEvent.ctrlKey) then
		local inc = 1
		if (keyEvent.shiftKey) then
			inc = 10
		end

		local m = LM.Matrix:new_local()
		moho.layer:GetFullTransform(moho.frame, m, moho.document)

		local fakeME = {}
		fakeME.view = keyEvent.view
		fakeME.pt = LM.Point:new_local()
		fakeME.pt:Set(keyEvent.view:Graphics():Width() / 2, keyEvent.view:Graphics():Height() / 2)
		fakeME.startPt = LM.Point:new_local()
		fakeME.startPt:Set(fakeME.pt)
		fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
		fakeME.startVec = keyEvent.view:Point2Vec(fakeME.pt, m)
		fakeME.shiftKey = false
		fakeME.ctrlKey = false
		fakeME.altKey = keyEvent.altKey
		fakeME.penPressure = 0

		if (keyEvent.keyCode == LM.GUI.KEY_UP) then
			self.selID = self:SelIDForNudge(moho, skel)
			self:OnMouseDown(moho, fakeME)
			self.boneEnd = 0
			fakeME.pt.y = fakeME.pt.y - inc
			fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
			self:OnMouseMoved(moho, fakeME)
			self:OnMouseUp(moho, fakeME)
		elseif (keyEvent.keyCode == LM.GUI.KEY_DOWN) then
			self.selID = self:SelIDForNudge(moho, skel)
			self:OnMouseDown(moho, fakeME)
			self.boneEnd = 0
			fakeME.pt.y = fakeME.pt.y + inc
			fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
			self:OnMouseMoved(moho, fakeME)
			self:OnMouseUp(moho, fakeME)
		elseif (keyEvent.keyCode == LM.GUI.KEY_LEFT) then
			self.selID = self:SelIDForNudge(moho, skel)
			self:OnMouseDown(moho, fakeME)
			self.boneEnd = 0
			fakeME.pt.x = fakeME.pt.x - inc
			fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
			self:OnMouseMoved(moho, fakeME)
			self:OnMouseUp(moho, fakeME)
		elseif (keyEvent.keyCode == LM.GUI.KEY_RIGHT) then
			self.selID = self:SelIDForNudge(moho, skel)
			self:OnMouseDown(moho, fakeME)
			self.boneEnd = 0
			fakeME.pt.x = fakeME.pt.x + inc
			fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
			self:OnMouseMoved(moho, fakeME)
			self:OnMouseUp(moho, fakeME)
		end
	end
end

function LM_TranslateBone:SelIDForNudge(moho, skel)
	for i = 0, skel:CountBones() - 1 do
		if (skel:Bone(i).fSelected) then
			return i
		end
	end
	return -1
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_TranslateBone.CHANGE = MOHO.MSG_BASE
LM_TranslateBone.RESET = MOHO.MSG_BASE + 1
LM_TranslateBone.DUMMY = MOHO.MSG_BASE + 2
LM_TranslateBone.SELECTITEM = MOHO.MSG_BASE + 3

function LM_TranslateBone:DoLayout(moho, layout)
	self.menu = LM.GUI.Menu("Select Bone")

	self.popup = LM.GUI.PopupMenu(128, false)
	self.popup:SetMenu(self.menu)
	layout:AddChild(self.popup)

	layout:AddChild(LM.GUI.StaticText("Position"))

	layout:AddChild(LM.GUI.StaticText("X:"))
	self.textX = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	self.textX:SetWheelInc(0.1)
	layout:AddChild(self.textX)

	layout:AddChild(LM.GUI.StaticText("Y:"))
	self.textY = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	self.textY:SetWheelInc(0.1)
	layout:AddChild(self.textY)

	layout:AddChild(LM.GUI.StaticText("Length:"))
	self.textL = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	self.textL:SetWheelInc(0.1)
	layout:AddChild(self.textL)

	self.reset = LM.GUI.Button("Reset", self.RESET)
	layout:AddChild(self.reset)
end

function LM_TranslateBone:UpdateWidgets(moho)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	local selID = skel:SelectedBoneID()

	MOHO.BuildBoneMenu(self.menu, skel, self.SELECTITEM, self.DUMMY)

	if (selID >= 0) then
		local bone = skel:Bone(selID)
		self.textX:SetValue(bone.fPos.x)
		self.textY:SetValue(bone.fPos.y)
		self.textL:SetValue(bone.fLength)
	else
		self.textX:SetValue("")
		self.textY:SetValue("")
		self.textL:SetValue("")
	end

	if (moho.frame == 0) then
		self.textL:Enable(true)
		self.reset:Enable(false)
	else
		self.textL:Enable(false)
		self.reset:Enable(true)
	end
end

function LM_TranslateBone:HandleMessage(moho, view, msg)
	local skel = moho:Skeleton()
	if (skel == nil) then
		return
	end

	if (msg == self.RESET) then
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			if (bone.fSelected) then
				moho.document:PrepUndo(moho.layer)
				moho.document:SetDirty()
				bone.fAnimPos:SetValue(moho.frame, bone.fAnimPos:GetValue(0))
				moho.layer:UpdateCurFrame()
				moho:NewKeyframe(CHANNEL_BONE_T)
				break
			end
		end
		self:UpdateWidgets(moho)
	elseif (msg == self.CHANGE) then
		for i = 0, skel:CountBones() - 1 do
			local bone = skel:Bone(i)
			if (bone.fSelected) then
				moho.document:PrepUndo(moho.layer)
				moho.document:SetDirty()
				bone.fPos.x = self.textX:FloatValue()
				bone.fPos.y = self.textY:FloatValue()
				bone.fLength = self.textL:FloatValue()
				bone.fAnimPos:SetValue(moho.frame, bone.fPos)
				moho.layer:UpdateCurFrame()
				moho:NewKeyframe(CHANNEL_BONE_T)
				break
			end
		end
	elseif (msg >= self.SELECTITEM) then
		for i = 0, skel:CountBones() - 1 do
			skel:Bone(i).fSelected = (i == msg - self.SELECTITEM)
		end
		moho:UpdateUI()
	end
end
