-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_TranslateLayer"

-- **************************************************
-- General information about this script
-- **************************************************

LM_TranslateLayer = {}

function LM_TranslateLayer:Name()
	return "Translate Layer"
end

function LM_TranslateLayer:Version()
	return "5.0"
end

function LM_TranslateLayer:Description()
	return "Move entire layer (hold <shift> to constrain, <alt> to move in Z)"
end

function LM_TranslateLayer:Creator()
	return "Lost Marble"
end

function LM_TranslateLayer:UILabel()
	return("Translate Layer")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_TranslateLayer.startVal = LM.Vector3:new_local()

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_TranslateLayer:OnMouseDown(moho, mouseEvent)
	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	self.startVal:Set(moho.layer.fTranslation.value)
	moho.layer.fTranslation:AddKey(moho.frame)

	mouseEvent.view:DrawMe()
end

function LM_TranslateLayer:OnMouseMoved(moho, mouseEvent)
	if (moho.document:IsOutsideViewEnabled()) then
		local v1 = LM.Vector3:new_local()
		local v2 = LM.Vector3:new_local()
		local vec = LM.Vector3:new_local()
		local m = LM.Matrix:new_local()

		if (moho.layer:Parent()) then
			moho.layer:Parent():GetFullTransform(moho.frame, m, moho.document)
		else
			moho.document:GetOutsideViewMatrix(m)
		end
		m:Invert()
		mouseEvent.view:Graphics():ScreenToWorld(mouseEvent.startPt, v1)
		mouseEvent.view:Graphics():ScreenToWorld(mouseEvent.pt, v2)

		vec.x = v2.x - v1.x
		vec.y = v2.y - v1.y

		if (mouseEvent.shiftKey) then
			if (math.abs(vec.x) > math.abs(vec.y)) then
				v2.y = v1.y
			else
				v2.x = v1.x
			end
		end

		v1.z = -0.95
		v2.z = -0.95
		m:Transform(v1)
		m:Transform(v2)

		vec = self.startVal + v2 - v1
		moho.layer.fTranslation:SetValue(moho.frame, vec)
	else
		moho.layer.fTranslation:SetValue(moho.frame, self.startVal)

		local layerM = LM.Matrix:new_local()
		local tempLayer = MOHO.MohoLayer:new_local()
		tempLayer.fTranslation:SetValue(0, moho.layer.fTranslation.value)

		if (moho.layer:Parent()) then
			local parentM = LM.Matrix:new_local()
			tempLayer:GetFullTransform(moho.frame, layerM, nil)
			moho.layer:Parent():GetFullTransform(moho.frame, parentM, moho.document)
			layerM:Multiply(parentM)
		else
			tempLayer:GetFullTransform(moho.frame, layerM, moho.document)
		end
		self.startVec = mouseEvent.view:Point2Vec(mouseEvent.startPt, layerM)
		self.nextVec = mouseEvent.view:Point2Vec(mouseEvent.pt, layerM)

		local vec = LM.Vector3:new_local()

		if (mouseEvent.altKey) then
			mouseEvent.view:Graphics():ScreenToWorld(mouseEvent.startPt, self.startVec)
			mouseEvent.view:Graphics():ScreenToWorld(mouseEvent.pt, self.nextVec)
			vec:Set(self.startVal)
			vec.z = self.startVal.z - (self.nextVec.y - self.startVec.y)
			moho.layer.fTranslation:SetValue(moho.frame, vec)

			if (moho.layer:Parent()) then
				moho.layer:Parent():DepthSort(moho.document)
			end
		else
			vec.x = self.nextVec.x - self.startVec.x
			vec.y = self.nextVec.y - self.startVec.y

			if (mouseEvent.shiftKey) then
				if (math.abs(vec.x) > math.abs(vec.y)) then
					vec.y = 0
				else
					vec.x = 0
				end
			end

			vec = vec + self.startVal
			moho.layer.fTranslation:SetValue(moho.frame, vec)
		end
	end

	moho.document:DepthSort()
	mouseEvent.view:DrawMe()
end

function LM_TranslateLayer:OnMouseUp(moho, mouseEvent)
	moho:NewKeyframe(CHANNEL_LAYER_T)
end

function LM_TranslateLayer:OnKeyDown(moho, keyEvent)
	if (keyEvent.ctrlKey) then
		local inc = 1
		if (keyEvent.shiftKey) then
			inc = 10
		end

		local m = LM.Matrix:new_local()
		moho.layer:GetFullTransform(moho.frame, m, moho.document)

		local fakeME = {}
		fakeME.view = keyEvent.view
		fakeME.pt = LM.Point:new_local()
		fakeME.pt:Set(keyEvent.view:Graphics():Width() / 2, keyEvent.view:Graphics():Height() / 2)
		fakeME.startPt = LM.Point:new_local()
		fakeME.startPt:Set(fakeME.pt)
		fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
		fakeME.startVec = keyEvent.view:Point2Vec(fakeME.pt, m)
		fakeME.shiftKey = false
		fakeME.ctrlKey = false
		fakeME.altKey = keyEvent.altKey
		fakeME.penPressure = 0

		if (keyEvent.keyCode == LM.GUI.KEY_UP) then
			self:OnMouseDown(moho, fakeME)
			fakeME.pt.y = fakeME.pt.y - inc
			fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
			self:OnMouseMoved(moho, fakeME)
			self:OnMouseUp(moho, fakeME)
		elseif (keyEvent.keyCode == LM.GUI.KEY_DOWN) then
			self:OnMouseDown(moho, fakeME)
			fakeME.pt.y = fakeME.pt.y + inc
			fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
			self:OnMouseMoved(moho, fakeME)
			self:OnMouseUp(moho, fakeME)
		elseif (keyEvent.keyCode == LM.GUI.KEY_LEFT) then
			self:OnMouseDown(moho, fakeME)
			fakeME.pt.x = fakeME.pt.x - inc
			fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
			self:OnMouseMoved(moho, fakeME)
			self:OnMouseUp(moho, fakeME)
		elseif (keyEvent.keyCode == LM.GUI.KEY_RIGHT) then
			self:OnMouseDown(moho, fakeME)
			fakeME.pt.x = fakeME.pt.x + inc
			fakeME.vec = keyEvent.view:Point2Vec(fakeME.pt, m)
			self:OnMouseMoved(moho, fakeME)
			self:OnMouseUp(moho, fakeME)
		end
	end
end

function LM_TranslateLayer:DrawMe(moho, view)
	if (not self.displayOn) then
		return
	end

	local endFrame = moho.layer.fTranslation:Duration()

	if (endFrame > 0) then
		local g = view:Graphics()
		local m = LM.Matrix:new_local()
		local vec = LM.Vector3:new_local()
		local oldVec = LM.Vector3:new_local()
		local origin = moho.layer:Origin()

		g:SetColor(102, 152, 203)
		g:SetSmoothing(true)
		for frame = 0, endFrame do
			moho.layer:GetFullTransform(frame, m, moho.document)
			vec:Set(origin.x, origin.y, 0)
			m:Transform(vec)
			if (frame > 0) then
				g:DrawLine(oldVec.x, oldVec.y, vec.x, vec.y)
			end
			if (moho.layer.fTranslation:HasKey(frame)) then
				g:DrawFatMarker(vec.x, vec.y, 3)
			end
			oldVec:Set(vec)
		end
		g:SetSmoothing(false)
	end
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_TranslateLayer.CHANGE = MOHO.MSG_BASE
LM_TranslateLayer.RESET = MOHO.MSG_BASE + 1
LM_TranslateLayer.TOGGLE_DISPLAY = MOHO.MSG_BASE + 2
LM_TranslateLayer.displayOn = true

function LM_TranslateLayer:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.StaticText("Position"))

	layout:AddChild(LM.GUI.StaticText("X:"))
	self.textX = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textX)

	layout:AddChild(LM.GUI.StaticText("Y:"))
	self.textY = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textY)

	layout:AddChild(LM.GUI.StaticText("Z:"))
	self.textZ = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_FLOAT)
	layout:AddChild(self.textZ)

	layout:AddChild(LM.GUI.Button("Reset", self.RESET))

	self.displayCheck = LM.GUI.CheckBox("Show path", self.TOGGLE_DISPLAY)
	layout:AddChild(self.displayCheck)
end

function LM_TranslateLayer:UpdateWidgets(moho)
	self.textX:SetValue(moho.layer.fTranslation.value.x)
	self.textY:SetValue(moho.layer.fTranslation.value.y)
	self.textZ:SetValue(moho.layer.fTranslation.value.z)
	self.displayCheck:SetValue(self.displayOn)
end

function LM_TranslateLayer:HandleMessage(moho, view, msg)
	local newVal = LM.Vector3:new_local()

	if (msg == self.RESET) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		if (moho.frame == 0) then
			newVal:Set(0, 0, 0)
			moho.layer.fTranslation:SetValue(0, newVal)
		else
			newVal:Set(moho.layer.fTranslation:GetValue(0))
			moho.layer.fTranslation:SetValue(moho.frame, newVal)
		end
		self:UpdateWidgets(moho)
		moho:NewKeyframe(CHANNEL_LAYER_T)
		moho.document:DepthSort()
	elseif (msg == self.CHANGE) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		newVal.x = self.textX:FloatValue()
		newVal.y = self.textY:FloatValue()
		newVal.z = self.textZ:FloatValue()
		local vec = moho.layer.fTranslation:GetValue(moho.frame) - newVal
		if (vec:Mag() > 0.0001) then
			moho.layer.fTranslation:SetValue(moho.frame, newVal)
			moho:NewKeyframe(CHANNEL_LAYER_T)
		end
		moho.document:DepthSort()
	elseif (msg == self.TOGGLE_DISPLAY) then
		self.displayOn = self.displayCheck:Value()
	end
end
