-- **************************************************
-- Provide Moho with the name of this script object
-- **************************************************

ScriptName = "LM_ZoomCamera"

-- **************************************************
-- General information about this script
-- **************************************************

LM_ZoomCamera = {}

function LM_ZoomCamera:Name()
	return "Zoom Camera"
end

function LM_ZoomCamera:Version()
	return "5.0"
end

function LM_ZoomCamera:Description()
	return "Zoom the camera in and out"
end

function LM_ZoomCamera:Creator()
	return "Lost Marble"
end

function LM_ZoomCamera:UILabel()
	return("Zoom Camera")
end

-- **************************************************
-- Recurring values
-- **************************************************

LM_ZoomCamera.startVal = 0

-- **************************************************
-- The guts of this script
-- **************************************************

function LM_ZoomCamera:OnMouseDown(moho, mouseEvent)
	moho.document:PrepUndo(moho.layer)
	moho.document:SetDirty()

	self.startVal = moho.document.fCameraZoom.value
	moho.document.fCameraZoom:AddKey(moho.frame)

	mouseEvent.view:DrawMe()
end

function LM_ZoomCamera:OnMouseMoved(moho, mouseEvent)
	local zoomX = 0
	local zoomY = 0
	local defaultFov = math.rad(60)
	local minFov = math.rad(0.25)
	local maxFov = math.rad(160)

	zoomX = self.startVal
	zoomY = self.startVal

	if (mouseEvent.pt.x > mouseEvent.startPt.x) then
		zoomX = zoomX * (1 + 3 *
			(mouseEvent.pt.x - mouseEvent.startPt.x) / mouseEvent.view:Graphics():Width())
	else
		zoomX = zoomX * (1 + 1.5 *
			(mouseEvent.pt.x - mouseEvent.startPt.x) / mouseEvent.view:Graphics():Width())
	end

	if (mouseEvent.pt.y > mouseEvent.startPt.y) then
		zoomY = zoomY * (1 + 3 *
			(mouseEvent.pt.y - mouseEvent.startPt.y) / mouseEvent.view:Graphics():Height())
	else
		zoomY = zoomY * (1 + 1.5 *
			(mouseEvent.pt.y - mouseEvent.startPt.y) / mouseEvent.view:Graphics():Height())
	end

	zoomX = (zoomX + zoomY) * 0.5
	if (zoomX < 0.01) then
		zoomX = 0.01
	end

	zoomX = defaultFov / zoomX
	if (zoomX < minFov) then
		zoomX = minFov
	end
	if (zoomX > maxFov) then
		zoomX = maxFov
	end
	zoomX = defaultFov / zoomX

	moho.document.fCameraZoom:SetValue(moho.frame, zoomX)
	moho.document:DepthSort()

	mouseEvent.view:DrawMe()
end

function LM_ZoomCamera:OnMouseUp(moho, mouseEvent)
	moho:NewKeyframe(CHANNEL_CAMERA_ZOOM)
end

-- **************************************************
-- Tool options - create and respond to tool's UI
-- **************************************************

LM_ZoomCamera.CHANGE = MOHO.MSG_BASE
LM_ZoomCamera.RESET = MOHO.MSG_BASE + 1
LM_ZoomCamera.DEFAULT_FOV = math.rad(60)

function LM_ZoomCamera:DoLayout(moho, layout)
	layout:AddChild(LM.GUI.StaticText("Zoom:"))
	self.rotZ = LM.GUI.TextControl(0, "00.0000", self.CHANGE, LM.GUI.FIELD_UFLOAT)
	self.rotZ:SetWheelInc(5)
	layout:AddChild(self.rotZ)

	layout:AddChild(LM.GUI.Button("Reset", self.RESET))
end

function LM_ZoomCamera:UpdateWidgets(moho)
	local camFov = self.DEFAULT_FOV / moho.document.fCameraZoom.value
	self.rotZ:SetValue(math.deg(camFov))
end

function LM_ZoomCamera:HandleMessage(moho, view, msg)
	local newVal = 0

	if (msg == self.RESET) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		if (moho.frame == 0) then
			newVal = 1
			moho.document.fCameraZoom:SetValue(0, newVal)
		else
			newVal = moho.document.fCameraZoom:GetValue(0)
			moho.document.fCameraZoom:SetValue(moho.frame, newVal)
		end
		self:UpdateWidgets(moho)
		moho:NewKeyframe(CHANNEL_CAMERA_ZOOM)
	elseif (msg == self.CHANGE) then
		moho.document:PrepUndo(moho.layer)
		moho.document:SetDirty()

		newVal = self.rotZ:FloatValue()
		if (newVal < 0.25 or newVal > 160) then
			newVal = LM.Clamp(newVal, 0.25, 160)
			self.rotZ:SetValue(newVal)
		end
		newVal = self.DEFAULT_FOV / math.rad(newVal)
		local d = moho.document.fCameraZoom:GetValue(moho.frame) - newVal
		if (math.abs(d) > 0.0001) then
			moho.document.fCameraZoom:SetValue(moho.frame, newVal)
			moho:NewKeyframe(CHANNEL_CAMERA_ZOOM)
		end
	end
end
