-- initialize the pseudo random number generator
math.randomseed(os.time())
math.random()
math.random()
math.random()

MOHO.UpdateTable = {}

function MOHO:RunUpdateTable(moho)
	for i, func in MOHO.UpdateTable do
		func(moho)
	end
end

function MOHO.RandomRange(min, max)
	local f = math.random()
	return LM.Lerp(f, min, max)
end

function MOHO.MultiplyColor(color, amount)
	local r = color.r * amount
	local g = color.g * amount
	local b = color.b * amount
	color.r = LM.Clamp(r, 0, 255)
	color.g = LM.Clamp(g, 0, 255)
	color.b = LM.Clamp(b, 0, 255)
end

-- return a table containing a list of all the selected points in the specified mesh
function MOHO.SelectedPointList(mesh)
	local l = {}

	for i = 0, mesh:CountPoints() - 1 do
		local pt = mesh:Point(i)
		if (pt.fSelected) then
			table.insert(l, pt)
		end
	end

	return l
end

function MOHO.DeleteSelectedPoints(mesh)
	local doAgain = true
	while doAgain do
		doAgain = false
		local i = 0
		while i < mesh:CountPoints() do
			local pt = mesh:Point(i)
			if (pt.fSelected) then
				mesh:DeletePoint(i)
				doAgain = true
			else
				i = i + 1
			end
		end
	end
end

function MOHO.BuildGroupMenu(menu, mesh, baseMsg, dummyMsg)
	menu:RemoveAllItems()
	if (mesh:CountGroups() == 0) then
		menu:AddItem("<None>", 0, dummyMsg)
		menu:SetEnabled(dummyMsg, false)
	else
		for i = 0, mesh:CountGroups() - 1 do
			menu:AddItem(mesh:Group(i):Name(), 0, baseMsg + i)
		end
	end
end

function MOHO.BuildBoneMenu(menu, skel, baseMsg, dummyMsg)
	menu:RemoveAllItems()
	for i = 0, skel:CountBones() - 1 do
		local bone = skel:Bone(i)
		if (bone:Name() ~= "") then
			menu:AddItem(bone:Name(), 0, baseMsg + i)
		end
	end
	if (menu:CountItems() == 0) then
		menu:AddItem("<None>", 0, dummyMsg)
		menu:SetEnabled(dummyMsg, false)
	end
end

function MOHO.BuildBoneChoiceMenu(menu, skel, baseMsg, exclude)
	menu:RemoveAllItems()
	menu:AddItem("<None>", 0, baseMsg)
	for i = 0, skel:CountBones() - 1 do
		local bone = skel:Bone(i)
		if (i ~= exclude and bone:Name() ~= "") then
			menu:AddItem(bone:Name(), 0, baseMsg + 1 + i)
		end
	end
end
